//Version mqe_java/source/examples/administration/commandline/HomeServerQueueCreator.java, MQe, la200, la200-L030414.1 1.9

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.administration.commandline;


import com.ibm.mqe.*;  
import com.ibm.mqe.administration.*; 

import examples.queuemanager.*;
                            

/**
 * To start a queue manager and add a home-server queue to its configuration.
 * 
 * <P><B>Syntax</B><P>
 * <P><TT>java examples.administration.commandline.HomeServerQueueCreator queue_name
 * &lt;get_from_qmgr&gt; &lt;compressor&gt; &lt;cryptor&gt; &lt;authenticator&gt; &lt;poll_period&gt; &lt;changing_qmgr&gt; &lt;ini_file&gt;
 * </TT></P>
 * <P><B>Parameters</B><P>
 * <DL>
 * <DT>queue_name</DT>
 * <DD>The name of the home server queue we want to create.
 * This should match the name of the home server queue from which mesages will be "pulled"</DD>
 * 
 * <DT>get_from_qmgr</DT>
 * <DD>The name of the MQe queue manager from which messages will be "pulled".
 * This queue manager will have a store-and-forward queue of the same name as the queue_name
 * parameter above.</DD>
 * 
 * <DT>poll_period</DT>
 * <DD>A time, in milliseconds, indicating the time period between successive attempts
 * for the home server queue to get messages from the remote store-and-foreward queue.
 * A special value of 0 means that polls are controlled by the queue manager rule, and that
 * the home server queue never polls of it's own accord.</DD>
 * 
 * <DT>changing_qmgr</DT>
 * <DD>The name of the queue manager which is going to have the home-server queue created on it.
 * This might normally be the same queue manager as described by the .ini file.</DD>
 * 
 * <DT>ini_file</DT>
 * <DD>The fully-qualified path of the .ini file. The queue manager described by this .ini file
 * will be instantiated in memory, and used to send and receive MQe admin messages.</DD>
 * </DL>
 *
 * <DT>compressor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * compressor the queue should use to store messages on the disk.
 * Use "null" if you don't want a compressor.</DD>
 *
 * <DT>cryptor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * cryptor the queue should use to store messages on the disk.
 * Use "null" if you don't want a cryptor.</DD>
 * <DT>authenticator</DT>
 *
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * authenticator the queue should use to store messages on the disk.
 * Use "null" if you don't want an authenticator.</DD>

 */
public class HomeServerQueueCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool {  

    public static short[] version = { 2, 0, 0, 5 }; 
    public static final String syntax = 
        "Syntax: HomeServerQueueCreator <queue_name> <get_from_qmgr> <poll_period> <changing_qmgr> <ini_file> [ <compressor> <cryptor> <authenticator> ]";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        HomeServerQueueCreator me = new HomeServerQueueCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers the command-line parameters into variables, and calls the method which creates the queue.
     * 
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        String q = null;
        String qmgr = null;
        String iniFile = null;
        String getFromQMgr = null;
        String compressorClass = "null";
        String cryptorClass = "null";
        String authenticatorClass = "null";
        long pollPeriod = 0L;

        if (args.length < 5) {
            throw new Exception("Not enough parameters ! " + syntax);
        } else if ((args.length > 5) & (args.length < 8)) {
            throw new Exception("Not enough parameters ! " + syntax);
        } else if (args.length > 8) {
            throw new Exception("Too many parameters ! " + syntax);
        }

        q = args[0].trim();
        getFromQMgr = args[1].trim();
        String pollPeriodStr = args[2].trim();

        qmgr = args[3].trim();
        iniFile = args[4].trim();

        if (args.length > 5) {
            compressorClass = args[5].trim();
            cryptorClass = args[6].trim();
            authenticatorClass = args[7].trim();
        }

        System.out.println("i: Attempting to define a home server queue " + q + " on queue manager " + qmgr
            + " using the " + iniFile + " ini file, So it can collect information from the "
            + "queue manager " + getFromQMgr + " poll period is " + pollPeriodStr);

        try {
            pollPeriod = Long.parseLong(pollPeriodStr);
        } catch (Exception e) {
            System.out.println("Error: pollPeriod (milliseconds) is not a valid (long) number!");
            throw e;
        }
        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true // Suppress stdout information
            );

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = MQeClient.queueManager;

        // Create the queue...
        createQueue(q, qmgr, getFromQMgr, compressorClass, cryptorClass, authenticatorClass, pollPeriod, myQMgr);

        // Close down the MQeClient
        client.close();
    } 

    /**
     * Creates an admin message which will create a queue, and sends it
     * to the queue manager.
     * 
     * @param q      The name of the home server queue
     * 
     * @param qmgr   The queue manager on which the home server queue will be created
     * 
     * @param getFromQMgr
     *                   The name of the queue manager from which this new home server queue will "pull" messages.
     * 
     * @param pollPeriod The period (milliseconds) used between successive "poll" attempts to "pull" messages from the store-and-forward queue.
     * 
     * @param myQMgr A reference to an active MQe queue manager, used to send and receive messages.
     * @return void
     * @exception Exception
     */
    static public void createQueue(String q, String qmgr, String getFromQMgr, String compressorClass, String cryptorClass, String authenticatorClass, long pollPeriod, MQeQueueManager myQMgr
    ) throws Exception {

        System.out.println("Building the admin message to create a home server queue " + q
            + " on queue manager " + qmgr + " so it can collect messages from the " + getFromQMgr
            + " queue manager.");

        MQeHomeServerQueueAdminMsg msg = new MQeHomeServerQueueAdminMsg(getFromQMgr, q
            );
        MQeFields params = new MQeFields();

        // Fill the params with any other information we need for our queue definition.
        if (!cryptorClass.equals("null")) {
            params.putAscii(MQeQueueAdminMsg.Queue_Cryptor, cryptorClass);
        }
        if (!authenticatorClass.equals("null")) {
            params.putAscii(MQeQueueAdminMsg.Queue_Authenticator, authenticatorClass);
        }
        if (!compressorClass.equals("null")) {
            params.putAscii(MQeQueueAdminMsg.Queue_Compressor, compressorClass);
        }

        // Fill the params with any other information we need for our queue definition.
        params.putLong(MQeHomeServerQueueAdminMsg.Queue_QTimerInterval, pollPeriod);

        // Say this admin message should create the queue...
        msg.create(params);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, qmgr, msg);

        System.out.println("Home server queue created successfully.");
    }  
} 
