package examples.administration.commandline;


// Version mqe_java/source/examples/administration/commandline/ListenerCreate.java, MQe, la200, la200-L030324.1 1.6
/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;


/**
 * A command-line tool to allow someone to add a Listener to an MQe queue manager without programming
 * anything in Java.  This version uses the admin messages to send the command to the destination
 * queue manager, and so can be used to add a listener to a remote queue manager. This version
 * is design to setup listeners that use tcpip based protocols.
 *
 * <b><p>Syntax</b></p>
 * <P><TT>java examples.administration.commandline.ListenerCreate
 * &lt;listenerName&gt; &lt;qMgr&gt; &lt;listenAdapter&gt; &lt;listenPort&gt; &lt;channelTimeout&gt; &lt;maxChannels&gt; &lt;ini_file&gt;
 * </TT></P>
 *
 * <dl>
 *
 * <dt>listenerName</dt>
 * <dd>The name of listener which is to be created.</dd>
 *
 * <dt>qMgr</dt>
 * <dd>The MQe queue manager to which the listener will be added.
 * Normally this would be the same as the queue manager that this example runs.</dd>
 *
 * <dt>listenAdapter</dt>
 * <dd>The communications adapter that will do the listening. This can be the name of the
 * adapter class like com.ibm.mqe.adapters.MQeTcpipHttpAdapter or an alias like Network</dd>
 *
 * <dt>listenPort</dt>
 * <dd>The port that adapter will use to listen for new incoming requests, for example
 * 8082</dd>
 *
 * <dt>channelTimeout</dt>
 * <DD>The time in milliseconds that idle incoming channels live before being timed out.</DD>
 *
 * <dt>maxChannels</dt>
 * <DD>The maximum number of channels the listener allows to be open at any one time.</DD>
 *
 * <DT>iniFile</DT>
 * <DD>The fully qualified path of the .ini file used by this tool to instantiate
 * a device queue manager, send the admin message, and wait for the reply. Works with
 * MQe 1.x style .ini files.
 * </DD>
 *
 * </dl>
 *
 * <B><P>Examples</B></P>
 * <P>This tool would normally be used to create a listener on the local queue manager
 * for example:
 * ...ListenerCreate ListenerName MyDeviceQM Network 8082 300000 100 C:\MQe\...\MyDeviceQM.ini
 * </P>
 *
 */
public class ListenerCreate extends MQeAbstractCmdLineAdminTool {

    public static short[] version = { 2, 0, 0, 5 }; 

    /**
     * Entry point from the command line.
     *
     * @param args   Command-line arguments being passed in.
     */
    public static void main(String args[]) {
        int rc = 0;
        ListenerCreate me = new ListenerCreate();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables, and calls the method which creates the queue.
     *
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        if (args.length < 7) {
            throw new Exception(
                    "Error: Not enough parameters. "
                    + "Syntax :ListenerCreate <listenerName> <qMgr> <listenAdapter> <listenPort> <channelTimeout> <maxChannels> <ini_file>");
        }
        String  listenerName = args[0].trim();
        String  qmgr = args[1].trim();
        String  listenAdapter = args[2].trim();
        int     listenPort = Integer.parseInt(args[3].trim());
        int     channelTimeout = Integer.parseInt(args[4].trim());
        int     maxChannels = Integer.parseInt(args[5].trim());
        String iniFile = args[6].trim();

        System.out.println(
            "i: Attempting to define listener "
            + listenerName
            + " listening with "
            + listenAdapter
            + " on port "
            + listenPort
            + " on local qmgr "
            + qmgr
            + " using ini file "
            + iniFile
            + " (Timeout = "
            + channelTimeout
            + "; maxChannels = "
            + maxChannels
            + ")");
        // Access the local MQe queue manager using the MQeClient example
        // class.
        MQeClient client = new MQeClient(iniFile, true/* Suppress stdout*/);

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = client.queueManager;

        // Create the listener...
        createListener(listenerName, listenAdapter, listenPort,
            channelTimeout, maxChannels, qmgr, myQMgr);

        // Close down the MQeClient
        client.close();
    }

    /**
     * Creates an admin message which will cause a listener to be created
     * then it sends it to a queue manager.
     * @param listenerName (java.lang.String) The name of listener which is to be created.
     *
     * @param listenAdapter (String)  The communications adapter used to listen for
     *                          new incoming requests. This may be an alias or actual
     *                          class name of the adapter.
     *
     * @param listenPort (int)  The port to listen on
     *
     * @param channelTimeout (long) Timeout period expressed in milliseconds. This is
     * used to control how long incoming channels can remain idle before being
     * timed out
     *
     * @param maxChannels(int) The maximum number of channels the listener allows
     * to be open at any one time. (A value of 0 means there is no limit)
     *
     * @param targetQMgr The name of the queue manager to which the admin message will be sent.
     *
     * @param myQMgr The reference to an active MQe queue manager object, which will be
     *               used to send the admin message, and receive the reply.
     *
     * @exception Exception
     */
    public static void createListener(String listenerName, String listenAdapter,
        int listenPort, long channelTimeout, int maxChannels,
        String targetQMgr, MQeQueueManager myQMgr) throws Exception {

        System.out.println("Build admin message to create a listener");
        MQeCommunicationsListenerAdminMsg msg = new MQeCommunicationsListenerAdminMsg();

        msg.setName(listenerName);
        msg.create(listenAdapter, listenPort, channelTimeout, maxChannels);

        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);

        System.out.println("Listener created successfully.");
       
    }

}
