package examples.administration.commandline;


// Version mqe_java/source/examples/administration/commandline/ListenerSetMaxChannels.java, MQe, la200, la200-L030324.1 1.6
/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;


/**
 * A command-line tool to allow someone to set the max channels attribute of a listener
 * without programming anything in Java.  This version uses admin messages to
 * send the command to the destination queue manager, so can be used to update
 * the max channels attribute for a listener on a remote queue manager.
 *
 * <b><p>Syntax</b></p>
 * <P><TT>java examples.administration.commandline.ListenerSetMaxChannels
 * &lt;listenerName&gt; &lt;qMgr&gt; &lt;maxChannels&gt; &lt;ini_file&gt;
 * </TT></P>
 *
 * <dl>
 *
 * <dt>listenerName</dt>
 * <dd>The name of listener which is to be SetMaxChannels.</dd>
 *
 * <dt>qMgr</dt>
 * <dd>The MQe queue manager on which the listener will be modified.
 * Normally this would be the same as the queue manager that this example runs.</dd>
 *
 * <dt>maxChannels</dt>
 * <DD>The maximum number of channels the listener can accept at any one time./dd
 *
 * <DT>iniFile</DT>
 * <DD>The fully qualified path of the .ini file used by this tool to instantiate
 * a device queue manager, send the admin message, and wait for the reply. Works with
 * MQe 1.x style .ini files.
 * </DD>
 *
 * </dl>
 *
 * <B><P>Examples</B></P>
 * <P>This tool would normally be used to create a straight listener on the local queue manager
 * for example:
 * ...ListenerSetMaxChannelsl ListenerName DeviceQM maxChannels C:\MQe\...\MyDeviceQM.ini
 * </P>
 *
 */
public class ListenerSetMaxChannels extends MQeAbstractCmdLineAdminTool {

    public static short[] version = { 2, 0, 0, 5 }; 

    /**
     * Entry point from the command line.
     *
     * @param args   Command-line arguments being passed in.
     */
    public static void main(String args[]) {
        int rc = 0;
        ListenerSetMaxChannels me = new ListenerSetMaxChannels();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables, and calls the method which creates the queue.
     *
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        // Get the parameters from the system properties...
        if (args.length < 4) {
            throw new Exception(
                    "Error: Not enough parameters. "
                    + "Syntax :ListenerSetMaxChannels <listenerName> <qMgr> <maxChannels> <ini_file>");
        }

        String listenerName = args[0].trim();
        String qmgr = args[1].trim();
        int maxChannels = Integer.parseInt(args[2].trim());
        String iniFile = args[3].trim();

        System.out.println(
            "i: Attempting to set max channels for listener "
            + listenerName
            + " on qmgr "
            + qmgr
            + " using ini file "
            + iniFile
            + " maxChannels = "
            + maxChannels);
        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true/* Suppress stdout*/);

        // Access the queue manager SetMaxChannelsd by the MQeClient class...
        MQeQueueManager myQMgr = client.queueManager;

        // SetMaxChannels the listener...
        setMaxChannels(listenerName, maxChannels, qmgr, myQMgr);

        // Close down the MQeClient
        client.close();
    }

    /**
     * Creates an admin message which will cause a listener to update
     * its maximum allowed channels.
     *
     * @param listenerName (java.lang.String) The name of listener.
     *
     * @param maxChannels (int) the maximum allowed number of channels.
     * @param targetQMgr The name of the queue manager to which the admin message will be sent.
     *
     * @param myQMgr The reference to an active MQe queue manager object, which will be
     *               used to send the admin message, and receive the reply.
     *
     * @exception Exception
     */
    public static final void setMaxChannels(String listenerName, int maxChannels, String targetQMgr, MQeQueueManager myQMgr) throws Exception {
        
        System.out.println("Build an admin message to set max channels on a listener");
        MQeCommunicationsListenerAdminMsg msg = new MQeCommunicationsListenerAdminMsg();

        msg.setName(listenerName);
        MQeFields parms = new MQeFields();

        parms.putInt(MQeCommunicationsListenerAdminMsg.LISTENER_MAXIMUM_CHANNELS, maxChannels);
        msg.update(parms);

        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);

        System.out.println("Set max channels was successful.");
        
    }
}
