//Version mqe_java/source/examples/administration/commandline/LocalQueueCreator.java, MQe, la200, la200-L030414.1 1.9

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.administration.commandline;


import com.ibm.mqe.*; 
import com.ibm.mqe.administration.*;

import examples.queuemanager.*;

                            
/**
 * To add a local queue to a queue manager from the command-line.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.administration.commandline.LocalQueueCreator
 * &lt;queue_name&gt;
 * &lt;compressor&gt;
 * &lt;cryptor&gt;
 * &lt;authenticator&gt;
 * &lt;maxMsgSize&gt;
 * &lt;maxQDepth&gt;
 * &lt;queue_manager_name&gt;
 * &lt;ini_file&gt;
 * &lt;storageLocation&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * <DT>queue_name</DT>
 * <DD>The name of the local queue to create on the target queue manager.</DD>
 * 
 * <DT>compressor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * compressor the queue should use to store messages on the disk.
 * Use "null" if you don't want a compressor.</DD>
 * 
 * <DT>cryptor</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * cryptor the queue should use to store messages on the disk.
 * Use "null" if you don't want a cryptor.</DD>
 * 
 * <DT>authenticator</DT>
 * <DD>The name of the class (or alias which resolves into a class) indicating which
 * authenticator the queue should use to store messages on the disk.
 * Use "null" if you don't want an authenticator.</DD>
 * 
 * <DT>maxMsgSize</DT>
 * <DD>The size of the largest message (in bytes) which can be accepted on this queue.
 * This must be an integer number.
 * Specify "nolimit" if you want to allow any size of message to be put to the queue.
 * Specify "default" if you want to set this to the default value chosen by MQe.
 * </DD>
 * 
 * <DT>maxQDepth</DT>
 * <DD>The max number of messages allowed to be stored in the queue, before the queue is considered to be "full".
 * This must be an integer number.
 * Specify "nolimit" if you want to allow any limit to the depth of the queue
 * Specify "default" if you want to set this to the default value chosen by MQe.
 * </DD>
 * 
 * <DT>queue_manager_name</DT>
 * <DD>The name of the queue manager on which the new local queue will be defined.
 * This would normally be the name of the queue manager described by the .ini file</DD>
 * 
 * <DT>ini_file</DT>
 * <DD>The fully-qualified path name of the .ini file to be used to send and receive the admin messages.</DD>
 * 
 * <DT>storageLocation</DT>
 * <DD>A string, following the syntax <TT>&lt;adapter&gt:&lt;path&gt;</TT> 
 * Where the adapter is a class name without the .class extension,
 * and the path is the name of the directory in which the files containing 
 * queue messages are stored.
 * eg: com.ibm.mqe.adapers.MQeDiskFieldsAdapter:C:\ServerQMgr23\RemoteQ48
 * Specify "default" if you want to set this to the default value chosen by MQe.</DD>
 * </DL>
 */
public class LocalQueueCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool {  

    public static short[] version = { 2, 0, 0, 5 }; 
    public static final String syntax =
        "Syntax: ...LocalQueueCreator <queue_name> <compressor> <cryptor> <authenticator>\n"
        + "<maxMsgSize> <maxQDepth> <queue_manager_name> <ini_file> <storageLocation>";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        LocalQueueCreator me = new LocalQueueCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables, and calls the method which creates the queue.
     * 
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {
        String q = null;
        String compressorClass = null;
        String cryptorClass = null;
        String authenticatorClass = null;
        String maxMsgSize = null;
        String maxQDepth = null;
        String qmgr = null;
        String iniFile = null;
        String storageLocation = null;

        if (args.length < 9) {
            throw new Exception("Not enough parameters !\n" + syntax);
        }

        q = args[0].trim();
        compressorClass = args[1].trim();
        cryptorClass = args[2].trim();
        authenticatorClass = args[3].trim();
        maxMsgSize = args[4].trim();
        maxQDepth = args[5].trim();
        qmgr = args[6].trim();
        iniFile = args[7].trim();
        storageLocation = args[8].trim();

        System.out.println("i: Attempting to define a queue " + q + " on queue manager " + qmgr + " using the " + iniFile + " ini file.");

        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true // suppress any stdout information.
            );

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = MQeClient.queueManager;

        // Create the queue...
        createQueue(q, compressorClass, cryptorClass, authenticatorClass, maxMsgSize, maxQDepth, qmgr, storageLocation, myQMgr 
        );

        // Close down the MQeClient
        client.close();
    } 

    /**
     * Creates an admin message which will create a queue, and sends it
     * to the target queue manager.
     * 
     * @param q      The name of the local queue to create
     * 
     * @param compressorClass
     *                   The name of the compression class to use (or alias which resolves to a class name)
     *                   Should include the .class extension.
     *                   Specify "null" if you don't want to use a compressor.
     * 
     * @param cryptorClass
     *                   The name of the cryptor class to use (or alias which resolves to a class name)
     *                   Should include the .class extension.
     *                   Specify "null" if you don't want to use a cryptor.
     * 
     * @param authenticatorClass
     *                   The name of the authenticator class to use (or alias which resolves to a class name)
     *                   Should include the .class extension.
     *                   Specify "null" if you don't want to use an authenticator
     * 
     * @param maxMsgSize The size of the largest message (in bytes) which can be accepted on this queue.
     *                   This must be an integer number.
     *                   Specify "nolimit" if you want to allow any size of message to be put to the queue.
     *                   Specify "default" if you want to set this to the default value chosen by MQe.
     * 
     * @param maxQDepth The max number of messages allowed to be stored in the queue, before the queue is considered to be "full".
     *                  This must be an integer number.
     *                  Specify "nolimit" if you want to allow any limit to the depth of the queue
     *                  Specify "default" if you want to set this to the default value chosen by MQe.
     * 
     * @param qmgr   The queue manager on which the local queue is created.
     *
     * @param storageLocation
     *               A string, following the syntax <TT>&lt;adapter&gt:&lt;path&gt;</TT> 
     *               Where the adapter is a class name without the .class extension,
     *               and the path is the name of the directory in which the files containing 
     *               queue messages are stored.
     *               eg: com.ibm.mqe.adapers.MQeDiskFieldsAdapter:C:\ServerQMgr23\RemoteQ48
     *               Specify "default" if you want to set this to the default value chosen by MQe
     * 
     * @param myQMgr A reference to an active MQe queue manager, which will be used to send and receive admin messages.
     * @return void
     * @exception Exception
     */
    public static void createQueue(String q, String compressorClass, String cryptorClass, String authenticatorClass, String maxMsgSize, String maxQDepth, String qmgr, String storageLocation, MQeQueueManager myQMgr
    ) throws Exception {

        System.out.println("Building the admin message to create a local queue " + q
            + " on queue manager " + qmgr);
     
        MQeQueueAdminMsg msg = new MQeQueueAdminMsg(qmgr, q
            );
        MQeFields params = new MQeFields();
     
        // Fill the params with any other information we need for our queue definition.
        if (!cryptorClass.equals("null")) {
            params.putAscii(MQeQueueAdminMsg.Queue_Cryptor, cryptorClass);
        }
        if (!authenticatorClass.equals("null")) {
            params.putAscii(MQeQueueAdminMsg.Queue_Authenticator, authenticatorClass);
        }
        if (!compressorClass.equals("null")) {
            params.putAscii(MQeQueueAdminMsg.Queue_Compressor, compressorClass);
        }

        if (maxMsgSize.equals("default")) {// If we don't set the field to anything explicitly, it gets implicitly set
            // to a sensible default.
        } else {
            int value;

            if (maxMsgSize.equals("nolimit")) {
                value = MQeQueueAdminMsg.Queue_NoLimit;
            } else {
                value = Integer.parseInt(maxMsgSize);
            }
            params.putInt(MQeQueueAdminMsg.Queue_MaxMsgSize, value);
        }

        if (maxQDepth.equals("default")) {// If we don't set the field to anything explicitly, it gets implicitly set
            // to a sensible default.
        } else {
            int value;

            if (maxQDepth.equals("nolimit")) {
                value = MQeQueueAdminMsg.Queue_NoLimit;
            } else {
                value = Integer.parseInt(maxQDepth);
            }
            params.putInt(MQeQueueAdminMsg.Queue_MaxQSize, value);
        }

        if (storageLocation.equals("default")) {// If we don't set the field to anything explicitly, it gets implicitly set
            // to a sensible default.
        } else {       
            params.putAscii(MQeQueueAdminMsg.Queue_FileDesc, storageLocation);
        }

        // Say this admin message should create the queue...
        msg.create(params);
     
        // Direct the admin message at the correct object...
        msg.setName(qmgr, q);
     
        System.out.println("Built the admin message.");
     
        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, qmgr, msg);
        
        System.out.println("Local queue created successfully.");        
    } 
}

