//Version mqe_java/source/examples/administration/commandline/QueueDelete.java, MQeExamples, la200, la200-L030414.1 1.1

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/*------------------------------------------------------------------------------*/
package examples.administration.commandline;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;


/**
 * A command-line tool to allow someone to delete a queue
 * without programming anything in Java.  This version uses the admin
 * messages to send the request to either a local or a remote queue manager. 
 * It can be called from the command line or from a Java program. The tool 
 * can be used to delete all types of queue. 
 *
 * <b><p>Syntax</b></p>
 * <P><TT>java examples.administration.commandline.QueueDelete
 * &lt;queueName&gt; 
 * &lt;destinationQMgrOfQueue&gt; 
 * &lt;qMgr&gt; 
 * &lt;ini_file&gt;
 * </TT></P>
 *
 * <dl>
 * <DT>queue_name</DT>
 * <DD>the name of the queue to delete.</DD>
 * 
 * <DT>destinationQMgrOfQueue</DT>
 * <DD>is the name of the queue manager that hosts the queue. For local queues
 * the local queue definition is hosted on the same queue manager as the local queue. 
 * For remote queues, the remote queue definition is normally hosted on a different 
 * queue manager to the destination queue.</DD>
 *
 * <dt>qMgr</dt>
 * <DD>the name of the queue manager where the delete request will be sent. This
 * is the queue manager where the delete action will be performed.</DD>
 *
 * <DT>iniFile</DT>
 * <DD>The fully qualified path of the .ini file used by this tool to instantiate
 * a queue manager, send the admin message, and wait for the reply. Works with
 * MQe 1.x style .ini files.
 * </DD>

 * </dl>
 *
 * <B><P>Examples</B></P>
 * <P>Assume two queue manangers: 1) ClientQM 2) ServerQM. ServerQM has a local
 * queue called OrderQ and ClientQM has a remote queue called OrderQ on ServerQM. If 
 * the delete requests are issued from ClientQM and ClientQM startup parameters
 * are provided in ini file ClientQM.ini, the following invocations could be 
 * made:
 * <dl>
 * <dt>To delete the remote queue definition on ClientQM from ClientQM issue</dt>
 * <dd>QueueDelete OrderQ ServerQM ClientQM C:\MQe\...\ClientQM.ini</dd>
 * <dt>To delete the local queue definition on ServerQM from ClientQM issue</dt>
 * <dd>QueueDelete OrderQ ServerQM ServerQM C:\MQe\...\ClientQM.ini</dd>
 * </dl> 
 * </P>
 *
 */
public class QueueDelete extends MQeAbstractCmdLineAdminTool {

    public static short[] version = { 2, 0, 0, 5 }; 

    /**
     * Entry point from the command line.
     *
     * @param args   Command-line arguments being passed in.
     */
    public static void main(String args[]) {
        int rc = 0;
        QueueDelete me = new QueueDelete();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables, and calls 
     * the method which inquires on the queue.
     *
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        // Get the parameters from the system properties...
        if (args.length < 4) {
            throw new Exception(
                    "Error: Not enough parameters. "
                    + "Syntax :QueueDelete <queueName> <destinatinQmgrOfQueue> <QMgr> <ini_file>");
        }
        String queueName = args[0].trim();
        String queueQMgrName = args[1].trim();
        String qmgr = args[2].trim();
        String iniFile = args[3].trim();

        System.out.println(
            "i: Attempting to delete queue "
            + queueName
            + " destination of "
            + queueQMgrName
            + " on qmgr "
            + qmgr
            + " using the "
            + iniFile
            + " ini file");

        // Access the local MQe queue manager...
        MQeClient client = new MQeClient(iniFile, true/* Suppress stdout*/);

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = MQeClient.queueManager;

        // delete the queue
        MQeFields data = deleteQueue(queueName, queueQMgrName, qmgr, myQMgr);

        // Close down the MQeClient
        client.close();
    }

    /**
     * Creates an admin message which will delete a queue and 
     * then it sends it to a queue manager.
     *
     * @param queueName The name of the queue
     * 
     * @param queueQMgrName  The name of the queue manager that hosts the queue. 
     * For local queues the local queue definition is hosted on the same queue 
     * manager as the local queue. For remote queues, the remote queue definition 
     * is normally hosted on a different queue manager to the destination queue. 
     *
     * @param targetQMgr The name of the queue manager to which the admin 
     * message will be sent.
     *
     * @param myQMgr The reference to an active MQe queue manager object, 
     * which will be used to send the admin message, and receive the reply.
     * 
     * @return MQeFields contaning result
     * 
     * @exception Exception
     */
    public static MQeFields deleteQueue(String queueName, String queueQMgrName, 
        String targetQMgr, MQeQueueManager myQMgr) throws Exception {

        System.out.println("Build an admin message to delete a queue");
        // A queue admin message can be used to delete any type of queue
        MQeQueueAdminMsg msg = new MQeQueueAdminMsg();

        msg.setName(queueQMgrName, queueName);
        msg.delete((MQeFields) null);

        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);

        System.out.println("Queue deleted successfully.");
       
        return results;
    }

}
