// Version mqe_java/source/examples/application/Example6.java, MQeExamples, la200, la200-L030324.1 1.10

/* -----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 1999                                                 *
 * -----------------------------------------------------------------------------*/

package examples.application;


import com.ibm.mqe.*;
import examples.queuemanager.*;


/***
 * <p>Extends ExampleBase, Application example base.
 *
 * <p>Synchronous assured message delivery example.
 *
 * <p>Example showing how to:
 * <pre>
 *  . start a queue manager
 *  . put a message using assured message delivery
 *  . retrieve that message using assured message delivery
 * </pre>
 * <p>Notes:  <ul>
 * <li>This example shows the use of assured message delivery with
 *        queues that are defined as synchronous. Asynchronous queues
 *        provide assured message delivery automatically.
 * <li>To be able to provide assured message delivery, a queue must
 *        be defined with a persistent queue store (such as disk). This
 *        allows the message to be recovered after a system restart.
 * <li>When an applicaiton uses assured delivery, the application should
 *        ensure that it can cope with a system failure during the delivery
 *        operation. This may mean the application storing delivery state
 *        information which can be used across restarts. This example does
 *        not do this but sets a message expiry so that any half delivered
 *        messages are removed from the queue after the expiry time.
 * </ul>
 */

public class Example6 extends ExampleBase {
    public static short[] version = { 2, 0, 0, 5 }; 

    /**
     * max number of retry attempts
     */
    int maxRetry = 5;

    /**
     * Constructor that uses the default queue manager.
     * @exception java.lang.Exception
     */
    public Example6() throws Exception {
        super();
    }

    /**
     * Constructor that uses the queue manager passed in on the command line.
     * @param queueManagerName java.lang.String
     * @exception java.lang.Exception
     */
    public Example6(String queueManagerName) throws Exception {
        super(queueManagerName);
    }

    /**
     * Constructor that starts a queue manager with paramters
     * from an ini file. This queue manager is then used as the
     * access point for interacting with the same or different
     * queue managers.
     * @param qMgrName java.lang.String, queue manager name,
     *        the name of the queue manager to interact with.
     *        The queue manager can be a local or a remote queue
     *        manager.
     * @param localQMgrIniFile java.lang.String, ini file which
     *        contains startup parameters for the local queue
     *        manager.
     * @exception java.lang.Exception
     */
    public Example6(String qMgrName, String localQMgrIniFile) throws Exception {
        super(qMgrName, localQMgrIniFile);
    }

    /**
     * All work is performed in this method.
     * @return void
     * @exception java.lang.Exception
     */
    public void doIt() throws Exception {
        MQeMsgObject returnedMsg = null;

        try {

            // create a message to be put
            System.out.println("..Creating a message");
            MQeMsgObject msgObj = new MQeMsgObject();

            msgObj.putAscii(msgField, msgData);

            // expiry time of 1 day - (60*60*24*1000 milliseconds)
            msgObj.putInt(MQe.Msg_ExpireTime, 86400000);

            // put a message and confirm it
            boolean result = putMessage(msgObj);

            // if put operation was successful, then attempt get operation
            if (result) {
                returnedMsg = getMessage(msgObj.getMsgUIDFields());
            }
            if (returnedMsg == null) {
                System.out.println(example + " Finished");
            } else {
                System.out.println(example + " Failed");
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println(example + " Failed! " + e);
            throw e;
        } finally {
            close();
        }
    }

    /**
     * Get a message and confirm it.
     * @param filter An MQeFields Object
     * @return An MQeMsgObject or null.
     * @exception java.lang.Exception
     */
    public MQeMsgObject getMessage(MQeFields filter) throws Exception {

        boolean msgGet = false;               // get message successful?
        boolean getConfirmed = false;         // get confirm successful?
        MQeMsgObject msg = null;
        long confirmId = uniqueValue();        // generate a confirm Id

        // get message unique Id fields from supplied filter
        String originQMgr = filter.getAscii(MQe.Msg_OriginQMgr);
        long timeStamp = filter.getLong(MQe.Msg_Time);

        // try to get the message that has just been put - retry if there are
        // problems until maximum retry count is exceeded
        int retry = 0;                              // initialise retry count

        while (!msgGet && retry < maxRetry) {
            try {
                System.out.println("..Attempting to get message " + originQMgr +
                    ":" + timeStamp + " from queue: " + qMgrName +
                    "/" + queueName);
                msg = myQM.getMessage(qMgrName, queueName, filter, null, confirmId);
                msgGet = true;                      // get successful
                System.out.println("..Get successful");
            } catch (Exception e) {

                // if the exception is of type Except_Q_NoMatchingMsg, meaning that
                // the message is unavailable then throw the exception
                if (e instanceof MQeException)
                    if (((MQeException) e).code() == Except_Q_NoMatchingMsg)
                        throw e;
                System.out.println("..Exception on get message: " +
                    e.getMessage());
                retry++;                            // increment retry count

                // As a precaution, undo the message on the queue. This will remove
                // any lock that may have been put on the message prior to the
                // exception occurring
                System.out.println("..Undo-ing any locks on message " + originQMgr +
                    ":" + timeStamp + " on queue: " + qMgrName +
                    "/" + queueName);
                myQM.undo(qMgrName, queueName, confirmId);
            }
        }
        // Check that the get was successful...
        if (!msgGet) {

            // Number of retry attempts has exceeded the maximum allowed, so abort
            // get message operation
            System.out.println("..Aborting get message " + originQMgr +
                ":" + timeStamp + " from queue: " + qMgrName +
                "/" + queueName);
            return null;  // get message unsuccessful
        }

        // try to confirm the previous get message operation - retry if there are
        // problems until maximum retry count is exceeded
        retry = 0;
        while (!getConfirmed && retry < maxRetry) {
            try {
                System.out.println("..Attempting to confirm get message " + originQMgr +
                    ":" + timeStamp + " from queue: " + qMgrName +
                    "/" + queueName);
                myQM.confirmGetMessage(qMgrName, queueName, filter);
                getConfirmed = true;
                System.out.println("..Confirm get message successful");
            } catch (Exception e) {

                // exception from confirm get operation
                System.out.println("..Exception on confirm get message: " +
                    e.getMessage());
                retry++;
            }
        }
        if (!getConfirmed) /* was the confirm successful?  */{
            System.out.println("..Aborting confirm get message " + originQMgr +
                ":" + timeStamp + " from queue: " + qMgrName +
                "/" + queueName);
            return null;
        }
        return msg;
    }

    /**
     * Starts the application.
     * @param param1  Queue Manager Name to interact with.
     *                      If queue manager name matches the local QM name then
     *                      all interactions are local otherwise the local QM acts
     *                      as a stub QM sending all messages to a remote QM.
     * @param param2  Local queue manager startup parameters ini file.
     * @return void
     * @exception java.lang.Exception
     */
    public static void main(String args[]) throws Exception {
        short rc = 0;

        try {
            if (args.length == 0)
                new Example6().doIt();
            else if (args.length == 1)
                new Example6(args[0]).doIt();
            else
                new Example6(args[0], args[1]).doIt();
        } catch (Exception ex) {
            System.out.println("Example failed: " + ex);
            ex.printStackTrace();
            rc = 99;
        }
        System.exit(rc);
    }

    /**
     * Put a message and confirm it.
     * @param msgObj An MQeMsgObject
     * @return Boolean True if put operation is successful. False, otherwise.
     * @exception java.lang.Exception
     */
    public boolean putMessage(MQeMsgObject msgObj) throws Exception {
        boolean msgPut = false;
        boolean putConfirmed = false;
        long confirmId = uniqueValue();

        // get the message's unique ID fields
        String originQMgr = myQM.getName();
        long timeStamp = msgObj.getLong(Msg_Time);

        // try to put message - retry if there are problems until maximum retry
        // count is exceeded
        int retry = 0;

        while (!msgPut && retry < maxRetry) {
            try {
                System.out.println("..Attempting to put message " + originQMgr +
                    ":" + timeStamp + " to queue: " + qMgrName +
                    "/" + queueName);
                myQM.putMessage(qMgrName, queueName, msgObj, null, confirmId);
                msgPut = true;
                System.out.println("..Put successful");
            } catch (Exception e) {

                // exception on put message operation
                System.out.println("..Exception on put message: " + e.getMessage());
                System.out.println("..Setting resend flag");

                // set the message resend flag
                msgObj.putBoolean(Msg_Resend, true);
                retry++;
            }
        }
        if (!msgPut) {
            // Number of retries has exceeded the maximum allowed, so abort the put
            // message attempt
            System.out.println("..Aborting put message " + originQMgr +
                ":" + timeStamp + " to queue: " + qMgrName +
                "/" + queueName);

            // Attempt to delete any copy of the message held on the target queue
            // This operation may well fail
            // if the message does exist on the target queue then it will remain
            // there until it expires (if an expiry value has been set on either
            // the message or queue.
            try {
                myQM.undo(qMgrName, queueName, confirmId);
                System.out.println("..Successful undo of message " + originQMgr +
                    ":" + timeStamp + " on queue: " + qMgrName +
                    "/" + queueName);
            } catch (Exception e) {
                System.out.println("..Attempt to undo message " + originQMgr +
                    ":" + timeStamp + " on queue: " + qMgrName +
                    "/" + queueName + "failed");
            }
            return false;
        }

        // get message Unique ID for use on confirm put operation
        MQeFields filter = msgObj.getMsgUIDFields();

        // try to confirm the previous put - retry if there are problems until
        // maximum retry count is exceeded
        retry = 0;
        while (!putConfirmed && retry < maxRetry) {
            try {
                System.out.println("..Attempting to confirm put message " + originQMgr +
                    ":" + timeStamp + " to queue: " + qMgrName +
                    "/" + queueName);
                myQM.confirmPutMessage(qMgrName, queueName, filter);
                putConfirmed = true;
                System.out.println("..Confirm put message successful");
            } catch (Exception e) {
                // An Except_NotFound exception means that the message has already
                // been confirmed
                if (e instanceof MQeException &&
                    ((MQeException) e).code() == Except_NotFound)
                    putConfirmed = true;

                // another type of exception - need to reconfirm message
                else {
                    System.out.println("..Exception on confirm put message: " +
                        e.getMessage());
                    retry++;
                }
            }
        }
        if (!putConfirmed) {
            System.out.println("..Aborting confirm put message " + originQMgr +
                ":" + timeStamp + " to queue: " + qMgrName +
                "/" + queueName);

            // Attempt to undo any copy of the message held on the target queue
            // This operation may well fail
            try {
                myQM.undo(qMgrName, queueName, confirmId);
                System.out.println("..Successful undo of message " + originQMgr +
                    ":" + timeStamp + " on queue: " + qMgrName +
                    "/" + queueName);
            } catch (Exception e) {
                System.out.println("..Attempt to undo message " + originQMgr +
                    ":" + timeStamp + " on queue: " + qMgrName +
                    "/" + queueName + "failed");
            }
            return false;
        }
        return true;
    }

    /**
     * Over-ride default parameter settings.
     */
    public void setParms() {
        example = "Example6";
    }
}

