// Version mqe_java/source/examples/assured/AtomicState.java, MQeExamples, la200, la200-L030324.1 1.6

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001,2002                                            */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.assured;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;


/**
 * Reads and writes the state of the program to persistent storage
 * in an atomic manner. Used by the SendTestMessagesAssured program.
 *
 * @see examples.assured.SendTestMessagesAssured
 */
public class AtomicState {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    /*
     * This program uses a finite-state machine...
     * STATE_START indicates the message can be built and sent. There is no debris lying around.
     * Other states mean what they say.
     */
    public final static int STATE_START = 0;
    public final static int STATE_BUILT_MSG_OK_MAY_HAVE_PUT = 1;
    public final static int STATE_PUT_MSG_OK_MAY_HAVE_CONFIRMED = 2;

    /*
     * Labels used to store away the atomic state in an MQe fields structure.
     */
    private static final String FIELD_NAME_CONFIRM_ID = "ConfirmID";
    private static final String FIELD_NAME_UNIQUE_ID = "UniqueID";
    private static final String FIELD_NAME_MSG_INDEX = "MsgIndex";
    private static final String FIELD_NAME_STATE = "State";

    /*
     * Private variables...
     */
    private long confirmID = 0;
    private MQeFields uniqueID = null;
    private int msgIndex = 0;
    private MQeAdapter adapter = null;
    private int state = STATE_START;

    /**
     * Create the atomic state, reading any data we can from the disk file specified.
     * @param stateFile A String containing the path of the file.
     * @exception java.lang.Exception
     */
    public AtomicState(String stateFile) throws Exception {
        adapter = new MQeDiskFieldsAdapter();
        adapter.activate(".", null, MQeAdapter.MQe_Adapter_UPDATE, 0, 0); // Set up the adapter.
        adapter.open(stateFile);  // Open the state file.
        try {
            MQeFields fields = (MQeFields) adapter.readObject(null); // Read in the file.

            this.confirmID = fields.getLong(FIELD_NAME_CONFIRM_ID);
            this.uniqueID = fields.getFields(FIELD_NAME_UNIQUE_ID);
            this.msgIndex = fields.getInt(FIELD_NAME_MSG_INDEX);
            this.state = fields.getInt(FIELD_NAME_STATE);
        } catch (java.io.FileNotFoundException e) {// The state file didn't exist.
            // The program has never been called with this state file name before.
            // Leave everything with sensible defaults.
        }
        System.out.println("        > initial state set to " + getStateString());
    }

    /***
     * <p> Gets the Message Index Value.
     * @param none
     * @return an Integer value
     */
    public int getMsgIndex() {
        return this.msgIndex;
    }

    /***
     * <p> Gets the ConfirmID Value.
     * @param none
     * @return A long integer value
     */
    public long getConfirmID() {
        return this.confirmID;
    }

    /***
     * <p> Gets the Unique ID.
     * @param none
     * @return an MQeFields Object
     */
    public MQeFields getUniqueID() {
        return this.uniqueID;
    }

    /***
     * <p> Gets the state of the object.
     * @param none
     * @return an Integer value
     */
    public int getState() {
        return this.state;
    }

    /**
     * Write the atomic state to disk using an MQe adapter.
     * @param uniqueID The UniqueID of the Object.
     * @param state  Integer containing the state of the Object.
     * @param confirmID A long value containing the ConfirmID
     * @param msgIndex Integer containing the Index of the message.
     * @return void
     * @exception java.lang.Exception
     */
    public void set(MQeFields uniqueID, int state, long confirmID, int msgIndex
    ) throws Exception {

        System.out.println("        >Pausing to allow time for a user interrupt...");
        java.lang.Thread.sleep(3000);
        System.out.println("        <Pausing over.");

        this.uniqueID = uniqueID;
        this.state = state;
        this.confirmID = confirmID;
        this.msgIndex = msgIndex;
        MQeFields fields = new MQeFields();

        fields.putLong(FIELD_NAME_CONFIRM_ID, confirmID);
        fields.putFields(FIELD_NAME_UNIQUE_ID, uniqueID);
        fields.putInt(FIELD_NAME_MSG_INDEX, msgIndex);
        fields.putInt(FIELD_NAME_STATE, state);
        adapter.writeObject(null, fields);

        System.out.println("        > state set to " + getStateString());
    }

    /**
     * Returns the state as a string for display.
     * @param none
     * @return String containing the State of the Object.
     * @exception none
     */
    public String getStateString() {
        String stateName = null;

        switch (this.state) {
        case STATE_START:
            stateName = "STATE_START";
            break;

        case STATE_BUILT_MSG_OK_MAY_HAVE_PUT:
            stateName = "STATE_BUILT_MSG_OK_MAY_HAVE_PUT";
            break;

        case STATE_PUT_MSG_OK_MAY_HAVE_CONFIRMED:
            stateName = "STATE_PUT_MSG_OK_MAY_HAVE_CONFIRMED";
            break;
        }
        return stateName;
    }
}
