// Version mqe_java/source/examples/assured/SendTestMessagesAssured.java, MQeExamples, la200, la200-L030324.1 1.4

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001,2002                                            */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.assured;


import com.ibm.mqe.*;
import examples.queuemanager.*;                           


/**
 * <p>To send a never-ending sequence of test messages from one queue manager to
 * another, with a specified time period waiting between successive send operations.</p>
 * 
 * <p>If the application is stopped, by killing the task, pick-up where we left off
 * in an assured manner, so if messages were half-put but not confirmed, then
 * these get confirmed, and we move onto the next message.<p>
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.assured.SendTestMessagesAssured
 * &lt;queue_name&gt;
 * &lt;hosting_queue_manager&gt;
 * &lt;ini_file&gt;
 * &lt;seconds&gt;
 * &lt;maxMsgsToSend&gt;
 * &lt;stateFile&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * <DT>queue_name</DT>
 * <DD>is the name of the remote queue we are creating</DD>
 * <DT>hosting_queue_manager</DT>
 * <DD>is the name of the queue manager on which the queue actually exists.
 * This will normally be the same queue manager described in the specified ini_file.</DD>
 * <DT>ini_file</DT>
 * <DD>Name of the ini file used to open a client queue manager, to
 * send the messages from.</DD>
 * <DT>milliseconds</DT>
 * <DD>The number of milliseconds between successive send message operations.
 * ie: The delay between each message in time.</DD>
 * <DT>StateFile</DT>
 * <DD>Name of the file to be used in the current directory, to store persistent information.</dd>
 * </DL>
 */
public class SendTestMessagesAssured {  

    /** 
     * The version of this program
     */
    public       static short version[] = { 2, 0, 0, 5 }; 

    /**
     * The number of seconds the program sleeps for after it's posted the message...
     * To give any asynchronous threads some chance of sending the message on to a 
     * different queue manager.
     */
    public long sleepMilliSeconds = 1000 * 5;

    static final String syntax = 
        "Syntax:\n"
        + "SendTestMessages <queue_name> <hosting_queue_manager> <ini_file> <milliSeconds> <stateFile>\n"
        + "Parameters:\n"
        + "<queue_name>\n"
        + "     is the name of the queue we are sending a message to.\n"
        + "<hosting_queue_manager>\n"
        + "     is the name of the queue manager on which the queue actually exists.\n"
        + "<ini_file>\n"
        + "     Name of the ini file used to open a client queue manager, to\n"
        + "     send the messages from.\n"
        + "<milliSeconds>\n"
        + "     MilliSeconds between successive message sends.\n"
        + "<stateFile>\n"
        + "     The name of a file we can use to record persistent how-far-have-we-got type information.\n";

    /**
     * The name of the field in the message in which the number of the message is put.
     */
    public static final String FIELD_NAME_MSG_INDEX = "Message_Sequence_Number";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        SendTestMessagesAssured me = new SendTestMessagesAssured();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: failed. " + e.toString());
            e.printStackTrace();
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gets the command-line parameters, and loops sending messages.
     * @param args
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(String args[]) throws Exception {

        String q = null;        // Name of the queue we're sending to
        String qmgr = null;     // Name of the queue manager we're sending to.
        String iniFile = null;  // Name of the ini file used to open a client queue manager, to 
        // send the admin messages from.
        String stateFile = null; // The name of the file we can store state information in.

        // Check we have enough command-line parameters.
        if (args.length < 4) {
            throw new Exception("Not enough parameters !\n" + syntax);
        }

        // Extract the command-line parameters into variables we can use...
        q = args[0].trim();
        qmgr = args[1].trim();
        iniFile = args[2].trim();
        String sleepMilliSecondsString = args[3].trim();

        sleepMilliSeconds = Long.parseLong(sleepMilliSecondsString);
        stateFile = args[4].trim();

        // Summarise what we intend to do...
        System.out.println("info: Attempting to send a stream of messages to queue " + q + " on queue manager " + qmgr);

        // Access the local MQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Suppress stdout information
            );

        // Access the queue manager created by the MQeClient class...
        MQeQueueManager myQMgr = server.queueManager;

        // Loop until we're killed.
        while (true) {

            sendTestMsg(q, qmgr, myQMgr, stateFile);

            // Hold the queue manager open for long enough that the message stands a chance
            // of getting to the other end...
            java.lang.Thread.sleep(sleepMilliSeconds);
        }

        // Close down the MQeClient
        // client.close();
    } 

    /**
     * Sends a test message.
     * @param targetQ A String containing the target Queue name
     * @param targetQMgr A String containing the target Queue Manager name
     * @param myQMgr MQeQueueManager Object
     * @param stateFile A String containing the path of statefile
     * @return void
     * @exception java.lang.Exception
     */
    protected void sendTestMsg(String targetQ, String targetQMgr, MQeQueueManager myQMgr, String stateFile
    ) throws Exception {

        // Open the state file and read it.
        AtomicState atomicState = new AtomicState(stateFile);

        // Read the state from the persistent file...
        int state = atomicState.getState();

        switch (state) {
        case AtomicState.STATE_START: {  
                // Read the index of the message we want to build and send next
                // so we maintain the sequence...
                int msgIndex = atomicState.getMsgIndex();

                // build the message we want to send.
                MQeMsgObject msg = buildMessage(msgIndex);

                // get its' unique id out and save it away in case we need to 
                // confirm it later, after a program termination.
                MQeFields uniqueID = null; 

                // Invent a confirm ID... using a unique value...
                long confirmID = MQe.uniqueValue();

                // Save the confirm ID to persistent store... atomically.
                atomicState.set(uniqueID, atomicState.STATE_BUILT_MSG_OK_MAY_HAVE_PUT, confirmID, msgIndex // The index of this message.
                );

                // We have built the message, but not yet posted it.
                // We are in the STATE_PRE_PUT
                System.out.println("Putting message " + msgIndex + " confirmID of " + confirmID + " ...");
                myQMgr.putMessage(targetQMgr, targetQ, msg, null         // No compressor/cryptor/authenticator 
                    , confirmID    // We will confirm it later.
                );
                System.out.println("Putting message " + msgIndex + " confirmID of " + confirmID + " ok.");

                // Re-extract the unique ID from the message... as the unique id
                // will now have the originating queue manager added.
                uniqueID = msg.getMsgUIDFields();
                
                // Write out the state to a persistent file in case the user
                // terminates this thread... 
                atomicState.set(uniqueID, atomicState.STATE_PUT_MSG_OK_MAY_HAVE_CONFIRMED, confirmID, msgIndex // The index of this message.
                );

                // We've put the message ok, so now confirm it.
                System.out.println("Confirming message " + msgIndex + " ...");
                myQMgr.confirmPutMessage(targetQMgr, targetQ, uniqueID);
                System.out.println("Confirmed message " + msgIndex + " ok.");

                // Write out the state to a persistent file in case the user
                // terminates this thread...
                atomicState.set(null // No unique id fields to remember.
                    , atomicState.STATE_START, 0, msgIndex + 1 // The index of the next message to send.
                );
            }
            break;

        case AtomicState.STATE_BUILT_MSG_OK_MAY_HAVE_PUT: {
                // The user previously terminated the program in the past.
                // The message may have only been built + never put, 
                // or it may have been built + put.
                // Try to clean-up any activity we had before...

                // read the confirm id from out atomic state...
                long confirmID = atomicState.getConfirmID();

                // Undo the putMessage() we did previously...
                try {
                    System.out.println("Undoing message with confirmID of " + confirmID + " ...");
                    myQMgr.undo(targetQMgr, targetQ, confirmID);
                    System.out.println("Undoing message with confirmID of " + confirmID + " ok.");
                } catch (MQeException e) {
                    if (e.code() == MQe.Except_Q_NoMatchingMsg) {// There was no message to undo... 
                        // so the previous instance of this program never got 
                        // that far.
                    } else {
                        // Some other error occurred. Pass it on and up.
                        throw e;
                    }
                }

                // Read back the msgIndex from the atomic state..
                int msgIndex = atomicState.getMsgIndex();

                // We have undone any message which was put without a confirm...
                // so revert to our initial state...
                atomicState.set(null      // No unique id fields to remember now.
                    , atomicState.STATE_START, 0         // No confirm id to remember now.
                    , msgIndex  // Re-build and send starting from where we left off.
                );
            }
            break;

        case AtomicState.STATE_PUT_MSG_OK_MAY_HAVE_CONFIRMED: {
                // The user previously terminated the program in the past...
                // The message has definately been put already, and may or may not 
                // have been confirmed...
                // So confirm it anyway, to make sure it was sent at least once.
                int msgIndex = atomicState.getMsgIndex();
                MQeFields uniqueID = atomicState.getUniqueID();

                // We've confirmed the message even if it has already been confirmed.
                try {
                    System.out.println("Confirming message which may have been confirmed already...");
                    myQMgr.confirmPutMessage(targetQMgr, targetQ, uniqueID);
                    System.out.println("Confirmed message ok.");
                } catch (MQeException e) {
                    if (e.code() == MQe.Except_NotFound) {// The message had already been confirmed... This is ok.
                    } else {
                        // Some other failure... throw it onwards and upwards.
                        throw e;
                    }
                }

                // One way or the other, the message has been sent.
                atomicState.set(null, atomicState.STATE_START, 0, msgIndex + 1 // Index of the next msg to write.
                );
            }
            break;
        }
    }

    /**
     * Builds a message, and sets the Msg_Index to be the indicated value.
     */
    MQeMsgObject buildMessage(int msgIndex) throws Exception {
        MQeMsgObject msg = new MQeMsgObject();              

        msg.putInt(FIELD_NAME_MSG_INDEX, msgIndex);
        return msg;
    }
}  

