// Version mqe_java/source/examples/awt/AwtOutputStream.java, MQeExamples, la200, la200-L030408 1.15

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 1999                                                *
 *-----------------------------------------------------------------------------*/
package examples.awt;


import examples.queuemanager.*;
import java.awt.*;
import java.io.*;
 

/**
 * A class which acts as an output stream and pushes anything it is passed
 * into a java TextArea object.
 * 
 * <p>The text area widget is passed by the caller of the constructor.
 * 
 * <p>Lines longer than a set length are truncated.
 * 
 * <p>When the maximum allowable amount of data has been collected ,
 * and put into the TextArea widget, then 66% of it is removed.
 * In this manner, space is created for more recent trace information while 
 * retaining 33% of the recent history.
 * 
 * <p>A filter string can be set into this object which will suppress output
 * to the text area if the filter string does not exist in each line of output.
 */
public final class AwtOutputStream extends FilterOutputStream implements MQeClearable {

    public static short version[] = { 2, 0, 0, 5 }; 

    /**
     * The maximum amount of text characters we allow into the Text Area
     * (roughly) before we remove 66% of the oldest output information.
     */
    private static final int DEFAULT_MAX_HISTORY_CHARS = 16 * 1024 * 4;

    private int maxHistoryChars;

    /**
     * The max length of a line we put into the text area.
     * 
     * <p>Lines longer than this number of characters are truncated.
     */
    private static final int DEFAULT_MAX_LINE_LENGTH = 450;
    
    private int maxLineLength;

    /**
     * A string holding the filter to which all output messages are compared
     * to see if they should be suppressed or not.
     */
    private String filter = null;

    /**
     * A local reference to the text area passed into the constructor.
     */
    private TextArea textArea = null;

    /**
     * Constructor. 
     * @param textArea  TextArea to which output passed to this output stream will
     * be sent. Should not be null.
     */
    public AwtOutputStream(TextArea textArea) {
        this(textArea, DEFAULT_MAX_LINE_LENGTH, DEFAULT_MAX_HISTORY_CHARS);
    }

    /**
     * Constructor. 
     * @param textArea  TextArea to which output passed to this output stream will
     * be sent. Should not be null.
     * @param maxLineLength The max number of characters which are allowed per line of text.
     * Lines longer than this are truncated to this length.
     * @param maxHistoryChars The max number of characters which are held by the text area
     * output stream. when full, 1/3rd of the data is thrown away, oldest data first.
     */
    public AwtOutputStream(TextArea textArea, int maxLineLength, int maxHistoryChars) {
		
        super(null);
        
        this.maxHistoryChars = maxHistoryChars;
        this.maxLineLength = maxLineLength;

        // Provoke a null pointer exception if the textArea passed value is null.
        textArea.getText();

        this.textArea = textArea;

        // Obtain a normal font, and a bold font.
        // Bold font is used to display Errors, warnings and exceptions.
        Font normalFont = new Font("Monospaced", Font.PLAIN, 12);
		
        textArea.setFont(normalFont);
    }
	
    /**
     * Update the TextArea with the new message.
     * @param data java.lang.String
     * @return void                                   
     */
    protected void dataOutput(String data) {
        String s = textArea.getText();

        // Find out how much text the widget already has stored.
        int j = s.length();

        // Is it holding more than the max allowed history ?
        if (j > maxHistoryChars) {

            // Prune 2/3rds off the start of the text area history.
            j = s.indexOf("\r\n", j - (j / 3));

            // Find a new line around the area we want to cut off...
            if (j > -1) {
                textArea.replaceRange("......\r\n", 0, j + 2);
            }
        }

        // is record length ok ?        
        if (data.length() > maxLineLength) {
            data = data.substring(0, maxLineLength) + " <<< truncated >>>";
        }

        textArea.append(data + "\r\n");
    }

    /**
     * From the MQeClearable interface. Clear all the output we have so far.
     */
    public void clear() {
        textArea.setText("");		
    }

    /**
     * Apply a filter to the output display (not applied to logfile).   
     * @param filter  java.lang.String
     * @return  java.lang.String
     */
    public String filter(String filter) {

        // Saves the old filter so we can return it later.
        String Temp = this.filter;

        // Set the new filter.
        this.filter = filter;

        // return the previous filter   
        return (Temp);
    }

    /**
     * Clean up before this object is garbage-collected.
     */
    protected void finalize() {
        try {
            textArea = null;
            filter = null;
        } catch (Exception e) {// Ignores errors.
        }
    }

    /**
     * Required by the output stream interface. This implementation does nothing.
     */
    public void flush() {}

    /**
     * Tormat tab, cr and lf characters if they are present. Pass the formatted 
     * data onto the output method.
     * 
     * @param data The text to output.
     * @param tabSize The number of spaces a tab should be substituted for.
     */
    protected void format(String data, int tabSize) {

        int l = 0; // Start line number.
        char c[] = new char[data.length()]; // Work buffer

        // Convert the data to a char array.
        data.getChars(0, data.length(), c, 0);
        StringBuffer result = new StringBuffer(maxLineLength);

        // Walk through every character in the data...
        for (int i = 0; i < c.length; i = i + 1) {

            // Do something different depending on the character being output.
            switch (c[i]) {

            // Carriage return.
            case '\r':
                break;

                // New Line                                 
            case '\n':
                l = 0;
                if (result.length() > 0) {
                    output(result.toString());
                    result = new StringBuffer(maxLineLength);
                }
                break;

                // Tab character                                    
            case '\t':
                // save current position 
                int m = l;

                // Replace a tab with spaces.
                for (l = m; l < tabSize + 1; l = l + 1) {
                    result.append(' ');
                }
                l = 0;
                break;

                // all other characters                                 
            default:
                result.append(c[i]);
                l = (l + 1) % tabSize;
                break;
            }
        }
		
        // If we have any data to output, do so.			
        if (result.length() > 0) {
            output(result.toString());
        }

    }

    /**
     * Replace one substring with another in a string.    
     * @param source String in which the sustring is to be replaced.
     * @param what String that needs to be replaced
     * @param with String to replace with  
     * @return java.lang.String 
     */
    public static String replace(String source, String what, String with) {
        StringBuffer result = new StringBuffer(source.length());
        int j = 0;

        // Find if there is a match...                
        int i = source.indexOf(what, j);

        // While there is a match... do another substitution.
        while (i > -1) {
            // get the prefix           
            result.append(source.substring(j, i));

            // add-in the replacement string           
            result.append(with);

            // new scan start               
            j = i + what.length();

            // is there a match ? 
            i = source.indexOf(what, j);
        }

        // Append any data which is left over...
        if (j < source.length()) {
            result.append(source.substring(j));
        }
        return (result.toString());	
    }
	
    /**
     * Save the current Text area contents.
     * @param thisFile file name.
     * @return void   
     * @exception java.lang.Exception                                   
     */
    public void saveAs(String thisFile) throws Exception {
        File lf = new File(thisFile);
        DataOutputStream sf = new DataOutputStream(new FileOutputStream(lf));

        sf.writeBytes(replace(textArea.getText(), "\n", "\r\n"));
        sf.close();
    }

    /**
     * Output the message data to display and/or to log (Filter permitting).
     * @param data   java.lang.String
     * @return void      
     */
    protected void output(String data) {
        // If the string to output passes the filter, put it out.
        if ((filter == null) || (data.indexOf(filter) > -1)) {
            dataOutput(data);
        }
    }

    /**
     * Write byte array.
     * @param data Data to be written in to the byte Array
     * @param offset Offset from where the write should happen
     * @param len The total number of bytes to be written
     * @return void
     */
    public synchronized void write(byte data[], int offset, int len) {

        // Convert the data to a string.
        String line = new String(data, offset, len);

        // just cr-lf ?                 
        if (!line.equals("\r\n")) {
            format(line, 8);
        }
    }

    /**
     * Write byte array.
     * @param data Data to be written in to the byte Array
     * @return void
     **/
    public synchronized void write(byte data[]) {
        write(data, 0, data.length);
    }

    /**
     * Write character.
     * @param Data an integer
     * @return void
     */
    public synchronized void write(int Data) {// TODO: should really do something with this character
    }
}
