// Version 1.5 mqe_java/source/examples/config/BasicAdministration.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.config;


import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeAdminMsg;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.administration.MQeQueueManagerAdminMsg;

import java.io.File;


/**
 * This class illustrates the basics of administration using admin messages.
 * <p>It uses {@link examples.config.CreateQueueManager} to create and start a queue manager
 * as an entry point to MQe, and as a subject of a queue manager query.
 */
public class BasicAdministration {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    /**
     * This method performs standard processing that decoprates an admin message
     * so that we can handle it in a standard way.
     * <p>This method:
     * <p> Sets the target queue manager (the queue maanger upon which
     * the admin action takes place.
     * <p> Requests that a reply message is sent to the admin reply queue on
     * the target queue manager.
     * <p> Incorporates a unique key in the message that can be used to retrieve
     * the reply for this message.  The unique key is returned as a string, to be
     * used by the routine extracting the reply.
     */
    public static final String decorateAdminMsg(MQeAdminMsg msg, String targetQMName) throws Exception {

        // set the target queue manager
        msg.setTargetQMgr(targetQMName);

        // indicate that we require a reply message
        msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

        // use default reply-to queue on the target queue manager.
        msg.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        msg.putAscii(MQe.Msg_ReplyToQMgr, targetQMName);

        // create a unique tag that we can identify the reply with
        String match = "Msg" + System.currentTimeMillis();

        msg.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        return match;
    }

    /**
     * Wait for message - waits for a message to arrive on the admin reply queue
     * of the specified target queue manager.  Will wait only for messages with the
     * specified unique tag
     * return message, or null if timed out
     */
    public static final MQeAdminMsg waitForRemoteAdminReply(
        MQeQueueManager localQueueManager,
        String remoteQueueManagerName,
        String match) throws Exception {
        // construct a filter to ensure we only get the matching reply
        MQeFields filter = new MQeFields();

        filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        // now wait for the reply message
        MQeMsgObject reply = localQueueManager.waitForMessage(
                remoteQueueManagerName,
                MQe.Admin_Reply_Queue_Name,
                filter,
                null,
                0L,
                10000);          // wait for 10 seconds

        return (MQeAdminMsg) reply;
    }

    /**
     *  Reply true if the given admin reply message represents a succesful
     * admin action.  Return false otherwise.  A message indicating success
     * or failure will be printed to the console.
     * If the admin action was not successful then the reason will be printed
     * to the console
     */
    public static final boolean isSuccess(MQeAdminMsg reply) throws Exception {
        boolean success = false;
        final int returnCode = reply.getRC();

        switch (returnCode) {
        case MQeAdminMsg.RC_Success:
            System.out.println("Admin succeeded");
            success = true;
            break;

        case MQeAdminMsg.RC_Fail:
            System.out.println("Admin failed, reason: " + reply.getReason());
            break;

        case MQeAdminMsg.RC_Mixed:
            System.out.println("Admin partially succeeded:\n" + reply.getErrorFields());
            break;
        }
        return success;
    }

}
