
// Version 1.3 mqe_java/source/examples/helloworld/Run.java, MQeExamples, la200, la200-L030324.1
/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.helloworld;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.trace.MQeTraceToReadable;
import java.util.*;


/**
 * Very simplified example to use a queue manager on disk.
 *
 * <p>The steps performed by this example application are:
 * <ul>
 * <li>Starts a queue manager which has previously been created using the
 * {@link examples.helloworld.Configure} program
 * <li>Put a message to a local queue
 * <li>Get the message from a local queue
 * <li>Stops the queue manager
 * </ul>
 *
 */
public class Run {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    /**
     * A definition of my queue manager.
     */
    private MQeQueueManager myQueueManager = null;

    /**
     * Constant indicating whether we should collect MQe trace or not.
     */
    private static final boolean MQE_TRACE_ON = false;

    /**
     * Private reference to an MQeTrace object to make sure it doesn't get
     * garbage collected in-between methods.
     */
    private MQeTrace myMQeTrace = null;

    /**
     * How long to wait for the queue manager to close cleanly, before it is closed by force ?
     * <p>Using 1 second for this example.
     */
    private static final long QUIESCE_TIME = (long) 1000;

    /**
     * The name of the queue manager.
     */
    private String queueManagerName = null;

    /**
     * Entry point from the command line.
     *
     * <p>This method will perform the following operations:
     * <ul>
     * <li>Create an instance of the MQeMemoryQM class.
     * <li>Define the queue manager in memory.
     * <li>Start the queue manager.
     * <li>Put a simple message to a queue on the queue manager.
     * <li>Get the message off the queue.
     * <li>Stop the queue manager
     * <li>Remove the queue manager definition from memory.
     * <li>Optionally trace using a MQe trace handler to stdout, depending on whether a constant is set or not.
     * </ul>
     */
    public static void main(String[] args) {
        try {
            Run me = new Run();

            if (MQE_TRACE_ON) {
                me.traceOn();
            }
            me.start();
            me.put();
            me.get();
            me.stop();
            if (MQE_TRACE_ON) {
                me.traceOff();
            }
        } catch (Exception error) {
            System.err.println("Error: " + error.toString());
            error.printStackTrace();
        }
    }

    /**
     * Turns MQe trace on.
     */
    public void traceOn() throws Exception {

        // Allocate a trace instance, so our handler isn't garbage collected when its' on.
        myMQeTrace = new MQeTrace();

        // Allocate a trace handler
        // This one puts trace output to stdout by default.
        MQeTraceHandler handler = (MQeTraceHandler) new com.ibm.mqe.trace.MQeTraceToReadable();

        // Set this handler as the one MQe uses.
        MQeTrace.setHandler(handler);

        // Set the filter so we collect those pieces of trace we are interrested in.
        // In this case, it't the MQe API calls, errors, warnings, and exceptions.
        MQeTrace.setFilter(MQeTrace.GROUP_API | MQeTrace.GROUP_ERROR
            | MQeTrace.GROUP_WARNING | MQeTrace.GROUP_EXCEPTION
        );
    }

    /**
     * Starts the queue manager.
     */
    public void start() throws Exception {

        System.out.println("Starting the queue manager.");

        queueManagerName = "HelloWorldQM";
        String baseDirectoryName = "./QueueManagers/" + queueManagerName;

        // Create all the configuration information needed to construct the
        // queue manager in memory.
        MQeFields config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

        registrySection.putAscii(MQeRegistry.Adapter, "com.ibm.mqe.adapters.MQeDiskFieldsAdapter");
        registrySection.putAscii(MQeRegistry.DirName, baseDirectoryName + "/Registry");

        config.putFields("Registry", registrySection);

        System.out.println("Starting the queue manager");
        myQueueManager = new MQeQueueManager();
        myQueueManager.activate(config);
        System.out.println("Queue manager started.");
    }

    /**
     * Puts a simple message to the system default local queue.
     */
    public void put() throws Exception {
        System.out.println("Putting the test message");
        MQeMsgObject msg = new MQeMsgObject();

        // Add my hello world text to the message.
        msg.putUnicode("myFieldName", "Hello World!");

        myQueueManager.putMessage(queueManagerName, MQe.System_Default_Queue_Name, msg, null, 0L);
        System.out.println("Put the test message");
    }

    /**
     * Gets a simple message from the system default local queue.
     */
    public void get() throws Exception {
        System.out.println("Getting the test message.");
        MQeMsgObject msg = myQueueManager.getMessage(queueManagerName, MQe.System_Default_Queue_Name, null, null, 0L
            );

        if (msg != null) {
            System.out.println("Got the test message.");

            if (msg.contains("myFieldName")) {
                String textGot = msg.getUnicode("myFieldName");

                System.out.println("Message contained the text '" + textGot + "'");
            }
        }
    }

    /**
     * Stops the queue manager.
     */
    public void stop() throws Exception {
        System.out.println("Stopping the queue manager.");
        myQueueManager.closeQuiesce(QUIESCE_TIME);
        myQueueManager = null;
        System.out.println("Queue manager stopped.");
    }

    /**
     * Turns MQe trace off.
     */
    public void traceOff() throws Exception {
        // Stop collecting trace information.
        MQeTrace.setFilter(MQeTrace.GROUP_MASK_NONE);

        // Remove the trace handler.
        MQeTrace.setHandler(null);
    }
}
