// Version mqe_java/source/examples/install/SimpleCreateQM.java, MQeExamples, la200, la200-L030414.1 1.12

/* -----------------------------------------------------------------------------*/

/*              WebSphere MQ Everyplace for Multiplatforms                      */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

package examples.install;


import com.ibm.mqe.*;
import examples.queuemanager.MQeQueueManagerUtils;

import java.io.*;
import java.util.ResourceBundle;


/*
 * <p>To create a basic queue manager from a configuration                    
 * (startup) .ini file.                                                   
 *                                                                              
 * <p>It creates the registry entries for the Queue Manager and the           
 * standard local queues. The configuration file must already exist.       
 * It illustrates the use of the MQeQueueManagerConfigure class.           
 *                                                                              
 * <p>The main program takes one mandatory and one optional parameter :       
 *                                                                              
 * <P><TT>java SimpleCreateQM configFile [queueDirectory [adminQStore
 * [adminReplyQStore [deadLetterQStore [systemDefaultLocalQStore ]]]]] </TT>
 *   
 * <DL>
 *  <DT>adminQStore</DT>
 * <DD>The message store, storage adapter, and directory name where the administration queue
 * will store its' messages. use "default" to default the value. Optional.</DD>
 * <DT>adminReplyQStore</DT>
 * <DD>The message store, storage adapter, and directory name where the administration reply 
 * queue will store its' messages. use "default" to default the value. Optional.</DD>
 * <DT>deadLetterQStore</DT>
 * The message store, storage adapter, and directory name where the dead letter queue
 * will store its' messages. use "default" to default the value. Optional.</DD>
 * <DT>systemDefaultLocalQStore</DT>
 * The message store, storage adapter, and directory name where the system default local queue
 * will store its' messages. use "default" to default the value.Optional.</DD>
 * </DL>
 *  
 *                                                                            
 * <p>The first parameter is the configuration file used to construct         
 * the Queue Manager. The second parameter is the directory where          
 * queues are to be placed. If it is not specified as a parameter          
 * it defaults to ".\Queues\". The queue directory will be created         
 * if it does not already exist.                                           
 *                                                                              
 * <p>This class is extended by CreateQueueManager, which uses a GUI          
 * to interact with the user and then uses the methods in this base        
 * class to create a Queue Manager.  
 * 	
 * <p>If the .ini file contains the section [QueueManagerCreateParameters]
 * then these parameters will be used by this program to explicitly set the 
 * locations of system queues, instead of using the default queue location.
 *                                                                                                                
 **/

public class SimpleCreateQM {

    public       static short version[] = { 2, 0, 0, 5 }; 
    // Default values
    public String defQueueDir = "Queues";
    public String defQueueAdapter = "MsgLog:";
    public String defRegDir = "Registry";
    public String defRegSeparator = "+";
    public String defChnlAttrRuleAlias = "ChannelAttrRules";
    public final static String TEXT_LABEL_DEFAULT = "default";

    // Utility methods
    ConfigUtils utils = null;

    // Configuration fields (used in the super class)
    MQeFields configFields = null;

    // Other Strings, for NLS
    String noFileStr;
    String notConfigFileStr;
    String errCreatingQMStr;
    String qmDescriptionStr;

    /**
     * SimpleCreateQM constructor.
     **/
    public SimpleCreateQM() {
        utils = new ConfigUtils();
        initStrings();
    }

    /**
     * This does all the work: validate the configuration file,
     * generate the queue directory and create the Queue manager.
     *
     * @param filename    Name of the startup .ini file
     * @param queueDir    Name of the directory to store the queues.
     *                    If this is null, a default is created.
     * @return            'true' if successful, 'false' if it didn't complete
     * @exception  java.lang.exception  Thrown on error
     */
    public boolean createQMgr(String filename, String queueDir) throws Exception {
        boolean result = createQMgr(filename, queueDir, 
                TEXT_LABEL_DEFAULT,
                TEXT_LABEL_DEFAULT,
                TEXT_LABEL_DEFAULT,
                TEXT_LABEL_DEFAULT
            );

        return result;									 
    }

    /**
     * This does all the work: validate the configuration file,
     * generate the queue directory and create the Queue manager.
     *
     * @param filename    Name of the startup .ini file
     * @param queueDir    Name of the directory to store the queues.
     *                    If this is null, a default is created.
     * @param qStoreAdminQ The exact location the queue will put it's data, or "default"
     * @param qStoreAdminReplyQ The exact location the queue will put it's data, or "default"
     * @param qStoreDeadLetterQ The exact location the queue will put it's data, or "default"
     * @param qStoreSystemDefaultLocalQ The exact location the queue will put it's data, or "default"
     * 
     * @return            'true' if successful, 'false' if it didn't complete
     * @exception  java.lang.exception  Thrown on error
     **/
    public boolean createQMgr(String filename, String queueDir, 
        String qStoreAdminQ,
        String qStoreAdminReplyQ,
        String qStoreDeadLetterQ,
        String qStoreSystemDefaultLocalQ
    ) throws Exception {
        // verify the configuration filename
        if (filename == null || filename.length() == 0)
            throw new Exception(noFileStr);

        configFields = utils.verifyIniFile(filename);
        if (configFields == null)
            throw new Exception("'" + filename + "' " + notConfigFileStr);

        // check for confirmation
        if (confirmDetails()) {
            // if a queue directory wasn't specified, generate a default
            if (queueDir == null) {
                // generate a default from the configuration file name
                queueDir = getDefaultQueuePath(filename);
            }

            // make the Queue Directory if necessary
            boolean queueDirCreated = utils.makeDirectory(queueDir);

            // we've got all the information we need
            try {
                // define a QM with all the local queues,
                // using the default queue store adapter
                defQMRegistry(configFields, defQueueAdapter + queueDir, true, true, true, true,
                    qStoreAdminQ,
                    qStoreAdminReplyQ,
                    qStoreDeadLetterQ,
                    qStoreSystemDefaultLocalQ
                );
            } catch (Exception e) {
                // If a queue directory was created, delete it
                if (queueDirCreated) {
                    new File(queueDir).delete();
                    queueDirCreated = false;
                }
                e.printStackTrace();
                throw new Exception(errCreatingQMStr + ": " + e.getMessage());
            }

            return true;
        } else
            return false;
    }

    /**
     * Confirm the details.
     *
     * @return   'true' to confirm the details, 'false' to reject them
     **/
    boolean confirmDetails() {
        // This is not used in this class and is provided so that it can
        // be overridden.

        return true;
    }

    public void defQMRegistry(MQeFields configFields,
        String    queueDir,
        boolean   adminQ,
        boolean   adminReplyQ,
        boolean   systemQ,
        boolean   deadLetterQ
    ) throws Exception {
        defQMRegistry(configFields,
            queueDir,
            adminQ,
            adminReplyQ,
            systemQ,
            deadLetterQ,
            TEXT_LABEL_DEFAULT,
            TEXT_LABEL_DEFAULT,
            TEXT_LABEL_DEFAULT,
            TEXT_LABEL_DEFAULT  
        );  		                             
    }

    /**
     * defQMRegistry - create the queue manager, ie create the
     * registry definitions. If there is an error, any newly
     * created definitions are deleted so that the registry is
     * left in its initial state.
     *
     * @param ConfigFields  A MQeFields object containing
     *                      the configuration parameters.
     * @param queueDir      The directory for the queue store.
     * @param adminQ        Define an admin queue.
     * @param adminReplyQ   Define an admin reply queue.
     * @param systemQ       Define a default system local queue.
     * @param deadLetterQ   Define a dead letter queue.
     * @exception         java.lang.exception
     **/
    public void defQMRegistry(MQeFields configFields,
        String    queueDir,
        boolean   adminQ,
        boolean   adminReplyQ,
        boolean   systemQ,
        boolean   deadLetterQ,
        String qStoreAdminQ,
        String qStoreAdminReplyQ,
        String qStoreDeadLetterQ,
        String qStoreSystemDefaultLocalQ
    ) throws Exception {
        // Configuration object
        MQeQueueManagerConfigure qMgrConfig = null;

        // progress flags
        boolean qMgrOpened = false;
        boolean createdQM = false;
        boolean createdAdminQ = false;
        boolean createdAdminReplyQ = false;
        boolean createdSystemQ = false;
        boolean createdDeadLetterQ = false;

        try {
            // Establish any aliases defined by the .ini file
            MQeQueueManagerUtils.processAlias(configFields);    
            MQeQueueManagerUtils.processSystemProperties(configFields);

            // Now instantiate QueueManagerConfigure
            qMgrConfig = new MQeQueueManagerConfigure(configFields, queueDir);
            qMgrOpened = true;

            // set values for the Queue Manager properties
            qMgrConfig.setDescription(qmDescriptionStr + qMgrConfig.getName());
            qMgrConfig.setChnlAttributeRuleName(defChnlAttrRuleAlias);

            // Create the Queue manager registry entry
            qMgrConfig.defineQueueManager();
            createdQM = true;

            // Create the default local queues as required
            if (adminQ) {
                if (qStoreAdminQ.equals(TEXT_LABEL_DEFAULT)) {
                    qMgrConfig.defineDefaultAdminQueue();
                } else {
                    String qDescription = "";

                    qMgrConfig.defineDefaultAdminQueue(qDescription, qStoreAdminQ);
                }
                createdAdminQ = true;
            }

            if (adminReplyQ) {
                if (qStoreAdminReplyQ.equals(TEXT_LABEL_DEFAULT)) {      		
                    qMgrConfig.defineDefaultAdminReplyQueue();
                } else {
                    String qDescription = "";

                    qMgrConfig.defineDefaultAdminReplyQueue(qDescription, qStoreAdminReplyQ);
                }
                createdAdminReplyQ = true;
            }

            if (systemQ) {
                if (qStoreSystemDefaultLocalQ.equals(TEXT_LABEL_DEFAULT)) {
                    qMgrConfig.defineDefaultSystemQueue();
                } else {
                    String qDescription = "";

                    qMgrConfig.defineDefaultSystemQueue(qDescription, qStoreSystemDefaultLocalQ);
                }
                createdSystemQ = true;
            }

            if (deadLetterQ) {
                if (qStoreDeadLetterQ.equals(TEXT_LABEL_DEFAULT)) {
                    qMgrConfig.defineDefaultDeadLetterQueue();
                } else {
                    String qDescription = "";

                    qMgrConfig.defineDefaultDeadLetterQueue(qDescription, qStoreDeadLetterQ);
                }	  		
                createdDeadLetterQ = true;
            }

            // Definition complete - close the instance
            qMgrConfig.close();

        } catch (Exception e) {
            // if there were any errors, cleanup and then propagate the exception

            // If a Queue Manager Definition has been started, remove it
            if (qMgrOpened && qMgrConfig != null) {
                try {
                    // delete any queue definitions that have just been created
                    if (createdDeadLetterQ)
                        qMgrConfig.deleteDeadLetterQueueDefinition();

                    if (createdSystemQ)
                        qMgrConfig.deleteSystemQueueDefinition();

                    if (createdAdminReplyQ)
                        qMgrConfig.deleteAdminReplyQueueDefinition();

                    if (createdAdminQ)
                        qMgrConfig.deleteAdminQueueDefinition();

                    // delete the queue manager definition if we created it
                    if (createdQM)
                        qMgrConfig.deleteQueueManagerDefinition();
                } catch (Exception ex) {} // ignore any errors deleting the definitions

                qMgrConfig.close();
            }

            // propagate the exception
            throw e;
        }
    }

    /**
     * Generate a default pathname for the Queues directory.
     * This is based on the pathname of the configuration file.
     * If this ends with the queue manager name, the default
     * queue directory will be:
     *    <congfig file pathname>\Queues\
     * If it doesn't end with the queue manager name, the
     * default queue directory will be:
     *    <congfig file pathname>\<queue manager name>\Queues\
     *
     * Note that this is used as a default which the user can
     * always override.
     *
     * @param configFile  The name of the configuration file.
     * @return            The default pathname for queues.
     **/
    String getDefaultQueuePath(String configFile) {
        String result = new File(".").getAbsolutePath() + File.separator + defQueueDir + File.separator;

        // get the Queue Manager Name
        MQeFields configFields = utils.verifyIniFile(configFile);

        if (configFields != null) {
            String QMgrName = null;

            try {
                // Get the QMgr section of the config params
                MQeFields QMgrParams = configFields.getFields(MQeQueueManager.QueueManager);

                QMgrName = QMgrParams.getAscii(MQeQueueManager.Name).trim();
            } catch (Exception e) {
                return result;  // return the standard default if there are any errors
            }

            // We've got the queue manager name, now get the full
            // pathname for the configuration file
            String configPath = new File(configFile).getAbsolutePath();
            // extract the last directory name from the full pathname
            int iEnd = configPath.lastIndexOf(File.separator);

            if (iEnd > -1) {
                configPath = configPath.substring(0, iEnd);

                String lastDir = "";

                iEnd = configPath.lastIndexOf(File.separator);
                if (iEnd > -1)
                    lastDir = configPath.substring(iEnd + 1);

                // if the last directory name does not match the queue manager name,
                // add the queue manager name to the path
                if (!lastDir.equalsIgnoreCase(QMgrName))
                    configPath = configPath + File.separator + QMgrName;

                // add "\Queues\" to the end of the path
                result = configPath + File.separator + defQueueDir + File.separator;
            }
        }

        return result;
    }

    /**
     * Initialise all the strings using a ResourceBundle.
     **/
    void initStrings() {
        // Resources
        ResourceBundle resources = utils.getResourceBundle();

        noFileStr = resources.getString("noFile");
        notConfigFileStr = resources.getString("notConfigFile");
        errCreatingQMStr = resources.getString("errCreatingQM");
        qmDescriptionStr = resources.getString("qmDescription");
    }

    /**
     * Start the application.
     **/
    public static void main(java.lang.String[] args) {
        String filename = null;
        String queueDir = null;
	
        // check the arguments
        if (args.length > 0) {
            filename = args[0];
        }
	
        if (args.length > 1) {
            queueDir = args[1];
        }
	    
        // Default all the system queue storage locations 
        String qStoreAdminQ = TEXT_LABEL_DEFAULT;
        String qStoreAdminReplyQ = TEXT_LABEL_DEFAULT;
        String qStoreDeadLetterQ = TEXT_LABEL_DEFAULT;
        String qStoreSystemDefaultLocalQ = TEXT_LABEL_DEFAULT;
		
        // Pick system queue storage locations up from the command line
        // parameters if they are specified.
        if (args.length > 2) {
            qStoreAdminQ = args[2];
        }
        if (args.length > 3) {
            qStoreAdminReplyQ = args[3];
        }
        if (args.length > 4) {
            qStoreDeadLetterQ = args[4];
        }
        if (args.length > 5) {
            qStoreSystemDefaultLocalQ = args[5];
        }			

        try {
            new SimpleCreateQM().createQMgr(filename, queueDir, qStoreAdminQ, qStoreAdminReplyQ, qStoreDeadLetterQ, qStoreSystemDefaultLocalQ
            );      
        } catch (Exception e) {
            System.err.println("*** " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
        System.out.println("SimpleCreateQM Successful");
    }

}

