// Version mqe_java/source/examples/install/SimpleDeleteQM.java, MQeExamples, la200, la200-L030324.1 1.6

/* -----------------------------------------------------------------------------*/

/*              WebSphere MQ Everyplace for Multiplatforms                      */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

package examples.install;


import com.ibm.mqe.*;
import examples.queuemanager.MQeQueueManagerUtils;

import java.io.*;
import java.util.ResourceBundle;


/*
 *    <p>To provide a simple program for deleting a basic queue manager.         
 *      It deletes the registry entries for the Queue Manager and the           
 *      standard local queues, and optionally deletes the configuration         
 *      file. It illustrates the use of the MQeQueueManagerConfigure            
 *      class.                                                                  
 *                                                                              
 *    <p>The main program takes one mandatory and one optional parameter.        
 *                                                                              
 *    <p>     java SimpleDeleteQM <config file> [<delete flag>]                   
 *                                                                              
 *    <p>The first parameter is the configuration file for the Queue             
 *      Manager. The second parameter specifies whether or not to delete        
 *      the configuration file. If it is "delete" (in any mixture of upper      
 *      and lower case) the file will be deleted. If the second parameter       
 *      is not specified, the configuration file will not be deleted.           
 *                                                                              
 *    <p>This class is extended by DeleteQueueManager, which uses a GUI          
 *      to interact with the user and then uses the methods in this base        
 *      class to delete a Queue Manager configuration.                          
 *                                                                              
 *                                                                                                                                      
 **/


public class SimpleDeleteQM {
    public       static short version[] = { 2, 0, 0, 5 }; 

    // Utility methods
    ConfigUtils utils = null;

    // Configuration fields (used in the super class)
    MQeFields configFields = null;

    // Other Strings, for NLS
    String noFileStr;
    String notConfigFileStr;
    String errDeletingQMStr;

    /**
     * SimpleDeleteQM constructor.
     **/
    public SimpleDeleteQM() {
        utils = new ConfigUtils();
        initStrings();
    }

    /**
     * This does most of the work: validate the configuration file,
     * check for confirmation, then delete the Queue Manager and
     * Queue definitions and optionally delete the configuration file.
     *
     * @param filename    Name of the configuration (.ini) file.
     * @param deleteFlag  If true, delete the configuration file.
     * @return            'true' if successful, 'false' if it didn't complete
     * @exception  java.lang.exception  Thrown on error
     **/
    public boolean deleteConfiguration(String filename, boolean deleteFlag) throws Exception {
        // verify the configuration filename
        if (filename == null || filename.length() == 0)
            throw new Exception(noFileStr);

        configFields = utils.verifyIniFile(filename);
        if (configFields == null)
            throw new Exception("'" + filename + "' " + notConfigFileStr);

        // check for confirmation
        if (confirmDetails()) {
            // delete the standard definitions
            try {
                // Establish any aliases defined by the .ini file
                MQeQueueManagerUtils.processAlias(configFields);

                // use MQeQueueManagerConfigure to perform the deletion
                MQeQueueManagerConfigure qMgrConfig = new MQeQueueManagerConfigure(configFields);

                qMgrConfig.deleteStandardQMDefinitions();
                qMgrConfig.close();

            } catch (Exception e) {
                throw new Exception(errDeletingQMStr + ": " + e.getMessage());
            }

            // delete the configuration file
            if (deleteFlag) {
                File f = new File(filename);

                f.delete();
            }

            return true;
        } else        // confirmDetails() returned 'false'
            return false;
    }

    /**
     * Confirm the details.
     *
     * @return   'true' to confirm the details, 'false' to reject them
     **/
    boolean confirmDetails() {
        // This is not used in this class and is provided so that it can
        // be overridden.

        return true;
    }

    /**
     * Initialise all the strings using a ResourceBundle.
     **/
    void initStrings() {
        // Resources
        ResourceBundle resources = utils.getResourceBundle();

        noFileStr = resources.getString("noFile");
        notConfigFileStr = resources.getString("notConfigFile");
        errDeletingQMStr = resources.getString("errDeletingQM");
    }

    /**
     * Start the application.
     **/
    public static void main(java.lang.String[] args) {
        String filename = null;
        boolean deleteFlag = false;

        // check the arguments
        if (args.length > 0)
            filename = args[0];

        if (args.length > 1)
            deleteFlag = args[1].equalsIgnoreCase("delete");

        try {
            new SimpleDeleteQM().deleteConfiguration(filename, deleteFlag);
        } catch (Exception e) {
            System.err.println("*** " + e.getMessage());
            System.exit(1);
        }
    }

}

