package examples.ivt;


// Version mqe_java/source/examples/ivt/MQeIVT.java, MQeExamples, la200, la200-L030324.1 1.1
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
import java.io.File;
import com.ibm.mqe.*;


/**
 *
 * <p>Run the MQe installation verification test (IVT).
 *
 * <p>Creates an queue manager and then executes the following applications:
 * <pre>
 *  . examples.install.Example1
 *  . examples.install.Example2
 *  . examples.install.Example3
 *  . examples.install.Example4
 *  . examples.install.Example5
 *  . examples.install.Example6
 *  . examples.administration.simple.Example1
 * </pre>
 * <p>If all suceed then a the application completes with a return code of 0
 * if it fails it exists with a non zero return code.
 *
 * <p>By default the IVT will start a queue manager using start up parameters
 * retreived from file .\ExamplesMQeClient.ini. The ini file used to
 * retrieve the startup parameters together with the name of the queue
 * manager that tests are run against can be altered by using the non-default
 * constructors.
 *
 * <p>The IVT will create the queue manager specified in the startup parameters
 * file and delete it at the end. Hence the queue manager must not exist
 * when the IVT is run or else it will fail.
 **/

public class MQeIVT {
    public static short[] version = { 2, 0, 0, 5 }; 

    /*
     * The default ini file to pick up the local queue managers startup
     * parameters from
     */
    protected String startupIni = "." + File.separator + "ExamplesMQeClient.ini";

    /*
     * The default name of the queue manager that the tests are
     * run against
     */
    protected String QMName = "ExampleQM";

    /**
     * Constructor that uses the default queue manager and ini file.
     * @exception java.lang.Exception
     */
    public MQeIVT() throws Exception {}

    /**
     * Constructor that uses the queue manager passed in on the command line
     * and the default ini file.
     * @param qMgrName java.lang.String , queue manager name.
     *        The name of the queue manager to interact with.
     *        The queue manager can be a local or a remote queue
     *        manager.
     * @exception java.lang.Exception
     */
    public MQeIVT(String qMgrName) throws Exception {
        QMName = qMgrName;
    }

    /**
     * <p>Constructor that starts a queue manager with paramters
     * from an ini file. This queue manager is then used as the
     * access point for interacting with the same or different
     * queue managers.
     * @param qMgrName java.lang.String, queue manager name.
     *        The name of the queue manager to interact with.
     *        The queue manager can be a local or a remote queue
     *        manager.
     * @param localQMIniFile java.lang.String, ini file which
     *        contains startup parameters for the local queue
     *        manager.
     * @exception java.lang.Exception
     */
    public MQeIVT(String qMgrName, String localQMgrIniFile) throws Exception {
        QMName = qMgrName;
        startupIni = localQMgrIniFile;
    }

    /**
     * Run each test in sequence.
     * If a failure occurs then no more tests will run and the exception
     * will be passed back to the caller.
     */
    public void doIt() throws Exception {
        System.out.println("MQeIVT Started using:");
        System.out.println("..Queue manager startup parameters: " + startupIni);
        System.out.println("..Queue manager to run tests on: " + QMName);

        /*
         * First create the queue manager that will be used by the tests.
         * It is important that if the queue mananger already exists that
         * the IVT does not delete it. Hence the call
         * to create the queue manager is made outside of the following
         * try, finally block.
         */
        System.out.println("MQeIVT Tidy up - create a queue manager");
        new examples.install.SimpleCreateQM().createQMgr(startupIni, null);

        try {

            /*
             * Run each test in sequence
             *  If any test fails then no more tests will run and the
             *  exception will be passed back to the caller.
             */
            new examples.application.Example1(QMName, startupIni).doIt();
            new examples.application.Example2(QMName, startupIni).doIt();
            new examples.application.Example3(QMName, startupIni).doIt();
            new examples.application.Example4(QMName, startupIni).doIt();
            new examples.application.Example5(QMName, startupIni).doIt();
            new examples.application.Example6(QMName, startupIni).doIt();
            new examples.administration.simple.Example1(QMName, startupIni).doIt();

            /*
             * All tests ran successfully
             */
            System.out.println("MQeIVT Finished Successfully");
        } finally {

            /*
             * If the queue manager was created by the IVT then
             * delete it ensuring everything is left in a tidy state.
             */
            System.out.println("MQeIVT Tidy up - delete the queue manager");
            new examples.install.SimpleDeleteQM().deleteConfiguration(startupIni, false);
        }
    }

    /**
     * <p>Starts the application.
     * @param param1 Queue Manager Name to interact with.
     * If queue manager name matches the local QM name then
     * all interactions are local otherwise the local QM acts
     * as a stub QM sending all messages to a remote QM.
     * @param param2 Local queue manager startup parameters ini file.
     * @return void
     * @exception java.lang.Exception
     */
    public static void main(String args[]) throws Exception {
        short rc = 0;

        try {
            if (args.length == 0)
                new MQeIVT().doIt();
            else if (args.length == 1)
                new MQeIVT(args[0]).doIt();
            else
                new MQeIVT(args[0], args[1]).doIt();
        } catch (Exception ex) {
            System.out.println("MQeIVT failed: " + ex);
            ex.printStackTrace();
            rc = 99;
        }
        System.exit(rc);

    }
}
