package examples.midp.exampleapp.messageservice;


/*----------------------------------------------------------------------------*/

/*                  WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/

import java.util.Enumeration;

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import com.ibm.mqe.adapters.MQeCommunicationsAdapter;


/**
 * This implements a message service and aims to wrap an mqe queue manager and provide
 * all basic functionality with a simpler interface.
 *
 * <p>This means that all queue and queue manager creation and administration will be done
 * via this class.
 *
 *
 * <p>Functions can be split up as follows:
 * <pre>
 * Message Service Functions
 * ---------------------------------------------
 * getMessageConsumer()
 * getQMname()
 * getStartupParameters()
 * setStartupParameters()
 * messageArrived() [inherited from MQeMessageListenerInterface]
 * processAlias()
 *
 * Queue Manager Functions
 * ---------------------------------------------
 * createListener()
 *
 * Messaging Functions
 * ---------------------------------------------
 * primeAdminMessage()
 * processAdminMessage()
 * </pre>
 */
public class MQeMessageService
    implements MessageService, MQeMessageListenerInterface, MQeMessageServiceConstants {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    // A reference to the queue manager being wrapped
    private static MQeQueueManager queueManager = null;

    // Reference to GUI/View - this is used both to display status reports and deal with
    // the arrival of new messages
    private static MessageConsumer GUI;

    // Startup parameters for the queue manager
    private MQeMessageServiceParameters startupParms;

    // How long to wait for reply message
    private int waitFor = 30000;

    /**
     *     Check to see if a good reply was received.
     *     If not, detail the error(s) that occured.
     *
     * @param replyMsg Reply message to check
     * @return boolean    - true if good
     * @exception java.lang.Exception
     */
    public boolean checkAdminReply(MQeAdminMsg replyMsg) throws Exception {
        // Was a reply received ?
        if (replyMsg == null) {
            throw new MQeException("No response message received");
        }

        // If the reply was bad then check why
        if (replyMsg.getRC() != MQeAdminMsg.RC_Success) {
            getMessageConsumer().processStatus("[Message Service] Action failed: " + replyMsg.getReason());

            // If mixed then list each error that occured
            if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
                MQeFields errors = replyMsg.getErrorFields();
                Enumeration en = errors.fields();

                System.out.println("ERRORS: " + errors.dumpToString("#0 \t #1\t #2\r\n"));

                // Process each error
                while (en.hasMoreElements()) {
                    String value[] = {};
                    String name = (String) en.nextElement();

                    // Field in error may be an array
                    if (errors.dataType(name) == MQeFields.TypeArrayElements) {
                        value = errors.getAsciiArray(name);
                    } else {
                        // in the case when it is not an array
                        // Ignore individual dynamic array elements, the array is handled
                        // as a whole.
                        if (name.indexOf(MQeFields.ArraySeparator) < 0) {
                            value = new String[] { errors.getAscii(name)};
                        }
                    }

                    for (int j = 0; j < value.length; j++) {
                        getMessageConsumer().processStatus("[Message Service] Field in error: " + name + "  " + value[j]);
                    }
                }
            }
            // Failed - throw exception
            throw new MQeException(replyMsg.getReason());
        }
        return true; // All is OK
    }

    /**
     *     Closes the message service, allowing it to tidy up.
     *     <br>- stop new incoming requests (stop channel listener)
     *     <br>- stop the queue manager.
     *     <br>- close any incoming requests.
     *
     *     <p>Although this only calls closeQMgr, in the future, other things may
     *     also require being closed.
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void close() throws Exception {
        closeQM();
    }

    /**
     *     Closes down the queue manager.
     *
     * @return void
     */
    protected void closeQM() throws Exception {
        if (queueManager != null && queueManager.isQueueManagerActive()) {
            queueManager.closeImmediate();
        }
        queueManager = null;
    }

    /**
     *     Create a new connection definition.
     *
     * @param onQMgr Which QMgr to send the admin message
     * @param name Name of the connection
     * @param adapter Adapter plus base parms to be used
     * @param parms Connection parameters
     * @param options Connection options
     * @return void
     */
    public void createConnection(
        String onQMgr,
        String name,
        String adapter,
        String parms,
        String options)
        throws Exception {
        // Create a new connection admin message supplying reply-to details
        // and set the queue manager that the connection will be going to
        MQeConnectionAdminMsg msg = new MQeConnectionAdminMsg();
        MQeFields msgTest = primeAdminMsg(onQMgr, msg);

        msg.setName(name);

        // Set the admin action to create a new queue
        // The connection is setup to use a default channel. This is an alias
        // which must have be setup on the queue manager for the connection to be
        // valid.
        msg.create(adapter, parms, options, "DefaultChannel", "Route to: " + name);

        // Send the admin message, wait for a reply and check for success.
        // An exception is thrown if it fails.
        MQeAdminMsg respMsg = processAdminMsg(onQMgr, msg);

        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("Connection to " + name + " created");
    }

    /**
     *     Create a new listener and start it running.
     *
     *     <p>This function can be divided into 2 parts
     *     <br> - the creation of the listener
     *     <br> - the starting of the listener
     *
     * @param listenerName The name of listener
     * @return void
     * @exception java.lang.Exception
     */
    public void createListener(String listenerName) throws Exception {
        // Part 1 - Create the listener

        // Create a new listener admin message specifying who to reply to and set it
        // to create a new listener
        MQeCommunicationsListenerAdminMsg msg = new MQeCommunicationsListenerAdminMsg();
        MQeFields msgTest = primeAdminMsg(queueManager.getName(), msg);

        String listenAdapter = getStartupParameters().getListenerAdapter();
        int listenPort = getStartupParameters().getListenerPort();

        msg.setName(listenerName);
        msg.create(listenAdapter, listenPort);

        // Send the admin message, wait for a reply and check for success.
        // An exception is thrown if it fails.
        MQeAdminMsg respMsg = processAdminMsg(queueManager.getName(), msg);

        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("Listener " + listenerName + " created on port " + listenPort);

        // Part 2 - Start the listener

        // Create a new listener admin message specifying who to reply to and set it
        // to start the listener
        msg = new MQeCommunicationsListenerAdminMsg();
        MQeFields msgTest2 = primeAdminMsg(queueManager.getName(), msg);

        msg.setName(listenerName);
        msg.start();

        // Send the admin message, wait for a reply and check for success.
        // An exception is thrown if it fails.
        respMsg = processAdminMsg(queueManager.getName(), msg);
        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("Listener " + listenerName + " has been created and started");
    }

    /**
     *     Creates a home server queue.
     *
     * @param onQMgr Which QMgr to send the admin message
     * @param qm Queue manager the queue resides on
     * @param q Name of the queue
     * @param pollTime The polltime for checking for messages
     *                   <br>0 = do not poll check for messages on call to triggerTransmission()
     * @return void
     * @exception java.lang.Exception
     */
    public void createHomeServerQueue(String onQMgr, String qm, String q, long pollTime)
        throws Exception {
        // Create a queue admin message prime it with who to reply to
        MQeHomeServerQueueAdminMsg msg = new MQeHomeServerQueueAdminMsg();
        MQeFields msgTest = primeAdminMsg(onQMgr, msg);

        msg.setName(qm, q);

        // Create a new MQeFields that will be used to specify the details of the queue
        // e.g. the description
        // Add any characteristics of queue here, otherwise charateristics will be left
        // to default values.
        MQeFields parms = new MQeFields();

        parms.putUnicode(
            MQeQueueAdminMsg.Queue_Description,
            "HomeServer queue " + q + " on queue manager " + qm);
        parms.putLong(MQeHomeServerQueueAdminMsg.Queue_QTimerInterval, pollTime);

        // Set the admin action to create a new queue with the supplied parameters
        // and send it
        msg.create(parms);
        MQeAdminMsg respMsg = processAdminMsg(onQMgr, msg);

        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("Homeserver Queue " + q + " on " + qm + " created");
    }

    /**
     *     Creates a new (simple) local queue with no cryptor or compressor.
     *
     * @param onQMgr The queue manager that the queue is located on
     * @param q The name of the queue
     * @param sync True for a synchronous queue, false for async
     * @return void
     * @exception java.lang.Exception
     */
    public void createLocalQueue(String onQMgr, String q, boolean sync) throws Exception {
        createLocalQueue(onQMgr, q, sync, null, null);
    }

    /**
     *     Creates a new local queue.
     *
     * @param onQMgr The queue manager that the queue is located on
     * @param q The name of the queue
     * @param sync True for a synchronous queue, false for async
     * @param compressor The compressor class to use
     * @param cryptor The cryptor class to use
     * @return void
     */
    public void createLocalQueue(
        String onQMgr,
        String q,
        boolean sync,
        String compressor,
        String cryptor)
        throws Exception {
        // Create a new queue admin message and sest who to reply to and the name of the queue
        // to manage
        MQeQueueAdminMsg msg = new MQeQueueAdminMsg();

        primeAdminMsg(onQMgr, msg);
        msg.setName(q);

        // parms will be used to define the queue so add any characteristics of queue here,
        // otherwise charateristics will be left to default values.
        MQeFields parms = new MQeFields();

        parms.putUnicode(MQeQueueAdminMsg.Queue_Description, "Queue " + q + " on qm " + onQMgr);

        // Add cryptor and/or compressor ir required
        if (compressor != null) {
            parms.putAscii(MQeQueueAdminMsg.Queue_Compressor, compressor);
        }

        if (cryptor != null) {
            parms.putAscii(MQeQueueAdminMsg.Queue_Cryptor, cryptor);
        }

        // Set queue as being accessed synchronously or asynchronously
        if (sync) {
            parms.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Synchronous);
        } else {
            parms.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Asynchronous);
        }

        // Set the admin action to create a new queue with these parameters and send it
        msg.create(parms);
        MQeAdminMsg respMsg = processAdminMsg(onQMgr, msg);

        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("Local queue " + q + " created");
    }

    /**
     *     Creates the queue manager.
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void createQueueManager() throws Exception {
        // Set the directory for the queues on the QM
        String queueDir =
            getStartupParameters().getBaseDir()
            + MQe.fileSeparator()
            + getQMname()
            + MQe.fileSeparator()
            + "queues"
            + MQe.fileSeparator();

        // Create the queue manager
        MQeQueueManagerConfigure qMgrConfig = null;

        try {
            // First create the queue manager
            // deepCopy() is used to prevent MQeQueuManagerConfigure from deleting the
            // PIN from the startup params is things go awry
            // All initial QM properties are set here
            qMgrConfig =
                    new MQeQueueManagerConfigure(
                        (getStartupParameters().getQMparms()).deepCopy(),
                        DEFAULT_MSGLOG + ":" + queueDir);
            qMgrConfig.setDescription("Queue manager " + getQMname());
            //            qMgrConfig.setChnlAttributeRuleName(DEFAULT_CHANNELATTRRULES);
            qMgrConfig.defineQueueManager();

            // and then create the default set of queues
            qMgrConfig.defineDefaultAdminQueue();
            qMgrConfig.defineDefaultAdminReplyQueue();
            qMgrConfig.defineDefaultSystemQueue();
        } finally {
            // Clean up now the queue manager has been configured.
            if (null != qMgrConfig) {
                qMgrConfig.close();
            }
        }
    }

    /**
     *     Creates a new (simple) remote queue with no cryptor of compressor.
     *
     * @param onQMgr Which QMgr to send the admin message
     * @param qm Queue manager the queue resides on
     * @param q Name of the queue
     * @param sync True for synchronous, false for asynchronous
     * @return void
     * @exception java.lang.Exception
     */
    public void createRemoteQueue(String onQMgr, String qm, String q, boolean sync) throws Exception {
        createRemoteQueue(onQMgr, qm, q, sync, null, null);
    }

    /**
     *     Creates a new remote queue.
     *
     * @param onQMgr Which QMgr to send the admin message
     * @param qm Queue manager the queue resides on
     * @param q Name of the queue
     * @param sync True for synchronous, false for asynchronous
     * @param compressor The compressor class to use
     * @param cryptor The cryptor class to use
     * @return void
     * @exception java.lang.Exception
     */
    public void createRemoteQueue(
        String onQMgr,
        String qm,
        String q,
        boolean sync,
        String compressor,
        String cryptor)
        throws Exception {
        // Create an empty queue admin message and set the reply-to name and details of the
        // queue that you're going to be managing
        MQeRemoteQueueAdminMsg msg = new MQeRemoteQueueAdminMsg();

        primeAdminMsg(onQMgr, msg);
        msg.setName(qm, q);

        // Define the parameters of the new queue using an MQeFields object
        MQeFields parms = new MQeFields();

        parms.putUnicode(MQeQueueAdminMsg.Queue_Description, "Remote Queue " + q + " on qm " + onQMgr);

        // Add cryptor and/or compressor if required
        if (compressor != null) {
            parms.putAscii(MQeQueueAdminMsg.Queue_Compressor, compressor);
        }

        if (cryptor != null) {
            parms.putAscii(MQeQueueAdminMsg.Queue_Cryptor, cryptor);
        }

        // Set queue as being accessed synchronously or asynchronously
        if (sync) {
            parms.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Synchronous);
        } else {
            parms.putByte(MQeQueueAdminMsg.Queue_Mode, MQeQueueAdminMsg.Queue_Asynchronous);
        }

        // Set the admin action to create a new queue with these parameters and send it
        msg.create(parms);
        MQeAdminMsg respMsg = processAdminMsg(onQMgr, msg);

        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("Remote queue to " + q + " on " + qm + " created");
    }

    /**
     *     Creates a store and forward queue.
     *
     * @param onQMgr Which QMgr to send the admin message
     * @param qm Queue manager the queue resides on
     * @param q Name of the queue
     * @return void
     * @exception java.lang.Exception
     */
    public void createStoreAndForwardQueue(String onQMgr, String qm, String q) throws Exception {
        // Create an empty queue admin message and specify who to reply to and te details of the
        // queue that you want to manage
        MQeStoreAndForwardQueueAdminMsg msg = new MQeStoreAndForwardQueueAdminMsg();

        primeAdminMsg(onQMgr, msg);
        msg.setName(qm, q);

        // Define the parameters of the new queue using an MQeFields object
        MQeFields parms = new MQeFields();

        parms.putUnicode(MQeQueueAdminMsg.Queue_Description, "Queue " + q + " on qm " + onQMgr);

        // Set the admin action to create a new queue and send the message
        msg.create(parms);
        MQeAdminMsg respMsg = processAdminMsg(onQMgr, msg);

        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("SF Queue to " + q + " on " + qm + " created");
    }

    /**
     *     Deletes all messages from a named queue.
     *
     * @param queueManagerName Queue manager the queue resides on
     * @param queueName Name of the queue
     * @return void
     * @exception java.lang.Exception
     */
    public void emptyQueue(String queueManagerName, String queueName) throws Exception {
        // Get a list of all messages on the queue
        MQeEnumeration recs = queueManager.browseMessages(queueManagerName, queueName, null, null, true);

        // Take each message in the list and delete it
        while (recs.hasMoreElements()) {
            MQeFields match = (MQeFields) recs.nextElement();

            queueManager.deleteMessage(queueManagerName, queueName, match);
        }
    }

    /**
     *     Get a message from a given queue manager/queue.
     *
     * @param queueManagerName Which QMgr to get the message from
     * @param queueName Which queue to get the message from
     * @param filter Get a message that matches the filter
     * @return MQeMsgObject
     * @exception java.lang.Exception
     */
    public MQeMsgObject getMessage(String queueManagerName, String queueName, MQeFields filter)
        throws Exception {
        return getSecureMessage(queueManagerName, queueName, filter, null);
    }

    /**
     *     Returns a reference to the GUI object that handles consumption of messages.
     *
     * @return MessageConsumer
     */
    public MessageConsumer getMessageConsumer() {
        return GUI;
    }

    /**
     *     Gets the name of the local queue manager.
     *
     * @return String
     * @exception java.lang.Exception
     */
    public String getQMname() throws Exception {
        return getStartupParameters().getQMname();
    }

    /**
     *     Gets a secure message from a given queue manager/queue.
     *
     * @param queueManagerName Which QMgr to get the message from
     * @param queueName Which queue to get the message from
     * @param filter Get a message that matches the filter
     * @param attribute The attribute used to decode the secure message
     * @return MQeMsgObject
     * @exception java.lang.Exception
     */
    public MQeMsgObject getSecureMessage(
        String queueManagerName,
        String queueName,
        MQeFields filter,
        MQeAttribute attribute)
        throws Exception {
        MQeMsgObject newMsg = null;

        try {
            // Get the message from the queue manager
            newMsg = queueManager.getMessage(queueManagerName, queueName, filter, attribute, 0);
        } catch (Exception e) {
            // If the get failed for any reason other than because the message doesnt exist,
            // throw the problem up
            if (!e.toString().equals("com.ibm.mqe.MQeException: Message not found")) {
                throw e;
            }
        }
        return newMsg;
    }

    /**
     *     Returns the startup parameters given to the message service when it was initialized.
     *
     * @return MQeMessageServiceParameters
     */
    public MQeMessageServiceParameters getStartupParameters() throws Exception {
        if (startupParms == null) {
            throw new Exception("no startup parameters have been set");
        }

        return startupParms;
    }

    /**
     *     Processes a new message event. A new message has arrived on a queue
     *     that is being monitored, tell the GUI about it.
     *
     * @param e The event that has occurred
     * @return void
     */
    public void messageArrived(MQeMessageEvent e) {
        System.out.println("messageArrived");
        try {
            // Get the UID of the message
            MQeFields match = e.getMsgFields();

            // now browse the queue to get the message with the UID.
            // ( the browse method will read the entire message
            // and pass it to the GUI )
            processMessages(e.getQueueManagerName(), e.getQueueName(), match);
        } catch (Exception ex) {
            getMessageConsumer().processStatus("MessageArrived: dodgy message " + ex);
        }
    }

    /**
     *     Setup a monitor that will check for new messages on a queue.
     *     If this is a local queue then a message listener can
     *     be used to automatically monitor for new messages. If
     *     this is a remote queue then the remote queue will have to be
     *     polled either automatically or by user request to see if there
     *     are any new messages.
     *
     *     <p>NOTE:   monitoring of remote queues is not yet implemented!
     *
     * @param queueManagerName Queue manager the queue resides on
     * @param queueName Name of the queue
     * @return void
     */
    public void monitorQueue(String queueManagerName, String queueName) {
        try {
            // Add a message listener if the queue is local.
            if (queueManager.getName().equals(queueManagerName)) {
                queueManager.addMessageListener(this, queueName, null);
                getMessageConsumer().processStatus("Queue monitor for queue: " + queueName + " started");
            }

            // Display any existing messages in the queue
            // Note: there is a small window where a new message arrives before
            // the browse hence will be processed twice, once by the message listener
            // and once by the browse!
            processMessages(queueManagerName, queueName, null);
        } catch (Exception e) {
            getMessageConsumer().processStatus("Error browsing queue: " + queueName + " " + e.toString());
        }
    }

    /**
     *     Performs the processing needed to activate the system.
     *     Starts the local queue manager (and configures it if
     *     it is not already configured).
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void open() throws Exception {
        try {
            // Only one queue manager per JVM is allowed
            if (queueManager != null) {
                throw new Exception("Queue manager already running");
            }

            // Process the alias section - this will set up aliases that may be required
            // by the queue manager
            processAlias();

            // Start the queue manager
            queueManager = new MQeQueueManager();
            if (queueManager != null) {

                /*
                 * If the registry does not exist, this will throw an exception.
                 * This will be handled by the powers upstairs, which will initialize
                 * the queue manager and then run open() again.
                 * i.e. On its first run, this will throw an exception, but once the
                 * registry exists, there will be no problems
                 *
                 * deepCopy is used here to prevent the queue manager from removing the PIN
                 * from the parameters
                 */
                System.out.println("qm activate");
                queueManager.activate((getStartupParameters().getQMparms()).deepCopy());
            }
        } catch (Exception ex) {
            System.out.println("[MQeMessageService] Start of queue manager failed: " + ex);
            queueManager = null;
            throw ex;
        }
    }

    /**
     *     Prime an admin message. Sets the admin message with items such as
     *     who to reply to along with a unique identifier that can be used to
     *     check for a matching response.
     *
     * @param onQMgr The name of the queue manager where the message will be sent.
     * @param msg The admin message to prime
     * @return MQeFields  - a unique identifier for this request
     * @exception java.lang.Exception
     */
    public MQeFields primeAdminMsg(String onQMgr, MQeAdminMsg msg) throws Exception {
        // Set the target queue manager that will process this message, who to reply to
        // and the 'style' of the message to be a request
        msg.setTargetQMgr(onQMgr);
        msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);
        msg.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        msg.putAscii(MQe.Msg_ReplyToQMgr, onQMgr);

        //  Setup the correl id so we can match the reply to the request
        msg.putArrayOfByte(MQe.Msg_CorrelID, Long.toString(MQe.uniqueValue(), 16).getBytes());

        // Ensure matching response message is retrieved
        MQeFields msgTest = new MQeFields();

        msgTest.putArrayOfByte(MQe.Msg_CorrelID, msg.getArrayOfByte(MQe.Msg_CorrelID));

        // Return the unique filter for this message
        return msgTest;
    }

    /**
     *     Send the admin message, wait for a reply and check for success.
     *     throws an exception if there is an error.
     *
     * @param onQMgr The name of the queue manager where the message will be sent.
     * @param msg The admin message to send
     * @return MQeAdminMsg - the response to the message
     * @exception java.lang.Exception
     */
    public MQeAdminMsg processAdminMsg(String onQMgr, MQeAdminMsg msg) throws Exception {
        // Setup the match field to check for a matching reply message.
        // The correl id is used as the match parameter.
        MQeFields msgTest = new MQeFields();

        msgTest.putArrayOfByte(MQe.Msg_CorrelID, msg.getArrayOfByte(MQe.Msg_CorrelID));

        //  Put the admin message to the admin queue and return the reply
        queueManager.putMessage(onQMgr, MQe.Admin_Queue_Name, msg, null, 0);
        MQeAdminMsg respMsg = (MQeAdminMsg) waitForReply(onQMgr, MQe.Admin_Reply_Queue_Name, msgTest);

        return respMsg;
    }

    /**
     *     Process the class alias names in the configuration data.
     *
     *     <p>This maps one class name to another, or acts as an alias to a class name.
     * <pre>
     * [Alias]
     *      (ascii)EventLog=.....        class to use to write event logs
     *      (ascii)Network=.....         Adapter class for network support
     *      (ascii)QueueManager=.....    class to use for Queue manager
     *      (ascii)Registry=.....        class to use for Registry functions
     *      (ascii)Trace=.....           class to provide trace functions
     * </pre>
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void processAlias() throws Exception {
        // Get the alias section of the startup parameters and, if it's there, set up the
        // aliases it specifies
        MQeFields section = getStartupParameters().getSection(getStartupParameters().Section_Alias);

        if (section != null) {
            Enumeration keys = section.fields();

            while (keys.hasMoreElements()) {
                // For each key, get the value and add the mapping to the MQe
                // internal alias table
                String key = (String) keys.nextElement();

                MQe.alias(key, section.getAscii(key).trim());
            }
        }
    }

    /**
     *     Process any messages that match a filter on a given queue.
     *
     *     <p>A very simplistic approach is taken to handling any messages found:
     *     If the message is a request then the processMessage() method is called on
     *     the GUI, if it is a reply or datagram or not specified then the displayMessage
     *     method is called on the GUI.
     *
     *     <p>NOTE: The message is only browsed in this method so the message is left on
     *     the queue it is down to anyone processing the message to get/delete it on
     *     the queue!
     *
     * @param queueManagerName Which QMgr to browse
     * @param queueName Which queue to browse
     * @param match Only return fields matching
     * @return void
     */
    public void processMessages(String queueManagerName, String queueName, MQeFields match) {
        try {
            // Get an enumeration of messages that match the filter and process them
            MQeEnumeration recs = queueManager.browseMessages(queueManagerName, queueName, match, null, false);

            while (recs.hasMoreElements()) {
                // Get a message
                MQeMsgObject msg = (MQeMsgObject) recs.nextElement();

                // Get the messageConsumer to deal with the message depending on their type
                // (request, reply, datagram etc)
                if (msg.contains(MQe.Msg_Style) && msg.getInt(MQe.Msg_Style) == MQe.Msg_Style_Request) {
                    getMessageConsumer().processRequestMsg(msg, queueManagerName, queueName);
                } else if (msg.contains(MQe.Msg_Style) && msg.getInt(MQe.Msg_Style) == MQe.Msg_Style_Reply) {
                    getMessageConsumer().processReplyMsg(msg, queueManagerName, queueName);
                } else {
                    getMessageConsumer().processDataGramMsg(msg, queueManagerName, queueName);
                }
            }
        } catch (Exception e) {
            // If an error occurs let the GUI know
            getMessageConsumer().processStatus("Browse failed: " + e);
        }
    }

    /**
     *     Send a message to a given queue manager/queue.
     *
     * @param queueManagerName Which QMgr to send the message
     * @param queueName Which queue to send the message to
     * @param message The message to send
     * @return void
     * @exception java.lang.Exception
     */
    public void sendMessage(String queueManagerName, String queueName, MQeMsgObject message)
        throws Exception {
        sendSecureMessage(queueManagerName, queueName, message, null);
    }

    /**
     *     Send a secure message to a given queue manager/queue. This is a message that
     *     requires a cryptor or compressor.
     *
     * @param queueManagerName Which QMgr to send the message
     * @param queueName Which queue to send the message to
     * @param message The message to send
     * @param MQeAttribute - The attribute used to secure the message
     * @return void
     * @exception java.lang.Exception
     */
    public void sendSecureMessage(
        String queueManagerName,
        String queueName,
        MQeMsgObject message,
        MQeAttribute attribute)
        throws Exception {
        // Hand the message over to the queue manager for processing
        queueManager.putMessage(queueManagerName, queueName, message, attribute, 0);
    }

    /**
     *   Let the message service know the graphical component that can display messages.
     *
     * @param GUI The message consumer to be set to
     * @return void
     */
    public void setMessageConsumer(MessageConsumer GUI) {
        this.GUI = GUI;
    }

    /**
     *  Set the object that contains any parameters required to start
     *  the message service.
     *
     * @param parms The name of the local queue manager
     */
    public void setStartupParameters(MQeMessageServiceParameters parms) {
        startupParms = parms;
    }

    /**
     *     Change the characteristics of a queue manager.
     *
     * @param onQMgr The name of the queue manager where the admin message will be sent.
     * @param parms The parameters that need updating
     * @return void
     * @exception java.lang.Exception
     */
    public void updateQueueManager(String onQMgr, MQeFields parms) throws Exception {
        // Create a queue admin message and tell it who to reply to, which QM you want
        // to manage and that you want to be updating the QM with the given parameters
        MQeQueueManagerAdminMsg msg = new MQeQueueManagerAdminMsg();
        MQeFields msgTest = primeAdminMsg(onQMgr, msg);

        msg.setName(onQMgr);
        msg.update(parms);

        // Send the admin message, wait for a reply and check for success.
        // An exception is thrown if it fails.
        MQeAdminMsg respMsg = processAdminMsg(onQMgr, msg);

        checkAdminReply(respMsg);

        getMessageConsumer().processStatus("Queue manager update successful");
    }

    /**
     *     Wait a while for a message to turn up on a queue.
     *
     * @param onQMgr Queue manager to wait on
     * @param onQueue Queue to wait on
     * @param msgTest Filter for a specific message
     * @return MQeMsgObject - the reply message
     * @exception java.lang.Exception
     */
    public MQeMsgObject waitForReply(String onQMgr, String onQueue, MQeFields msgTest)
        throws Exception {
        MQeMsgObject respMsg = null;
        int maxWaitTime = waitFor;

        // Wait until you receive a response or you run out of time
        while (respMsg == null && maxWaitTime > 0) {
            try {
                respMsg = queueManager.getMessage(onQMgr, onQueue, msgTest, null, 0);
            } catch (MQeException e) {
                if (e.code() != MQe.Except_Q_NoMatchingMsg) {
                    // if the exception is 'no matching message' then ignore it.  This
                    // will result in a null return value.
                    // Rethrow all other exceptions.
                    throw e;
                }
            }
            Thread.sleep(100);
            maxWaitTime -= 100;
        }
        return respMsg;
    }
}
