package examples.midp.exampleapp.messageservice;


/*----------------------------------------------------------------------------*/

/*                  WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.registry.MQeRegistry;


/**
 * An MQe queue manager requires a set of parameters in order to be started.
 * These parameters have been encapsulated within this class.
 *
 * <p>The parameters required by a queue manager are stored in an MQeFields object
 * and fall into the following 3 groups:
 * <pre>
 * QueueManager
 *  Information on the queue manager
 *    - queue manager's name
 *    - queue manager's description
 *
 * Registry
 *  Holds information about the registry
 *    - a path to the location of the registry
 *    - whether is it public or private
 *    - the registry's attributes (e.g. the PIN for a private registry)
 *
 * ChannelManager
 *  Contains information specific to the channel manager
 *    - channel timeout
 *    - max number of channels
 *
 * Default paramaters are set in buildDefaultParms().
 *
 * -------------------------------------------------------------
 *
 * Alias
 *  Contains aliases for classes so, for example,
 *  "com.ibm.mqe.adapters.MQeDiskFieldsAdapter" can be referred to as "Adapter".
 *  This is the equivalent of the [alias] section found in an ini file.
 * </pre>
 */
public class MQeMessageServiceParameters implements MQeMessageServiceConstants {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    // Parameters passed to a queue manager containing the information it required to start
    private MQeFields QMstartupParms;

    // Root directory for the queue manager
    private String rootDir;

    // Parameters for the channel manger
    public final static String Section_ChannelManager = "ChannelManager";

    // Parameters for Listeners
    public final static String Section_Listener = "Listener";

    // Aliases (equivalent to the [Alias] section in an ini file)
    public final static String Section_Alias = "Alias";

    // Queue manager registry
    public final static String Section_Registry = "Registry";

    /**
     *     Setup a default set of parameters needed to start a queue manager running in
     *     either client or server mode. This method only need be called if the parameters were
     *     not passed using an alternative means like an ini file.
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void buildDefaultParms() throws Exception {
        // First setup aliases - equivalent to the [Alias] section in an ini file
        setAlias("EventLog", DEFAULT_EVENTLOG);
        setAlias("Network", DEFAULT_NETWORK);
        setAlias("FastNetwork", DEFAULT_FASTNETWORK);
        setAlias("QueueManager", DEFAULT_QUEUEMANAGER);
        setAlias("MsgLog", DEFAULT_MSGLOG);
        setAlias("FileRegistry", DEFAULT_FILEREGISTRY);
        setAlias("PrivateRegistry", DEFAULT_PRIVATEREGISTRY);
        setAlias("DefaultChannel", DEFAULT_CHANNEL);
        setAlias("DefaultTransporter", DEFAULT_TRANSPORTER);
        setAlias("ChannelAttrRules", DEFAULT_CHANNELATTRRULES);

        // the channel manager and listener sections
        setListenerAdapter(DEFAULT_LISTENERADAPTER);
        setListenerPort(DEFAULT_LISTENERPORT);
        setChannelTimeout(DEFAULT_TIMEINTERVAL);
        setMaxChannels(DEFAULT_MAXCHANNELS);

        // Finally the registry
        // set the type of registry to be public (most cryptors need a private one)
        setRegistryTypePublic();
        setRegistryDir(
            getBaseDir()
            + MQe.fileSeparator()
            + getQMname()
            + MQe.fileSeparator()
            + "Registry"
            + MQe.fileSeparator());
    }

    /**
     *     Returns a named section from the parameters.
     *
     * @param section The name of the section to return (use one of the section constants)
     * @return MQeFields
     */
    protected MQeFields getSection(String section) {
        // If the section exists then return it, otherwise create it
        MQeFields outFields = null;

        try {
            if (!getQMparms().contains(section)) {
                outFields = new MQeFields();
                getQMparms().putFields(section, outFields);
            } else {
                outFields = getQMparms().getFields(section);
            }
        } catch (Exception e) {}

        return outFields;
    }

    /**
     *     Returns the root directory of the queue manager.
     *
     * @return String
     */
    protected String getBaseDir() {
        if (rootDir == null) {
            rootDir = DEFAULT_BASEDIRNAME;
        }

        return rootDir;
    }

    /**
     *     Returns the timeout value of the channel.
     *
     * @return int
     */
    protected int getChannelTimeout() throws Exception {
        return getSection(Section_Listener).getInt("TimeInterval");
    }

    /**
     *     Returns the listener adapter to be used.
     *
     * @return String
     */
    protected String getListenerAdapter() throws Exception {
        return getSection(Section_Listener).getAscii("Network");
    }

    /**
     *     Returns the port that the listener should listen on.
     *
     * @return int
     */
    protected int getListenerPort() throws Exception {
        String listen = getSection(Section_Listener).getAscii("Listen");

        return Integer.valueOf(listen).intValue();
    }

    /**
     *     Returns the maximum number of channels allowed.
     *
     * @return int
     */
    protected int getMaxChannels() throws Exception {
        return getSection(Section_ChannelManager).getInt("MaxChannels");
    }

    /**
     *     Returns the queuemanager's startup parameters or a clean MQeFields.
     *
     * @return MQeFields
     */
    public MQeFields getQMparms() {
        if (QMstartupParms == null) {
            QMstartupParms = new MQeFields();
        }
        return QMstartupParms;
    }

    /**
     *     Returns the name of the local queue manager.
     *
     * @return String
     */
    protected String getQMname() {
        String qmName = null;

        if (getSection(MQeQueueManager.QueueManager).contains(MQeQueueManager.Name)) {
            try {
                qmName = getSection(MQeQueueManager.QueueManager).getAscii(MQeQueueManager.Name);
            } catch (Exception e) {}
        }
        return qmName;
    }

    /**
     *     Add an alias to the parameters.
     *
     * @param alias The alias
     * @param value The value of the alias
     * @return void
     */
    public void setAlias(String alias, String value) {
        try {
            getSection(Section_Alias).putAscii(alias, value);
        } catch (Exception e) {
            System.out.println("setAlias unexpected exception " + e);
        }
    }

    /**
     *     Sets an attribute in the registry section
     *
     * @param alias The alias
     * @param value The value of the alias
     * @return void
     */
    protected void setRegistryAttribute(String alias, String value) {
        try {
            getSection(Section_Registry).putAscii(alias, value);
        } catch (Exception e) {
            System.out.println("setRegAlias unexpected exception " + e);
        }
    }

    /**
     *     Sets the base directory.
     *
     * @param baseDir The root dir of the queue manager
     * @return void
     */
    public void setBaseDir(String baseDir) {
        rootDir = baseDir;
    }

    /**
     *     Sets the timeout value of the channel.
     *
     * @param timeout The channel timeout
     * @return void
     * @exception java.lang.Exception
     */
    public void setChannelTimeout(int timeout) throws Exception {
        getSection(Section_Listener).putInt("TimeInterval", timeout);
    }

    /**
     *     Sets the adapter to be used by the listener.

     *     Normally, an ini file has a : at the end of the adapter name (but this doesnt).
     *
     * @param adapter The adapter to listen with
     * @return void
     * @exception java.lang.Exception
     */
    public void setListenerAdapter(String adapter) throws Exception {
        getSection(Section_Listener).putAscii("Network", adapter);
    }

    /**
     *     Sets the port that the listener will be listening on.
     *
     * @param port The port to listen on
     * @return void
     * @exception java.lang.Exception
     */
    public void setListenerPort(int port) throws Exception {
        if (getListenerAdapter() == null) {
            throw new Exception("Cannot set port until listener adapter is set");
        }

        String strPort = new Integer(port).toString();

        getSection(Section_Listener).putAscii("Listen", strPort);
    }

    /**
     *     Set the maximum number of channels allowed.
     *
     * @param maxChannels The maximum number of concurrent channels
     * @return void
     * @exception java.lang.Exception
     */
    public void setMaxChannels(int maxChannels) throws Exception {
        getSection(Section_ChannelManager).putInt("MaxChannels", maxChannels);
    }

    /**
     *     Sets the name of the local queue manager and then setup the startup
     *     parameters that are used to start the named queue manager.
     *
     *     <p>Alternatively the setQMgrIniFile method can be used to obtain the
     *     queue manager name and startup parms from an init file.
     *
     * @param qmgrName The new name of the local queue manager
     * @return void
     * @exception java.lang.Exception
     */
    public void setQMname(String qmgrName) throws Exception {
        if (qmgrName == null || qmgrName.length() < 1) {
            throw new Exception("queue manager name not specified");
        }

        getSection(MQeQueueManager.QueueManager).putAscii(MQeQueueManager.Name, qmgrName);

        buildDefaultParms();
    }

    /**
     *     Sets the directory that the registry is located in.
     *
     * @param registryDir The registry dir
     * @return void
     * @exception java.lang.Exception
     */
    public void setRegistryDir(String registryDir) throws Exception {
        getSection(MQeQueueManager.Registry).putAscii(MQeRegistry.DirName, registryDir);
    }

    /**
     *     Sets the type of registry that will be used.
     *
     * @param registryType The type of registry
     */
    protected void setRegistryType(String registryType) throws Exception {
        getSection(MQeQueueManager.Registry).putAscii(MQeRegistry.LocalRegType, registryType);
    }

    /**
     *     Sets the registry to be a private registry and sets the PIN required by the registry.
     * <p> Most cryptors require you to have a private registry.
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void setRegistryTypePrivate() throws Exception {
        setRegistryType(MQeRegistry.PrivateRegistry);
        setRegistryAttribute(MQeRegistry.PIN, DEFAULT_PIN);
    }

    /**
     *     Sets the registry to be a public registry.
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void setRegistryTypePublic() throws Exception {
        setRegistryType(MQeRegistry.FileRegistry);
    }
}
