package examples.midp.exampleapp.messageservice;


/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/

import com.ibm.mqe.MQeAttribute;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;


/**
 * This acts as a frontend to WMQe, wrapping a queuemanager and performing functions
 * such as queue creation and message sending. This is the core of the examples and
 * allows the apps to be written with minimal calls to the WMQe API. This also means
 * that to see the code required to create a local queue for example, a user can
 * simply look at: <b>createLocalQueue()</b>.
 *
 * <p>Functions can be split up as follows:</p>
 *
 * <pre>
 * Message Service Functions
 * ---------------------------------------------
 * open()
 * close()
 * setMessageConsumer()
 *
 * Queue Manager Functions
 * ---------------------------------------------
 * createQueueManager()
 * updateQueueManager()
 * deleteQueueManager()
 * createConnection()
 *
 * Queue Functions
 * ---------------------------------------------
 * createLocalQueue()
 * createRemoteQueue()
 * createStoreAndForwardQueue()
 * createHomeServerQueue()
 * monitorQueue()
 * emptyQueue()
 *
 * Messaging Functions
 * ---------------------------------------------
 * sendMessage()
 * sendSecureMessage()
 * getMessage()
 * getSecureMessage()
 * processMessages()
 * waitForReply()
 * </pre>
 */

public interface MessageService {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    /**
     *     When an application wishes to stop, this method must be called
     *     to allow the model to tidy up. For instance to shut down the
     *     queue manager.
     *
     * @return void
     * @exception java.lang.Exception
     */
    void close() throws Exception;

    /**
     *     Create a new connection to a queue manager.
     *
     * @param onQMgr Queue manager to send the admin message
     * @param name Name of the connection
     * @param adapter Adapter plus base parms to be used
     * @param parms Connection parameters
     * @param options Connection options
     * @return void
     * @exception java.lang.Exception
     */
    public void createConnection(
        String onQMgr,
        String name,
        String adapter,
        String parms,
        String options)
        throws Exception;

    /**
     *     Create a home server queue.
     *
     * @param onQMgr Which QMgr to send the admin message
     * @param qm Queue manager the queue resides on
     * @param q Name of the queue
     * @param pollTime The polltime for checking for messages
     *                  0 - do not poll check for messages on
     *                      call to triggerTransmission()
     * @return void
     * @exception java.lang.Exception
     */
    public void createHomeServerQueue(String onQMgr, String qm, String q, long pollTime)
        throws Exception;

    /**
     *     Create a new local queue.
     *
     * @param onQMgr Queue manager the queue will be on
     * @param q Queue name
     * @param sync True for synchronous, false for async
     * @param compressor The compressor to use
     * @param cryptor The cryptor to use
     * @exception java.lang.Exception
     */
    public void createLocalQueue(
        String onQMgr,
        String q,
        boolean sync,
        String compressor,
        String cryptor)
        throws Exception;

    /**
     *     Creates the queue manager.
     *     <p>Part of the programatic creation of a queue manager.
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void createQueueManager() throws Exception;

    /**
     *     Create a new remote queue.
     *
     * @param onQMgr The QMgr to send the admin message
     * @param qm Queue manager the queue resides on
     * @param q Name of the queue
     * @param sync True for synchronous, false for asynchronous
     * @param compressor The name of the compressor to use, or null for none
     * @param cryptor The name of the cryptor to use, or null for none
     * @return void
     * @exception java.lang.Exception
     */
    public void createRemoteQueue(
        String onQMgr,
        String qm,
        String q,
        boolean sync,
        String compressor,
        String cryptor)
        throws Exception;

    /**
     *     Create a store and forward queue.
     *
     * @param onQMgr Which QMgr to send the admin message
     * @param qm Queue manager the queue resides on
     * @param q Name of the queue
     * @return void
     * @exception java.lang.Exception
     */
    public void createStoreAndForwardQueue(String onQMgr, String qm, String q) throws Exception;

    /**
     *      This method is called when a requestor wishes to delete all messages
     *      from a queue.
     *
     * @param queueManagerName Queue manager that queue is located on
     * @param queueName Name of queue
     * @return void
     * @exception java.lang.Exception
     */
    void emptyQueue(String queueManagerName, String queueName) throws Exception;

    /**
     *     Get a message from a given queue manager/queue.
     *
     * @param queueManagerName Which QMgr to get the message from
     * @param queueName Which queue to get the message from
     * @param filter Get a message that matches the filter
     * @return MQeMsgObject  message
     * @exception java.lang.Exception
     */
    public MQeMsgObject getMessage(String queueManagerName, String queueName, MQeFields filter)
        throws Exception;

    /**
     *     Get a secure message from a given queue manager/queue.
     *
     * @param queueManagerName Which QMgr to get the message from
     * @param queueName Which queue to get the message from
     * @param filter Get a message that matches the filter
     * @param attribute The attribute used to decode the secure message
     * @return MQeMsgObject  message
     * @exception java.lang.Exception
     */
    public MQeMsgObject getSecureMessage(
        String queueManagerName,
        String queueName,
        MQeFields filter,
        MQeAttribute attribute)
        throws Exception;

    /**
     *     Montior the named queue for new messages as a result of which a
     *     view components displayMessage method will be called.
     *
     *     <p>If the queue is a local queue then a message listener can
     *     be used to automatically monitor for new messages. If it is
     *     a remote queue then message listeners cannot be used and an
     *     alternative approach needs to be found for instance the remote
     *     queue could be polled either automatically or by user request
     *     to see if there are any new messages.
     *
     * @param queueManagerName Queue manager name
     * @param queueName Queue name
     * @return void
     */
    void monitorQueue(String queueManagerName, String queueName);

    /**
     *     Performs processing needed to activate the system.
     *     <p>For Example,  start/configure the system.
     *
     * @return void
     * @exception java.lang.Exception
     */
    public void open() throws Exception;

    /**
     *     This method reads messages from a queue that match a filter.
     *     Any messages that match the filter are then passed to the GUI for display.
     *
     * @param queueManager Name which QMgr to browse
     * @param queueName Which queue to browse
     * @param match Only return fields matching
     * @return void
     */
    public void processMessages(String queueManagerName, String queueName, MQeFields match);

    /**
     *     Send a message to a queue on a specific queue manager.
     *
     * @param queueManagerName The queue manager that queue is on
     * @param queueName The queue to put message on
     * @param message The message to send
     * @return void
     * @exception java.lang.Exception
     */
    void sendMessage(String queueManagerName, String queueName, MQeMsgObject message) throws Exception;

    /**
     *     Send a secure message to a queue on a specific queue manager.
     *
     * @param queueManagerName The queue manager that queue is located on
     * @param queueName The queue that message is going to
     * @param message The message to send
     * @param MQeAttribute Attributes of message
     * @exception java.lang.Exception
     */
    void sendSecureMessage(
        String queueManagerName,
        String queueName,
        MQeMsgObject message,
        MQeAttribute attribute)
        throws Exception;

    /**
     *     Provide the model with an instance of a message consumer that can
     *     process messages. I.e. a class that implements the AppGUIInterface.
     *
     * @param msgConsumer The message consumer to be set
     * @return void
     */
    public void setMessageConsumer(MessageConsumer msgConsumer);

    /**
     *     Change the characteristics of a queue manager.
     *
     * @param onQMgr A String containing the name of the queue manager
     * where the admin message will be sent.
     * @param parma An MQeFields Object. The parameters that need updating.
     * @return void
     * @exception java.lang.Exception
     */
    public void updateQueueManager(String onQMgr, MQeFields parms) throws Exception;

    /**
     *     Wait a while for a message to turn up on a queue.
     *
     * @param onQMgr Wait for message on queue manager
     * @param onQueue Wait for message on queue
     * @param msgTest Filter for a specific message
     * @return MQeMsgObject The reply
     * @exception java.lang.Exception
     */
    public MQeMsgObject waitForReply(String onQMgr, String onQueue, MQeFields msgTest)
        throws Exception;
}
