package examples.midp.exampleapp.msgpump;


/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.RecordStore;


/**
 * This class acts as both controller and user interface for a MIDP pump client.
 *
 * <p>Textboxes and Forms are used to interact with users as well as display information
 * about the current state of the client.
 *
 * <p>The client can be viewed as being in one of the following states:
 *
 * <br> - Initializing      - in the process of initializing the client
 * <br> - WaitingForQMname  - occurs while initializing and waiting for the user to
 *                            enter the name of the QM
 * <br> - Running           - in the process of pumping messages
 * <br> - DisplayingStatus  - displaying the status of the client to the user
 * <br> - DisplayingOptions - displaying the options screen to the user
 * <br> - ShuttingDown      - in the process of shutting down
 *
 * <p>The following jad properties are available:
 * <pre>
 *      Pump_SecurityLevel
 *        Specifies the security level that the app should use.
 *          0 for no security
 *          1 for message based security
 *          2 for queue based security
 *
 *      Pump_ServerQueue
 *        Specifies the name of the queue that messages should be sent to.
 *
 *      Pump_ServerIP
 *        Specifies the IP of the server that will sent messages.
 *
 *      Pump_ServerPort
 *        Specifies the port that the server will be listening on.
 *
 *      Pump_ServerQueueManager
 *        Specifies the name of the queue manager that messages will be sent to.
 * </pre>
 */


public class MidpClient extends MIDlet
    implements CommandListener, MsgPumpController {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    // The pump model associated with the controller
    protected MsgPumpModel msgPump = null;

    // The thread used for the model. This allows user interaction with the system while
    // pump remains active
    protected Thread thread = null;

    // Shows whether the app has been started or not.
    // Used so that when startApp() is called, the class can tell if it is
    // starting for the first time or merely resuming after being paused
    private boolean started = false;

    // This MIDlet's display
    private Display display = null;

    // The main screen shown to the user
    private TextBox text;

    // Form showing the status of the pump
    private TextBox statusForm;

    // Form allowing the user to change options
    private Form optionForm;

    // Form prompting for the QM name
    private Form namePrompt;

    // The amount of time the client model should wait between sending messages
    private TextField waittime;

    // Used to prompt for the name of the queue manager
    private TextField QMPrompt;

    // Some buttons for the user to click on
    private static Command pause = new Command("Pause", Command.SCREEN, 0);
    private static Command resume = new Command("Resume", Command.SCREEN, 0);
    private static Command options = new Command("Options", Command.SCREEN, 0);
    private static Command status = new Command("Status", Command.SCREEN, 0);
    private static Command ok = new Command("Ok", Command.OK, 0);

    /**
     *   This class initializes the controller.
     *
     *   <p>Initialization involves trying to get the name of the queue manager from
     *   the RMS store. If no name is found, the user is prompted for one.
     *
     *   <p>When the name is received, control will continue from commandAction()
     *
     * @return void
     */
    public void initialize() throws Exception {
        String localQMname = getQMname();

        if (localQMname != null) {
            acquiredQMName(localQMname);
        } else {
            promptForQMName();
        }
    }

    /**
     *   Completely removes the queue manager from the system. This involves
     *   indiscriminately purging all record stores that the MIDlet / MIDlet
     *   suite has access to.
     *
     *   <p>This will of course mean that if you have both the textapp and the
     *   message pump in the same MIDlet suite, wiping one queue manager will
     *   also remove the other.
     *
     * @return void
     */
    public void deleteQueueManager() {
        try {
            getModel().getMsgService().close();
        } catch (Exception e) {}
        // Get list of all recordstores in suite and delete
        String[] qs = javax.microedition.rms.RecordStore.listRecordStores();

        if (qs != null) {
            for (int i = 0; i < qs.length; i++) {
                try {
                    javax.microedition.rms.RecordStore.deleteRecordStore(qs[i]);
                } catch (Exception e) {}
            }
        }
    }

    /**
     *   Checks the local record stores for signs of a client having been
     *   run previously. If this is the case, the name of the client is
     *   returned, otherwise null.
     * @return String - the name of the queue manager
     * @see MsgPumpController#getQMname
     */
    public String getQMname() {
        String stores[] = RecordStore.listRecordStores();
        String current;

        if (stores != null) {
            for (int i = 0; i < stores.length; ++i) {
                current = new String(stores[i]);
                if (current.startsWith("QMname-")) {
                    return current.substring(7);
                }
            }
        }

        /* QM name could not be found */
        return null;
    }

    /**
     *    Returns the model or, if one doesnt currently exist, create a new one.
     *
     * @return MsgPumpModel - the model
     */
    public MsgPumpModel getModel() {
        if (msgPump == null) {
            msgPump = new MsgPumpModelClient();
        }
        return msgPump;
    }

    /**
     *    Processes a message to be displayed.
     *
     * @param String - the message
     * @return void
     */
    public void processMsg(String msg) {
        displayMsg(msg);
    }

    /**
     *    Starts the controller running - this involves putting the model in a new
     *    thread and starting it.
     *
     * @return void
     */
    public void start() throws Exception {
        processMsg("[Controller] Starting model...");
        Thread thread = new Thread(getModel());

        thread.start();
    }

    /**
     *    For the Midpclient, messages are displayed by updating the output screen.
     * @see MsgPumpController#displayMsg
     */
    public void displayMsg(String msg) {
        System.out.println(msg);
        getOutputScreen().setString(msg);
    }

    /**
     *   This writes the name of the QM to a record store, allowing it to be used
     *   in the future.
     *
     * @param String - the name of the queue manager
     * @return void
     */
    protected void setQMName(String name) {
        try {
            RecordStore temp = RecordStore.openRecordStore("QMname-" + name, true);

            temp.closeRecordStore();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *   This function is run once the name of the queuemanager has been acquired.
     *   It continues the initialization of the controller.
     *
     * @param String - name of the queue manager
     * @return void
     */
    protected void acquiredQMName(String name) {
        setQMName(name);
        try {
            changeDisplay(getOutputScreen());
            getModel().initialize(this, name);
            start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Constructor for MidpClient
     */
    public MidpClient() {
        super();
    }

    /**
     * See javax.microedition.midlet#startApp.
     */
    public void startApp() throws MIDletStateChangeException {
        if (!started) {
            // Initial starting point
            initializeMidlet();
        } else {
            // Resume the pump
            getModel().resume();
        }
    }

    /**
     * See javax.microedition.midlet#pauseApp.
     */
    public void pauseApp() {
        getModel().pause();
    }

    /**
     * See javax.microedition.midlet#destroyApp.
     */
    public void destroyApp(boolean arg0) throws MIDletStateChangeException {
        getModel().stop();
        System.out.println("[Midp Client] Finished");
    }

    /**
     *   Uses properties set within the JAD file of this app to set properties
     *   of the model. If the properties do not exist within the JAD, defaults will be used
     *
     * @return void
     */
    protected void setPumpProperties() {
        getModel().setProperty("Security", getAppProperty("Pump_Security"));
        getModel().setProperty("ServerIP", getAppProperty("Pump_ServerIP"));
        getModel().setProperty("ServerPort", getAppProperty("Pump_ServerPort"));
        getModel().setProperty("ServerQMname", getAppProperty("Pump_ServerQueueManager"));
        getModel().setProperty("ServerQname", getAppProperty("Pump_ServerQueue"));
    }

    /**
     *   Generates a new options form ( it has to fill in the default values )
     *   and then returns it.
     *
     * @return Form - the options form
     */
    protected Form getOptionScreen() {
        optionForm = new Form("Options");
        waittime =
                new TextField(
                    "Wait time (ms)",
                    String.valueOf(((MsgPumpModelClient) getModel()).getWaitTime()),
                    6,
                    TextField.NUMERIC);
        optionForm.append(waittime);
        optionForm.addCommand(ok);
        optionForm.setCommandListener(this);
        return optionForm;
    }

    /**
     *   Generates and returns a status report screen.
     *
     * @return TextBox - The status report
     */
    protected TextBox getStatusScreen() {
        statusForm = new TextBox("Status Report", "", 500, TextField.ANY);

        String currentStatus =
            "QM Name: "
            + getModel().getLocalQMname()
            + "\nPump rate: "
            + ((MsgPumpModelClient) getModel()).getWaitTime()
            + "ms"
            + "\nServer Address: "
            + ((MsgPumpModelClient) getModel()).getServerAddress()
            + "\nServer QM Name: "
            + ((MsgPumpModelClient) getModel()).getServerQMname()
            + "\nServer Q Name: "
            + ((MsgPumpModelClient) getModel()).getServerQname();

        statusForm.setString(currentStatus);
        statusForm.addCommand(ok);
        statusForm.setCommandListener(this);
        return statusForm;
    }

    /**
     *   Returns the default screen used to output information to the user.
     *
     * @return TextBox - default screen
     */
    protected TextBox getOutputScreen() {
        return text;
    }

    /**
     *   The equivilant of initialize.
     *   This function sets up the main display screen and te controller.
     *
     * @return void
     */
    public void initializeMidlet() {
        try {
            text = new TextBox("Pump Client", "", 500, TextField.ANY);

            setPumpProperties();

            display = Display.getDisplay(this);
            text.addCommand(pause);
            text.addCommand(resume);
            text.addCommand(status);
            text.addCommand(options);
            text.setCommandListener(this);
            changeDisplay(text);

            started = true;
            initialize();
        } catch (Exception e) {
            e.printStackTrace();
            try {
                destroyApp(true);
            } catch (Exception ex) {}
        }
    }

    /**
     *   Creates and displays a prompt to the user for a new QM name.
     *
     * @return void
     */
    protected void promptForQMName() {

        /* Doesnt exist - make new */
        namePrompt = new Form("Client Name");
        QMPrompt = new TextField("Enter a name for the client", "QM_MidpClient", 20, TextField.ANY);
        namePrompt.append(QMPrompt);
        namePrompt.addCommand(ok);
        namePrompt.setCommandListener(this);
        changeDisplay(namePrompt);
    }

    /**
     *   This is called if the user clicks a button.
     * <p>See CommandListener#commandAction for more details.
     */
    public void commandAction(Command c, Displayable d) {
        if (c == pause) {
            getModel().pause();
        } else if (c == resume) {
            getModel().resume();
        } else if (c == status) {
            changeDisplay(getStatusScreen());
        } else if (c == options) {
            changeDisplay(getOptionScreen());
        } else if (c == ok) {
            if (display.getCurrent() == namePrompt) {
                acquiredQMName(QMPrompt.getString());
            } else if (display.getCurrent() == optionForm) {
                ((MsgPumpModelClient) getModel()).setWaitTime(Integer.parseInt(waittime.getString()));
                changeDisplay(text);
            } else if (display.getCurrent() == statusForm) {
                changeDisplay(text);
            }
        }
    }

    /**
     *   Changes the current display.
     *
     * @param newDisplayable The new screen to be displayed
     * @return void
     */
    protected void changeDisplay(Screen newDisplayable) {
        display.setCurrent(newDisplayable);
    }
}
