package examples.midp.exampleapp.msgpump;


/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;


/**
 * This is the client implementation of the message pump model.
 * What this means is that the logic required pump the messages to a server
 * is located in this class. The actual implementation of that logic (creating and
 * sending admin messages etc) is done in the Message Service. The interaction with
 * the user is done by the controller, which also controls such things as where the
 * messages are being pumped to and how fast.
 */
public class MsgPumpModelClient extends MsgPumpModel {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version
  
    // Current state of the client
    private boolean stop = false;
    private boolean paused = false;

    // Details of server to pump messages to
    // These are set to default values, but can be changed using setProperty
    // This should only happen before the model has been initialized however
    private String serverQMname = DEFAULT_SERVER_QM_NAME;
    private String serverQname = DEFAULT_SERVER_Q_NAME;
    private String serverIP = "127.0.0.1";
    private String serverPort = "8082";
    private String serverCommand = null;
    private String serverOptions = null;

    // Number of messages to send
    private long maxMsgsToSend = 1000;

    // Time to wait between sending messages (millseconds)
    private long timeBetweenMsgs = 3000;

    MsgPumpModelClient() {
        super();
    }

    /**
     *   sets a property of the pump
     *
     * @param property The property to set
     * @param value The value to set it to
     * @return void
     */
    protected void setProperty(String property, String value) {
        if (property.equals("ServerIP")) {
            if (value != null) {
                serverIP = value;
            }
        } else if (property.equals("ServerPort")) {
            if (value != null) {
                serverPort = value;
            }
        } else if (property.equals("ServerQMname")) {
            if (value != null) {
                serverQMname = value;
            }
        } else if (property.equals("ServerQname")) {
            if (value != null) {
                serverQname = value;
            }
        } else {
            super.setProperty(property, value);
        }
    }

    /**
     *   Gets the address that messages will be pumped to.
     *   <br>e.g. Network://127.0.0.1:8082.
     *
     * @return String - the address of the server
     */
    public String getServerAddress() {
        return getMsgService().DEFAULT_LISTENERADAPTER + "://" + serverIP + ":" + serverPort;
    }

    /**
     *     The client of the message pump will send a message every [timeBetweenMsgs] milliseconds
     *     to the server until it is stopped or [maxMsgsToSend] messages have been sent.
     *
     * @return void
     */
    public void run() {
        try {
            int i = 0;

            while (!stop && i < maxMsgsToSend) {

                /* Create a small datagram test message */
                MQeMsgObject msg = new MQeMsgObject();
                String timestamp = getTimestamp();

                msg.putInt(MQe.Msg_Style, MQe.Msg_Style_Datagram);
                msg.putInt("MsgNumber", i);
                msg.putUnicode("Timestamp", timestamp);

                // Send message asynchronously
                // Method of sending depends on the type of security being used in the model
                // For message based security, an attribute with the relevant cryptor is included
                if (isSecurity(NO_SECURITY) || isSecurity(QUEUE_SECURITY)) {
                    // No security / queue based security
                    getMsgService().sendMessage(serverQMname, serverQname, msg);
                } else if (isSecurity(MESSAGE_SECURITY)) {
                    // Message based security - need to encrypt message
                    getMsgService().sendSecureMessage(serverQMname, serverQname, msg, getMessageAttribute());
                }

                processStatus(
                    "[Client Model]\nSent message "
                    + i
                    + "\nAt "
                    + timestamp
                    + "\nWaiting for "
                    + timeBetweenMsgs
                    + "ms");

                /* Wait for a while before sending another message */
                Thread.sleep(timeBetweenMsgs);
                i++;

                /* If you have been told to pause, wait on the controller */
                if (paused) {
                    synchronized (getController()) {
                        processStatus("-= Paused =-");
                        getController().wait();
                        processStatus("-= Unpaused =-");
                    }
                }
            }

            /* Leaving the while loop means you have been told to stop, so finish up */
            finish();
            System.out.println("[Client Model] Finished");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @see MsgPumpModel#stop()
     */
    public void stop() {
        stop = true;
        resume();
        setController(null);
    }

    /**
     * @see MsgPumpModel#pause()
     */
    public void pause() {
        paused = true;
    }

    /**
     * @see MsgPumpModel#resume()
     */
    public void resume() {
        paused = false;
        synchronized (getController()) {
            getController().notifyAll();
        }
    }

    /**
     *   Set how long the pump should wait between messages.
     *
     * @param wait Wait time in milliseconds
     * @return void
     */
    public void setWaitTime(long wait) {
        processStatus("[Client Model] Time between messages: " + wait + "ms");
        timeBetweenMsgs = wait;
    }

    /**
     *   Get the wait time of the client.
     *
     * @return long - wait time in milliseconds
     */
    public long getWaitTime() {
        return timeBetweenMsgs;
    }

    /**
     *     The client only requires a remote queue as it is not expecting to
     *     receive any messages.
     * @see MsgPumpModel#performInitialConfiguration()
     */
    protected void performInitialConfiguration() throws Exception {
        processStatus("[Client Model] Initial configuration...");

        try {
            processStatus("[Client Model] Creating local queue");
            getMsgService().createQueueManager();

            processStatus("[Client Model] Opening queue");
            getMsgService().open();

            processStatus("[Client Model] Creating connection to server");
            getMsgService().createConnection(
                getLocalQMname(),
                serverQMname,
                getServerAddress(),
                serverCommand,
                serverOptions);

            processStatus("[Client Model] Creating remote queue");
            // If we're using queue based escurity, set the queue up with the appropriate
            // attributes
            if (isSecurity(QUEUE_SECURITY)) {
                getMsgService().createRemoteQueue(
                    getLocalQMname(),
                    serverQMname,
                    serverQname,
                    sync,
                    compressor,
                    cryptor);
            } else {
                getMsgService().createRemoteQueue(
                    getLocalQMname(),
                    serverQMname,
                    serverQname,
                    sync);
            }

            processStatus("[Client Model] Updating queue manager");
            MQeFields qmParms = new MQeFields();

            qmParms.putAscii("rules", getQMrules());
            getMsgService().updateQueueManager(getLocalQMname(), qmParms);
        } catch (Exception e) {
            try {
                getController().deleteQueueManager();
            } catch (Exception ex) {}
            throw e;
        }
    }

    /**
     * @see MsgPumpModel#processDataGramMsg(MQeMsgObject, String, String)
     */
    public void processDataGramMsg(MQeMsgObject message, String qMgrName, String qName) {}

    /**
     * @see MsgPumpModel#processReplyMsg(MQeMsgObject, String, String)
     */
    public void processReplyMsg(MQeMsgObject message, String qMgrName, String qName) {}

    /**
     * @see MsgPumpModel#processRequestMsg(MQeMsgObject, String, String)
     */
    public void processRequestMsg(MQeMsgObject message, String qMgrName, String qName) {}

    /**
     *     Gets the name of the server's queue manager that is being pumped to.
     *
     * @return String - the name of the server queue manager
     */
    public String getServerQMname() {
        return serverQMname;
    }

    /**
     *     Gets the name of the queue on the server that is being pumped to.
     *
     * @return String - the name of the queue being pumped to
     */
    public String getServerQname() {
        return serverQname;
    }
}
