package examples.midp.exampleapp.textapp;



/*----------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                  */

/*                                                                            */

/* Licensed Materials - Property of IBM                                       */

/*                                                                            */

/* 5765-E63                                                                   */

/*                                                                            */

/* (C) Copyright IBM Corp. 1999                                               */

/*----------------------------------------------------------------------------*/


import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeAdminMsg;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;
import com.ibm.mqe.administration.MQeStoreAndForwardQueueAdminMsg;



/**
 * The aim of the registration server is to give an example of how a server can be
 * used to provide resources for new clients.
 * In this case, when a client registers with the server, it will be added
 * to a store and forward queue on the gateway. This is a simple example of how
 * resources can be set up for clients by another queue manager, allow greater
 * flexibility and scalability in the design of the system.
 *
 * <p>If the registration succeeds or fails, a reply is left for the client on the
 * registration server's reply queue informing them of the success / failure of the
 * registration.
 */
public class AppModelRegistrationServer extends AppModelServer {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version
  

    /**
     *
     * Need request and reply registration queues as well as a normal reply queue.
     * @see AppModel#chooseLocalQueues()
     */
    protected String[] chooseLocalQueues() {
        return new String[] {
                DEFAULT_REPLY_Q,
                "async",
                DEFAULT_REGISTRATION_Q,
                "sync",
                DEFAULT_REGISTRATION_REPLY_Q,
                "async" };
    }


    /**
     *
     * <p>Reg server has no other queues to initialize.
     * @see AppModel#initializeOtherQueues()
     */
    protected void initializeOtherQueues() throws Exception {}


    /**
     * @see AppModel#chooseRemoteQueues()
     */
    protected String[] chooseRemoteQueues() {
        return new String[] {
                DEFAULT_GATEWAY_QM,
                MQe.Admin_Queue_Name,
                "sync",
                DEFAULT_GATEWAY_QM,
                MQe.Admin_Reply_Queue_Name,
                "sync" };
    }


    /**
     * @see AppModel#chooseConnections()
     */
    protected String[] chooseConnections() {
        return new String[] { DEFAULT_GATEWAY_QM, DEFAULT_GATEWAY, GatewayCommand, GatewayOptions };
    }


    /**
     *   Creates a reply message for a queue manager that has just been registered.
     *
     * @param regQMname The name of the queue manager that has been registered
     * @return MQeMsgObject - the reply message to send to the queue manager
     */
    protected MQeMsgObject createRegistrationReplyMsg(String regQMname) throws Exception {
        MQeMsgObject replyMsg = null;

        replyMsg = new MQeMsgObject();
        replyMsg.putInt(MQe.Msg_Style, MQe.Msg_Style_Reply);
        replyMsg.putArrayOfByte(MQe.Msg_CorrelID, MQe.asciiToByte(regQMname));
        return replyMsg;
    }


    /**
     *   Add resource definitions required for a client on the central site
     *   for instance add client as a target to a store and forward queue,
     *   add remote queue definition to big MQ
     *
     * @param regQMname Name of the queue manager to be registered
     * @return void
     */
    protected void createResourcesForNewClientQM(String regQMname) //, MQeMsgObject regMsg)
        throws Exception {

        /*
         * Create and send an admin message to add the target client to a store-and-forward
         * queue on the gateway
         */
        MQeStoreAndForwardQueueAdminMsg msg = new MQeStoreAndForwardQueueAdminMsg();
        MQeFields msgTest = getMsgService().primeAdminMsg(DEFAULT_GATEWAY_QM, msg);

        msg.setName(DEFAULT_GATEWAY_QM, DEFAULT_HOMESERVER_Q);
        msg.addQueueManager(regQMname);
        MQeAdminMsg replyMsg = getMsgService().processAdminMsg(DEFAULT_GATEWAY_QM, msg);


        /* Check the reply. An 'already exists' error is allowed as the client may
         * be trying to reregister after a failure.
         */
        if (replyMsg == null) {
            processStatus("..No response received to the request");
            throw new MQeException("No response message received");
        }


        // If the reply was bad then check why
        if (replyMsg.getRC() != MQeAdminMsg.RC_Success) {
            MQeFields errors = replyMsg.getErrorFields();

            if (errors != null && errors.contains(MQeStoreAndForwardQueueAdminMsg.Queue_QMgrNameList)) {

                /*
                 * The client may have crashed, lost its registry and restarted.
                 * In this case, the Except_QMgr_QExists error would be throws as the queue
                 * created for the client already exists. If this is the case, ignore the error
                 *
                 * To find this, what we're looking for in the exception is 'Code:105 .....'
                 * where 105 == Except_QMgr_QExists
                 * hence the use of substring(5,8)
                 */
                String errs[] = errors.getAsciiArray(MQeStoreAndForwardQueueAdminMsg.Queue_QMgrNameList);

                if (errs[0].substring(5, 8).equals(String.valueOf(MQe.Except_QMgr_QExists))) {
                    processStatus("target already registered on: " + DEFAULT_GATEWAY_QM);
                } else {
                    throw new MQeException(replyMsg.getReason());
                }
            } else {
                throw new MQeException(replyMsg.getReason());
            }
        }
    }


    /**
     *   Called upon reveiving a new registration request from a client.
     *   Used to set up all the required resources for the client.
     *
     * @param msg The request message
     * @return void
     */
    protected void processRegistrationRequest(MQeMsgObject msg)
        throws Exception { //, String replyQMname, String replyQname) {


        // Determine the name of the queue manager that is trying to register
        String regQMname = null;

        try {
            regQMname = msg.getAscii("QMgrName");
            processStatus("\n== Processing registration request from " + regQMname + " ==");
        } catch (Exception e) {
            // The request does not contain a queue manager name, so cannot be
            // processed. Also as the queue manager name is used for the correlid
            // in the reply message, a reply cannot be created.
            processStatus("Registration cannot process non registration message");
            processMessage(msg);
            return;
        }


        //Prepare the reply message
        MQeMsgObject replyMsg = createRegistrationReplyMsg(regQMname);


        try {
            // Create the resources for the registering queue manager
            createResourcesForNewClientQM(regQMname); //, msg);


            // Successful...
            replyMsg.putBoolean("Success", true);
            processStatus("Registration was a success");
        } catch (Exception ex) {
            // Failed....
            processStatus("Registration failed, will send failure message: " + ex);
            ex.printStackTrace();
            try {
                replyMsg.putBoolean("Success", false);
                replyMsg.putUnicode("Reason", ex.toString());
            } catch (Exception exx) {
                // error setting up reply message so cannot send it just return
                processStatus("Registration failed sending reply: " + exx);
                exx.printStackTrace();
                return;
            }
        }


        try {
            // Put the reply message on the reply queue, ready for the queue
            // manager that is registering to pick up
            getMsgService().sendMessage(getLocalQMname(), DEFAULT_REGISTRATION_REPLY_Q, replyMsg);
        } catch (Exception e) {
            processStatus("Failed sending registration reply msg: " + e);
        }
    }


    /**
     *   A new request message has been received, do something with it for instance
     *   display it.
     *
     *   <p>Note: the message still exists on the queue, it has been browsed so will stay
     *   on the queue until a get or delete operation.
     *
     * @param msg The message to display.
     * @param qMgrName The queue manager the message is on
     * @param qName The queue the message is on
     * @return void
     */
    public void processRequestMsg(MQeMsgObject msg, String qMgrName, String qName) {
        // It should be a registration request so process it.
        try {
            processRegistrationRequest(msg); //, qMgrName, qName);
            getMsgService().getMessage(getLocalQMname(), DEFAULT_REGISTRATION_Q, msg.getMsgUIDFields());
        } catch (Exception e) {
            processStatus("Failed to process registration request: " + e);
        }
    }


    /**
     *   The registration server has no need to deal with reply messages.
     *
     * @param message The message to display.
     * @param qMgrName The queue manager the message is on
     * @param qName The queue the message is on
     * @return void
     */
    public void processReplyMsg(MQeMsgObject message, String qMgrName, String qName) {}


    /**
     * @see AppModel#getListenerPort()
     */
    protected int getListenerPort() {
        return DEFAULT_REGISTRATION_LISTENER_PORT;
    }
}
