// Version 1.1 mqe_java/source/examples/mqbridge/administration/commandline/IsQueueManagerBridgeCapable.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2001                                                 *
 *                                                                              *
 *------------------------------------------------------------------------------*/

package examples.mqbridge.administration.commandline;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;


/**
 * Queries the queue manager and finds whether the queue manager is 
 * bridge capable or not.
 * 
 * <p>If the queue manager is bridge capable, the text "bridge-capable=true"
 * is put to stdout. 
 * <p>If the queue manager is not bridge capable, the text
 * "bridge-capable=false" is put to stdout.
 * <p>If the answer cannot be obtained, the text
 * "bridge-capable=unknown" is put to stdout.
 * 
 * <p><b>Syntax:</b>
 * <tt>java examples.mqbridge.administration.commandline.IsQueueManagerBridgeCapable 
 * &lt;targetQMgr&gt; &lt;iniFile&gt;</tt>
 * <dl>
 * <dt>targetQMgr</dt>
 * <dd>The name of the MQe queue manager to which an Inquire admin message will be sent.
 * The queue manager must be known to the one described in the .ini file.
 * </dd>
 * <dt>iniFile</dt>
 * <dd>An .ini file containing details describing a queue manager which will be 
 * instantiated in memory by this tool. This queue manager will be used to send 
 * the administration message required to find out whether the target queue manager
 * is bridge capable or not.</dd>
 * </dl>
 */
public class IsQueueManagerBridgeCapable extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 

    /**
     * The version of this example.
     */
    public static short[] version = { 2, 0, 0, 5 }; 

    /**
     * Is the target queue manager bridge-capable ?
     */
    private boolean isBridgeCapable = false;
	
    /**
     * Do we have a valid result from our query yet ?
     */
    private boolean gotResult = false;

    /**
     * Does this tool want the .ini file to be output to the stdout stream ?
     */
    private static final boolean SUPPRESS_COPYING_INI_FILE_TO_STDOUT = true;
	
    private static final boolean START_UP = true;
    private static final boolean SHUT_DOWN = false;

    /**
     * Syntax of the command.
     */
    public static final String syntax = 
        "Syntax: ...IsQueueManagerBridgeCapable <targetQMgr> <iniFile>\n"
        + "Where...\n"
        + "targetQMgr\n"
        + "  The name of the queue manager to which the Inquire admin msg should be sent.\n"
        + "  This can be the name of the queue manager described  by the iniFile if a local\n"
        + "  query is required.\n"
        + "iniFile\n"
        + "  The fully-qualified path name of the .ini file of the queue manager used to\n"
        + "  send the admin message.";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        IsQueueManagerBridgeCapable me = new IsQueueManagerBridgeCapable();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); 
    }

    /**
     * Gathers the parameters and passes them to a method which does the command.
     * @param args
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(String args[]) throws Exception {
        if (args.length < 2) {
            throw new Exception("Too few parameters." + syntax);
        }
        if (args.length > 2) {
            throw new Exception("Too many parameters." + syntax);
        }    	

        try {

            String targetQMgr = args[0];
            String iniFile = args[1];

            // Access the local MQe queue manager...
            MQeClient client = new MQeClient(iniFile, SUPPRESS_COPYING_INI_FILE_TO_STDOUT);

            // Access the queue manager created in this JVM... the server QM
            MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

            // Inquire about the target queue manager...
            // Throws an exception if the result is not found.
            isBridgeCapable = inquireBridgeCapable(targetQMgr, myQMgr);

            gotResult = true;

            // Close down the MQeClient
            client.close();	        
	        
        } finally {
			
            if (gotResult) {
                if (isBridgeCapable) {
                    System.out.println("bridge-capable=true");
                } else {
                    System.out.println("bridge-capable=false");
                }
            } else {
                System.out.println("bridge-capable=unknown");
            }
        }
    } 

    /**
     * Find out if the target queue manager is bridge capable, using the queue manager passed.
     * 
     * @return true if the target queue manager is bridge capable, false otherwise.
     * @throws Exception if a definate true/false cannot be obtained.
     */
    public boolean inquireBridgeCapable(String targetQMgr, MQeQueueManager myQMgr 
    ) throws Exception {
        // Assume the target qmgr is NOT bridge capable...
        boolean result = false;

        System.out.println("Building the admin message to inquire about the queue manager.");
                        
        MQeQueueManagerAdminMsg msg = new MQeQueueManagerAdminMsg();

        // Create a fields object in which we can pre-initialise query 
        MQeFields params = new MQeFields();

        // When we do the inquire, the bridge capable flag is the only 
        // thing we're interrested in.
        params.putBoolean(MQeQueueManagerAdminMsg.QMgr_BridgeCapable, true);

        // Set the command in the admin msg to "create".
        msg.inquire(params);

        // Set the name of the bridge we wish to create into the admin msg...
        msg.setName(targetQMgr);

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);
        
        System.out.println(results.toString());

        if (!results.contains(MQeQueueManagerAdminMsg.QMgr_BridgeCapable)) {
            throw new Exception("Error: cannot find out if the target queue manager is bridge capable.\n"
                    + "       Could indicate that the target queue manager is using a version\n"
                    + "       of MQe prior to v1.2.7"
                );
        } else {								
            // The result has been sent back to us.
            result = results.getBoolean(MQeQueueManagerAdminMsg.QMgr_BridgeCapable);
        } 

        return result;	
    }	    	
}
