// Version mqe_java/source/examples/mqbridge/administration/commandline/MQBridgeCreator.java, MQe, la200, la200-L030324.1 1.8

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.administration.commandline; 


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
import examples.mqbridge.queuemanager.*;
import com.ibm.mqe.mqbridge.*;
import examples.administration.commandline.*;


/**
 * A command-line tool to add a bridge to a bridge-enabled queue manager.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.mqbridge.administration.commandline.MQBridgeCreator 
 * &lt;bridgeName&gt;
 * &lt;transformer&gt; 
 * &lt;qMgr&gt; 
 * &lt;iniFile&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * 
 * <DT>bridgeName</DT>
 * <DD>The name of the bridge to be created</DD>
 * 
 * <DT>transformer</DT>
 * <DD>The name of the class which will be used as the "default" transformer
 * for converting between MQe and MQ message formats.
 * Should include the .class extension.
 * Could be an alias name of class also.
 * </DD>
 * 
 * <DT>qMgr</DT>
 * <DD>The name of the queue manager to which the admin msg should be sent.</DD>
 * 
 * 
 * <DT>iniFile</DT>
 * <DD>The fully-qualified path name of the .ini file of the server queue manager.</DD>
 * 
 * </DL>
 * 
 * <P><B>Example</B></P>
 * <P><TT>java ...MQBridgeCreator myBridge myTransformer.class MyQMgr C:\MQe\QMgrs\MyQMgr\MyQMgr.ini
 * </TT></P>
 * <P>Adds a bridge called "myBridge" to the MyQMgr queue manager.</P>
 */
public class MQBridgeCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 

    public       static short version[] = { 2, 0, 0, 5 }; 

    /**
     * Syntax of the command.
     */
    public static final String syntax = 
        "Syntax: ...MQBridgeCreator <bridgeName> <transformer> <qMgr> <iniFile>\n"
        + "Where...\n"
        + "bridgeName\n"
        + "  The name of the bridge to be created\n"
        + "transformer\n"
        + "  The name of the class which will be used as the \"default\" transformer\n"
        + "  for converting between MQe and MQ message formats.\n"
        + "  (should not include the .class extension)\n"
        + "qMgr\n"
        + "  The name of the queue manager to which the admin msg should be sent.\n"
        + "iniFile\n"
        + "  The fully-qualified path name of the .ini file of the server queue manager.\n";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        MQBridgeCreator me = new MQBridgeCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers the parameters and passes them to a method which does the command.
     * @param args
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(String args[]) throws Exception {

        String targetQMgr = null;   // The queue manager to which the admin msg is sent.
        String iniFile = null;      // The ini file used to create the server while we use it.
        String bridgeName = null;   // The name of the bridge to create
        String transformerClass = null;     // The name of the transformer class we will use.

        if (args.length < 4) {
            throw new Exception("Error: Not enough parameters. " + syntax);
        }
        bridgeName = args[0].trim();
        transformerClass = args[1].trim();
        targetQMgr = args[2].trim();
        iniFile = args[3].trim();

        // Access the local MQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Supress any stdout information.
            );

        // Start up the server...
        server.activate(true);

        // Access the queue manager created in this JVM... the server QM
        MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

        // Create the queue...
        createMQBridge(bridgeName, transformerClass, targetQMgr, myQMgr);

        // Ask the server to shut down peacefully...
        server.activate(false);

        // Close down the MQeClient
        server.close();
    } 

    /**
     * Creates an admin message which will create a bridge, and sends it
     * to a named queue manager.
     * 
     * @param bridgeName The name of the bridge to create
     * 
     * @param transformerClass
     *                   The name of the transformer class to set into the bridge configuration.
     *                   This transformer will be used whenever a listener or bridge queue have a ""
     *                   value in their transformer configuration parameters.
     * 
     * @param targetQMgr The name of the MQe queue manager on which the bridge will be created.
     * 
     * @param myQMgr A reference to an active MQe queue manager, used to send mesages from and receive messages using.
     * @return void
     * @exception Exception
     */
    public void createMQBridge(String bridgeName, String transformerClass, String targetQMgr, MQeQueueManager myQMgr
    ) throws Exception {

        System.out.println("Building the admin message to create a bridge " + bridgeName
            + " using default transformer " + transformerClass);
        MQeMQBridgeAdminMsg msg = new MQeMQBridgeAdminMsg();

        // Create a fields object in which we can pre-initialise the bridge...
        MQeFields params = new MQeFields();

        // When we create the bridge, set the default transformer.
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER, transformerClass
        );

        // Set the command in the admin msg to "create".
        msg.create(params);

        // Set the name of the bridge we wish to create into the admin msg...
        msg.setName(bridgeName);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);
    } 

} 
