// Version mqe_java/source/examples/mqbridge/administration/commandline/MQConnectionCreator.java, MQe, la200, la200-L030324.1 1.7

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/

package examples.mqbridge.administration.commandline; 


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
import examples.mqbridge.queuemanager.*;
import com.ibm.mqe.mqbridge.*;
import examples.administration.commandline.*;


/**
 * A command line tool to create an WMQ client connection object on the
 * WebSphere MQ Everyplace bridge on a specified WMQe queue manager.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.mqbridge.administration.commandline.MQConnectionCreator 
 * &lt;bridgeName&gt;
 * &lt;proxyName&gt;
 * &lt;connectionName&gt; 
 * &lt;port&gt; 
 * &lt;syncQName&gt; 
 * &lt;qMgr&gt; 
 * &lt;iniFile&gt;
 * </TT></P>
 * <P><B>Parameters</B></P>
 * <DL>
 * <DT>bridgeName</DT>
 * <DD>The name of the parent bridge.</DD>
 * <DT>proxyName</DT>
 * <DD>The name of the WMQ queue manager proxy.</DD>
 * <DT>connectionName</DT>
 * <DD>The name of the connection. This must match the name of a server-channel
 * connection on the WMQ queue manager.</DD>
 * <DT>port</DT>
 * <DD>The port number on which the WMQ queue manager is listening on for client connections</DD>
 * <DT>syncQName</DT>
 * <DD>The name of the sync queue to use (a local queue on the WMQ queue manager used to store state)</DD>
 * <DT>qMgr</DT>
 * <DD>The name of the WMQe queue manager to which the administration message is
 * to be sent to, and which will have the new object configured as a result.
 * The queue manager must be a bridge-enabled queue manager.</DD>
 * <DT>iniFile</DT>
 * <DD>The fully-qualified path of the .ini file which describes a queue manager which will be
 * be used to issue the administration message.</DD>
 * <DL>
 */
public class MQConnectionCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 

    public       static short version[] = { 2, 0, 0, 5 }; 

    static final String syntax = 
        "Syntax: java MQConnectionCreator <bridgeName> <proxyName> <connectionName> <port> <syncQName> <qMgr> <iniFile>\n"
        + "Where...\n"
        + "bridgeName\n"
        + "  The name of the parent bridge.\n"
        + "proxyName\n"
        + "  The name of the WMQ queue manager proxy.\n"
        + "connectionName\n"
        + "  The name of the connection. This must match the name of a server-channel\n"
        + "  connection on the WMQ queue manager.\n"
        + "port\n"
        + "  The port number on which the WMQ queue manager is listening on for client connections\n"
        + "syncQName\n"
        + "  The name of the sync queue to use (a local queue on the WMQ queue manager used to store state)\n"
        + "qMgr\n"
        + "  The name of the WMQe queue manager to which the administration message is\n"
        + "  to be sent to, and which will have the new object configured as a result.\n"
        + "  The queue manager must be a bridge-enabled queue manager.\n"
        + "iniFile\n"
        + "  The fully-qualified path of the .ini file which describes a queue manager which will be\n"
        + "  be used to issue the administration message.\n";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        MQConnectionCreator me = new MQConnectionCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Does all the work.
     * @param args
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(String args[]) throws Exception {
        String targetQMgr = null;   // The queue manager to which the admin msg is sent.
        String iniFile = null;      // The ini file used to create the server while we use it.
        String bridgeName = null;   // The name of the bridge to create
        String proxyName = null;   // The name of the proxy to create
        String connectionName = null;// The name of the WMQ client connection to use.
        int    port = 0;   // The port number to use.
        String syncQName = null;     // The name of the sync queue on WMQ

        // Get the parameters from the system properties...
        String portString = null;

        if (args.length < 7) {
            throw new Exception("Error: Not enough parameters.\n" + syntax);
        }
        bridgeName = args[0].trim();
        proxyName = args[1].trim();
        connectionName = args[2].trim();
        portString = args[3].trim();
        syncQName = args[4].trim();
        targetQMgr = args[5].trim();
        iniFile = args[6].trim();

        System.out.println("Atempting to create a new WMQConnection on qmgr " + targetQMgr
            + " bridge " + bridgeName + " proxy " + proxyName + " which has a port of " + portString
            + " using the ini file " + iniFile);

        // Convert the port string into an integer we can use as a number...
        port = Integer.parseInt(portString);

        // Access the local MQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Supress any stdout information.
            );

        // Start up the server...
        server.activate(true);

        // Access the queue manager created in this JVM... the server QM
        MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

        // Create the queue...
        createMQConnection(bridgeName, proxyName, connectionName, port, syncQName, targetQMgr, myQMgr);

        // Ask the server to shut down peacefully...
        server.activate(false);

        // Close down the MQeClient
        server.close();
    } 

    /**
     * Creates an admin message which will create an WMQ connection on the bridge, and sends it
     * to a named queue manager.
     * 
     * @param bridgeName The name of the bridge for which an WMQ connection will be created.
     * 
     * @param proxyName The name of the WMQ queue manager proxy for which connection will be created.
     *                  ie: The future parent object of the WMQ client connection.
     * 
     * @param connectionName
     *                   The name of the WMQ client connection which will be created.
     *                   It should match the name of an WMQ server connection channel.
     * 
     * @param port   The port on which the WMQ system is listening on for in-coming client connections.
     * 
     * @param syncQName The name of the sync queue used by WMQe. This is a local queue which must exist
     *                  on the WMQ queue manager (whose name is given by the "proxyname" parameter above)
     *                  It must be a local queue, persistent, and read-writable.
     * 
     * @param targetQMgr The queue manager on which the new WMQ connection will be created.
     * 
     * @param myQMgr A reference to an active WMQe queue manager.
     *               The queue manager will be used to send and receive WMQe administration messages and responses.
     * @return void 
     * @exception Exception
     */
    protected void createMQConnection(String bridgeName, String proxyName, String connectionName, int    port, String syncQName, String targetQMgr, MQeQueueManager myQMgr
    ) throws Exception {

        MQeClientConnectionAdminMsg msg = new MQeClientConnectionAdminMsg();

        // Create a fields object in which we can pre-initialise the object...
        MQeFields params = new MQeFields();

        params.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_PORT, port
        );
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_SYNC_Q_NAME, syncQName
        );

        // Set the command in the admin msg to "create".
        msg.create(params);

        // Set the name of the object we wish to create into the admin msg...
        msg.setName(bridgeName, proxyName, connectionName);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);
    }  
}
