// Version mqe_java/source/examples/mqbridge/administration/commandline/MQQMgrProxyCreator.java, MQe, la200, la200-L030324.1 1.7

/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2001                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.administration.commandline;


import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import examples.queuemanager.*;
import examples.mqbridge.queuemanager.*;
import com.ibm.mqe.mqbridge.*;
import examples.administration.commandline.*;


/**
 * A command-line tool to create an MQ queue manager proxy object.
 * 
 * <P><B>Syntax</B></P>
 * <P><TT>java examples.mqbridge.administration.commandline.MQQMgrProxyCreator 
 * &lt;bridgeName&gt;
 * &lt;proxyName&gt; 
 * &lt;hostName&gt; 
 * &lt;qMgr&gt; 
 * &lt;iniFile&gt;
 * </TT></P>
 * 
 * <P><B>Parameters</B></P>
 * <DT>bridgeName</DT>
 * <dd>The name of the parent bridge.</DD>
 * 
 * <DT>proxyName</DT>
 * <DD>The name of the WMQ queue manager for which you are creating the proxy.</DD>
 * 
 * <DT>hostName</DT>
 * <DD>The name of the host machine on which the WMQ queue manager sits.
 * If this is set to "null" then connections to the WMQ queue manager
 * will use the WMQ java bindings rather than the WMQ java client classes,
 * with the consequential performance increase. Only specify "null" if your WMQ
 * queue manager is on the same machine as the WMQe queue manager you are
 * configuring.</DD>
 * 
 * <DT>qMgr</DT>
 * <DD>The name of the WMQe queue manager to which the administration message is
 * to be sent to, and which will have a new MQQMgrProxy object configured as a result.
 * The queue manager must be a bridge-enabled queue manager.</DD>
 * 
 * <DT>iniFile</DT>
 * <DD>The fully-qualified path of the .ini file which describes a queue manager which will be
 * be used to issue the administration message.</DD>
 * 
 * </DL>
 */
public class MQQMgrProxyCreator extends examples.administration.commandline.MQeAbstractCmdLineAdminTool { 

    public       static short version[] = { 2, 0, 0, 5 }; 

    static final String syntax = 
        "Syntax: java MQQMgrProxyCreator <bridgeName> <proxyName> <hostName> <qMgr> <iniFile>\n"
        + "Where...\n"
        + "bridgeName\n"
        + "  The name of the parent bridge.\n"
        + "proxyName\n"
        + "  The name of the WMQ queue manager for which you are creating the proxy.\n"
        + "hostName\n"
        + "  The name of the host machine on which the WMQ queue manager sits.\n"
        + "  If this is set to \"null\" then connections to the WMQ queue manager\n"
        + "  will use the WMQ java bindings rather than the WMQ java client classes,\n"
        + "  with the consequential performance increase. Only specify \"null\" if your WMQ\n"
        + "  queue manager is on the same machine as the WMQe queue manager you are\n"
        + "  configuring.\n"
        + "qMgr\n"
        + "  The name of the WMQe queue manager to which the administration message is\n"
        + "  to be sent to, and which will have a new MQQMgrProxy object configured as a result.\n"
        + "  The queue manager must be a bridge-enabled queue manager.\n"
        + "iniFile\n"
        + "  The fully-qualified path of the .ini file which describes a queue manager which will be\n"
        + "  be used to issue the administration message.\n";

    /**
     * Entry point from the command line.
     */
    public static void main(String args[]) {
        int rc = 0;
        MQQMgrProxyCreator me = new MQQMgrProxyCreator();

        try {
            me.activate(args);
        } catch (Exception e) {
            System.out.println("Error: activate() failed. " + e.toString());
            rc = 1;
        }
        System.exit(rc); // exit,killing the trace window by force if necessary.
    }

    /**
     * Gathers command-line parameters into variables and calls a method which creates the proxy object.
     * 
     * @param args
     * @return void
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        String targetQMgr = null;   // The queue manager to which the admin msg is sent.
        String iniFile = null;      // The ini file used to create the server while we use it.
        String bridgeName = null;   // The name of the bridge to create
        String proxyName = null;   // The name of the proxy to create
        String hostName = null;   // The host on which the MQ queue manager sits, or "null"

        if (args.length < 5) {
            throw new Exception("Error: Not enough parameters.\n" + syntax);
        }
        bridgeName = args[0].trim();
        proxyName = args[1].trim();
        hostName = args[2].trim();
        targetQMgr = args[3].trim();
        iniFile = args[4].trim();

        // Access the local MQe queue manager...
        MQeServer server = new MQeServer(iniFile, true // Supress any stdout information.
            );

        // Start up the server...
        server.activate(true);

        // Access the queue manager created in this JVM... the server QM
        MQeQueueManager myQMgr = MQeQueueManager.getDefaultQueueManager();

        // Create the queue...
        createMQQMgrProxy(bridgeName, proxyName, hostName, targetQMgr, myQMgr);

        // Ask the server to shut down peacefully...
        server.activate(false);

        // Close down the MQeClient
        server.close();
    } 

    /**
     * Creates an admin message which will create a bridge, and sends it
     * to a named queue manager.
     * 
     * @param bridgeName The name of the bridge for which the new proxy object will be created.
     * 
     * @param proxyName The name of the new MQ queue manager proxy object.
     *                  This will correspond to the name of the WMQ queue manager it will represent.
     * 
     * @param hostName The host name on which the WMQ queue manager is located.
     *                 A special value of "null" indicates that the WMQ queue manager is sited on the local host.
     *                 (and that the WMQ java bindings will be used under the covers, which offer improved performance
     *                 over specifying "localhost", "127.0.0.1" or the name of the local host machine).
     * 
     * @param targetQMgr The name of the queue manager on which the new proxy will be created.
     * 
     * @param myQMgr A reference to an active MQe queue manager which will be used to send and receive MQe
     *               administration messages and responses.
     * @return void
     * @exception Exception
     */
    protected void createMQQMgrProxy(String bridgeName, String proxyName, String hostName, String targetQMgr, MQeQueueManager myQMgr
    ) throws Exception {

        MQeMQQMgrProxyAdminMsg msg = new MQeMQQMgrProxyAdminMsg();

        // Create a fields object in which we can pre-initialise the bridge...
        MQeFields params = new MQeFields();

        // When we create the bridge, set the default transformer.
        if (hostName.equals("null")) {
            hostName = "";    // Force the config to use MQ java bindings 
            // rather than java client classes.
        }
        params.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_HOST_NAME, hostName
        );

        // Set the command in the admin msg to "create".
        msg.create(params);

        // Set the name of the object we wish to create into the admin msg...
        msg.setName(bridgeName, proxyName);

        System.out.println("Built the admin message.");

        // Send the message.
        MQeFields results = sendWaitMessage(myQMgr, targetQMgr, msg);
    }  
} 
