// Version 1.2 mqe_java/source/examples/mqbridge/administration/programming/MQAgent.java, MQeExamples, la200, la200-L030324.1

/*------------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms                *
 *                                                                              *
 * Licensed Materials - Property of IBM                                         *
 *                                                                              *
 * 5765-E63                                                                     *
 *                                                                              *
 * (C) Copyright IBM Corp. 2002                                                 *
 *------------------------------------------------------------------------------*/
package examples.mqbridge.administration.programming;


import com.ibm.mqe.*;
import java.util.*;
import com.ibm.mq.pcf.*;
import com.ibm.mq.*;
import java.io.*;


/**
 * An agent which can be used to manipulate the MQSeries queue manager.
 */
public class MQAgent {
    public static short[] version = { 2, 0, 0, 5 }; 

    /**
     * An active session to the MQSeries queue manager we're manipulating.
     */
    private PCFAgent agent;
	
    private Reporter reporter;

    private String mqQueueManagerHostName; 
	
    private String mqQueueManagerName;

    private int portMQListensOn = 1414;

    /**
     * Simple constructor. 
     * The MQ Agent is connected to the default MQSeries queue manager.
     */
    public MQAgent(Reporter reporter) throws Exception {
        super();
        this.reporter = reporter;

        try {
            MQQueueManager mqQueueManager = new MQQueueManager(null);

            agent = new PCFAgent(mqQueueManager);
            mqQueueManagerName = getDefaultMQQMgrName();
			
        } catch (Exception e) {
            throw new Exception("Failed to connect to the default Websphere MQ queue manager.\n"
                    + "This could indicate that the queue manager is not started,\n"
                    + "does not exist, or that a default queue manager has not been set."
                );
        }
		
        // We know that the MQSeries queue manager is local on this machine.
        // remember this fact so we can be asked.
        mqQueueManagerHostName = TcpipUtils.getHostName();
    }
	
    /**
     * Gets the hostname of the machine on which the MQSeries system resides, to which this MQ agent is connected.
     */
    public String getMQQueueManagerHostName() {
        return mqQueueManagerHostName;
    }
	
    /**
     * Gets the port to which this agent is connected, in order to talk to the MQSeries queue manager.
     */
    public int getPortMQListensOn() {
        return portMQListensOn;
    }

    /**
     * Gets the name of the MQ queue manager to which this MQAgent is connected.
     */
    public String getMQQueueManagerName() {
        return mqQueueManagerName;
    }

    /**
     * Finds the name of the current MQSeries queue manager which has been set as
     * the "default" queue manager.
     * @returns A String. The name of the default MQSeries queue manager, or null
     * if there is a problem getting the information in some way.
     */
    public String getDefaultMQQMgrName() {
        String name = null;

        reporter.enter("Getting the default MQ queue manager name");

        try {
            int[] attrs = { CMQC.MQCA_Q_MGR_NAME };
            // int [] 			attrs = { CMQCFC.MQIACF_ALL };
            PCFAgent agent;
            PCFParameter[] parameters =
                { new MQCFIL(CMQCFC.MQIACF_Q_MGR_ATTRS, attrs)};
            MQMessage[] responses;
            MQCFH cfh;
            PCFParameter p;

            reporter.comment("Opening the default MQ queue manager...");

            // Open the default queue manager...
            MQQueueManager qmgr = new MQQueueManager(null);

            reporter.comment("Opened the default MQ queue manager ok.");

            // Connect a PCFAgent to the specified queue manager
            reporter.comment("creating PCF agent...");
            agent = new PCFAgent(qmgr);

            reporter.comment("PCF agent Connected ok.");

            // Use the agent to send the request
            // Use the agent to send the request

            reporter.comment("Sending PCF request... ");
            responses = agent.send(CMQCFC.MQCMD_INQUIRE_Q_MGR, parameters);

            reporter.comment("Received reply.");
            cfh = new MQCFH(responses[0]);

            // Check the PCF header (MQCFH) in the first response message
            if (cfh.reason == 0) {
                reporter.comment("No failure reported.");

                // Walk through the returned attributes
                reporter.indent();
                for (int i = 0; i < cfh.parameterCount; i++) {
                    p = PCFParameter.nextParameter(responses[0]);
                    if (p.getParameter() == CMQC.MQCA_Q_MGR_NAME) {
                        name = ((String) p.getValue()).trim();
                    }
                    reporter.debug(
                        "attribute id "
                        + p.getParameter()
                        + "="
                        + p.getValue());
                }
                reporter.outdent();

            } else {
                reporter.error(cfh.toString());

                // Walk through the returned parameters describing the error

                reporter.indent();
                for (int i = 0; i < cfh.parameterCount; i++) {
                    reporter.error(
                        (PCFParameter.nextParameter(responses[0])).toString());
                }
                reporter.outdent();
            }

            // Disconnect

            reporter.comment("Disconnecting... ");
            agent.disconnect();
            reporter.comment("Done.");

            /* } catch (ArrayIndexOutOfBoundsException abe) {
             System.out.println ("Usage: \n" + 
             "\tjava GetDefaultMQQMgrName\n"
             );
             }
             */

        } catch (MQException mqe) {
            reporter.error(mqe);
        } catch (IOException ioe) {
            reporter.error(ioe);
        } catch (Throwable t) {
            reporter.error(t);
        }

        reporter.exit("Returning after attempt to find the name of the default MQ queue manager. Result=" + name);
        return name;
    }

    public static final boolean NOT_TRANSMIT_QUEUE = false;
    public static final boolean TRANSMIT_QUEUE = true;

    public static final boolean REPLACE = true;
    public static final boolean DO_NOT_REPLACE = false;

    public static final boolean PERSIST = true;
    public static final boolean NON_PERSTIST = false;

    /**
     * Use the passed agent to go create a local/transmission queue on the MQ system.
     * 
     * @param qName The name of the queue to create.
     * @param qDescription The descriptive text to be associated with the created queue.
     * @param isXmitQ A boolean. True if the local queue is a transmission queue, false otherwise.
     * Try using {@link NOT_TRANSMIT_QUEUE} or {@link TRANSMIT_QUEUE}
     * @param replace A boolean. If true, and the queue already exists, the queue definition 
     * will be replaced with this one. If false, and the queue already exists, the queue definition
     * will not be replaced.
     * Try using {@link #REPLACE} or {@link DO_NOT_REPLACE}
     * @param persist A boolean. The default persistence attribute associated with the local queue.
     * Try using {@link #PERSIST} or {@link #NON_PERSIST}
     */
    public boolean createMQQueueLocal(
        String qName,
        String qDescription,
        boolean isXmitQ,
        boolean replace,
        boolean persist)
        throws Exception {
        boolean isOk = true;

        reporter.enter("Attempting to create a local queue on MQSeries");

        try {

            PCFParameter[] tempParameters = new PCFParameter[MAX_PCF_PARAMETERS];
            int paramCounter = 0;

            // The name of the queue needs setting.
            tempParameters[paramCounter++] = new MQCFST(CMQC.MQCA_Q_NAME, qName);

            // It's a local queue.
            tempParameters[paramCounter++] =
                    new MQCFIN(CMQC.MQIA_Q_TYPE, CMQC.MQQT_LOCAL);

            // Whether messages put on this queue persist or not is controlled by the caller.
            if (persist) {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQC.MQIA_DEF_PERSISTENCE, CMQC.MQPER_PERSISTENT);
            } else {
                tempParameters[paramCounter++] =
                        new MQCFIN(
                            CMQC.MQIA_DEF_PERSISTENCE,
                            CMQC.MQPER_NOT_PERSISTENT);
            }

            // The fact on whether the queue is replaced if it already exists is controlled by the user.
            if (replace) {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES);
            } else {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_NO);
            }
	
            // The fact of whether the queue is a transmission queue is set by the user...
            if (!isXmitQ) {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQC.MQIA_USAGE, CMQC.MQUS_NORMAL);
            } else {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQC.MQIA_USAGE, CMQC.MQUS_TRANSMISSION);
            }
	
            // Don't set a description if it's a blank string.
            if (!qDescription.equals("")) {
                int length = qDescription.length();

                if (length > MQC.MQ_Q_DESC_LENGTH) {
                    qDescription = qDescription.substring(0, MQC.MQ_Q_DESC_LENGTH);
                    reporter.error("queue description parameter is too long."
                        + "supplied description is " + length + " characters. Max allowable is " + MQC.MQ_Q_DESC_LENGTH 
                        + "supplied value is " + qDescription
                        + "cropped to value of " + qDescription
                        + "cropped description is of length " + qDescription.length() 
                        + "continuing with the cropped description."
                    );
                }	
                tempParameters[paramCounter++] = new MQCFST(CMQC.MQCA_Q_DESC, qDescription);
            }
	
            // Copy the parameters into an array which is only big enough to house them, not bigger.
            PCFParameter[] parameters = new PCFParameter[paramCounter];

            for (int i = 0; i < paramCounter; i += 1) {
                parameters[i] = tempParameters[i];
            }
	
            isOk = sendPCFMessage(CMQCFC.MQCMD_CREATE_Q, parameters);
        } finally {
            reporter.exit("Exiting after attempting to create a local queue on MQSeries");
        }

        return isOk;
    }

    /**
     * Sends the PCF message to MQ, and interprets the result into a pass/fail.
     * 
     * @return true if the PCF message worked ok, false if there was a problem.
     */
    private boolean sendPCFMessage(int command, PCFParameter[] parameters) {

        boolean isOk = true;

        try {
            reporter.enter("Sending the PCF message...");

            MQMessage[] responses;
            MQCFH cfh;
            PCFParameter p;

            // Use the agent to send the request

            // System.out.print ("Sending PCF request... ");
            responses = agent.send(command, parameters);
            // System.out.println ("Received reply.");
            cfh = new MQCFH(responses[0]);

            // Check the PCF header (MQCFH) in the first response message
            if (cfh.reason == 0) {
                reporter.comment("PCF message worked.");
				
                /*
                 comment(cfh.toString());
                 // Walk through the returned attributes
                 for (int i = 0; i < cfh.parameterCount; i++) {
                 p = PCFParameter.nextParameter(responses[0]);
                 comment(
                 "\t attribute id "
                 + p.getParameter()
                 + "="
                 + p.getValue());
                 }
                 */
                isOk = true;
            } else {
                isOk = false;
                reporter.error("MQ failed to create queue. reason code=" + cfh.reason);
                reporter.error(cfh.toString());
                for (int i = 0; i < cfh.parameterCount; i++) {
                    reporter.error((PCFParameter.nextParameter(responses[0])).toString());
                }
            }
        } catch (Exception e) {
            reporter.error(
                "Failure when sending/receiving PCF command: " + e.toString());
            reporter.exception(e);
        } finally {
            reporter.exit("returning after sending the PCF comment.");
        }
        return isOk;
    }

    public static final String NO_REMOTE_Q_NAME_SET = "";

    /**
     * Use the passed agent to go create a remote queue on the MQ system.
     * 
     * @param qName The name of the remote queue definition to be created.
     * @param qDescription The descriptive text to be associated with the queue.
     * @param xmitQ The name of the xmit queue to which messages sent using this remote
     * queue will be routed.
     * @parmam replace Should we replace a definition which already exists ?
     * @param persist Default persistence setting for the remote queue.
     * @param remoteQ The remote q name to be set into the remote queue definition. 
     * Use NO_REMOTE_Q_NAME_SET if you don't want to set this parameter.
     * @param remoteQMgr The name of the remote queue manager to which this remote queue definition refers.
     * Use "" if you don't want to set this parameter.
     */
    public boolean createMQQueueRemote(
        String qName,
        String qDescription,
        String xmitQ,
        boolean replace,
        boolean persist,
        String remoteQ,
        String remoteQMgr)
        throws Exception {
        boolean isOk = true;
		
        try {
            reporter.enter("Create a remote queue definition on the MQSeries system.");

            //System.out.println("Debug: Agent "+agent+" creating a remote queue defn "+qName
            //                   +" description="+qDescription+" xmitq="+xmitQ+" replace="+replace
            //                   +" persist="+persist+" remoteQ="+remoteQ+" remoteQMgr="+remoteQMgr
            //                   );

            int[] attrs = { CMQC.MQCA_Q_MGR_NAME };
            // int [] 			attrs = { CMQCFC.MQIACF_ALL };
	
            PCFParameter[] tempParameters = new PCFParameter[MAX_PCF_PARAMETERS];
            int paramCounter = 0;
	
            tempParameters[paramCounter++] = new MQCFST(CMQC.MQCA_Q_NAME, qName);
            tempParameters[paramCounter++] =
                    new MQCFIN(CMQC.MQIA_Q_TYPE, CMQC.MQQT_REMOTE);
            tempParameters[paramCounter++] =
                    new MQCFST(CMQC.MQCA_XMIT_Q_NAME, xmitQ);
	
            // Whether the queue default is persist or not is controlled by the user...
            if (persist) {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQC.MQIA_DEF_PERSISTENCE, CMQC.MQPER_PERSISTENT);
            } else {
                tempParameters[paramCounter++] =
                        new MQCFIN(
                            CMQC.MQIA_DEF_PERSISTENCE,
                            CMQC.MQPER_NOT_PERSISTENT);
            }
	
            // The fact on whether the queue is replaced if it already exists is controlled by the user...
            if (replace) {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES);
            } else {
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_NO);
            }

            // Don't set a description if it's a blank string.
            if (!qDescription.equals("")) {			
                int length = qDescription.length();

                if (length > MQC.MQ_Q_DESC_LENGTH) {
                    qDescription = qDescription.substring(0, MQC.MQ_Q_DESC_LENGTH);
                    reporter.error("queue description parameter is too long."
                        + "supplied description is " + length + " characters. Max allowable is " + MQC.MQ_Q_DESC_LENGTH 
                        + "supplied value is " + qDescription
                        + "cropped to value of " + qDescription
                        + "cropped description is of length " + qDescription.length()
                        + "continuing with the cropped description."
                    );
                }	
                tempParameters[paramCounter++] = new MQCFST(CMQC.MQCA_Q_DESC, qDescription);
            }
	
            // Don't set a remote queue name if it's a blank string.
            if (!remoteQ.equals(NO_REMOTE_Q_NAME_SET)) {
                tempParameters[paramCounter++] =
                        new MQCFST(CMQC.MQCA_REMOTE_Q_NAME, remoteQ);
            }
            // Don't set a remote queue manager name if it's a blank string.
            if (!remoteQMgr.equals("")) {
                tempParameters[paramCounter++] =
                        new MQCFST(CMQC.MQCA_REMOTE_Q_MGR_NAME, remoteQMgr);
            }
	
            // Copy the parameters into an array which is only big enough to house them, not bigger.
            PCFParameter[] parameters = new PCFParameter[paramCounter];

            for (int i = 0; i < paramCounter; i += 1) {
                parameters[i] = tempParameters[i];
            }
	
            isOk = sendPCFMessage(CMQCFC.MQCMD_CREATE_Q, parameters);
        } finally {
            reporter.exit("returning after attempt to create a remote MQ queue definition. rc=" + isOk);
        }
        return isOk;
    }

    /**
     * A "worst-case" max number of PCF parameters we want to stuff into a PCF message.
     * 
     * <p>Used to make the code more readable.
     */
    private static final int MAX_PCF_PARAMETERS = 20;

    /**
     * Use the passed agent to go create a queue on the MQ system.
     * 
     * @param chName The name of the channel to create.
     * @param chDescription The descriptive text associated with the channel. Null or blank string 
     * if you don't want to set the description.
     * @param replace Should we replace a definition if one already exists ?
     */
    public boolean createMQServerConnectionChannel(
        String chName,
        String chDescription,
        boolean replace)
        throws Exception {
			
        boolean isOk = true;

        reporter.enter(
            "Agent "
            + agent
            + " creating a server connection channel "
            + chName
            + " description="
            + chDescription
            + " replace="
            + replace);
        try {				

            int maxDescLength = MQC.MQ_PROCESS_DESC_LENGTH;

            if (chDescription.length() > maxDescLength) {
                chDescription = chDescription.substring(0, maxDescLength);
                reporter.error("Desrciption is too long. Length given:" + chDescription.length() + " max allowed:" + maxDescLength
                    + "cropped to '" + chDescription + "'"
                    + "cropped data has length of " + chDescription.length()
                    + "continuing with cropped data."
                );
            }
	
            if (chName.length() > MQC.MQ_CHANNEL_NAME_LENGTH) {
                reporter.error(
                    "Channel name "
                    + chName
                    + " is too long. Length given:"
                    + chName.length()
                    + " max length:"
                    + MQC.MQ_CHANNEL_NAME_LENGTH);
                isOk = false;
            } else {
	
                PCFParameter[] tempParameters =
                    new PCFParameter[MAX_PCF_PARAMETERS];
                int paramCounter = 0;
	
                tempParameters[paramCounter++] =
                        new MQCFST(CMQCFC.MQCACH_CHANNEL_NAME, chName);
                tempParameters[paramCounter++] =
                        new MQCFIN(CMQCFC.MQIACH_CHANNEL_TYPE, CMQXC.MQCHT_SVRCONN);
	
                // The fact on whether the queue is replaced if it already exists is controlled by the user...
                if (replace) {
                    tempParameters[paramCounter++] =
                            new MQCFIN(CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_YES);
                } else {
                    tempParameters[paramCounter++] =
                            new MQCFIN(CMQCFC.MQIACF_REPLACE, CMQCFC.MQRP_NO);
                }
	
                // Add the description comment if its' around.
                if ((chDescription != null) && (!chDescription.equals(""))) {
                    // A description has been passed. 
                    // Pass it to MQ.
                    tempParameters[paramCounter++] =
                            new MQCFST(CMQCFC.MQCACH_DESC, chDescription);
                }
	
                // Copy the parameters into an array which is only big enough to house them, not bigger.
                PCFParameter[] parameters = new PCFParameter[paramCounter];

                for (int i = 0; i < paramCounter; i += 1) {
                    parameters[i] = tempParameters[i];
                }
	
                // Use the agent to send the request
                isOk = sendPCFMessage(CMQCFC.MQCMD_CREATE_CHANNEL, parameters);
            }
        } finally {	
            reporter.exit("exiting after attempt to create a server connection channel. rc=" + isOk);
        }
        return isOk;
    }

}
