/*------------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.application;


import com.ibm.mq.*;               
               

/**
 * Tool to destructively read any message appearing on a specified WebSphere MQ queue.
 *
 * <P><B>Syntax : </B></P>
 * <TT>java examples.mqbridge.application.GetFromMQ 
 * &lt;mq_q_mgr&gt; &lt;mq_q&gt &lt;display_flag&gt;</TT>
 *
 * <P><B>Parameters : </B></P>
 * <DL>
 * <DT>mq_q_mgr : </DT>
 * <DD>The name of the MQSeries queue manager to get the messages from.
 * As this programs uses java bindings classes to access the MQ qmgr, the
 * MQ queue manager needs to be on the same machine as the test program.</DD>
 *
 * <DT>mq_q : </DT>
 * <DD>The name of the MQSeries queue from which messages will be taken and consumed.</DD>
 *
 * <DT>display_flag : </DT>
 * <DD>One of the following strings...
 * <DL>
 * <DT>-contents</DT><DD>Displays the content of the message.</DD>
 * <DT>-nocontents</DT><DD>Does not display the content of the message.</DD>
 * </DL></DD>
 *
 * </DL>
 *
 * <P><B>How to stop : </B></P>
 * <P>Hit Ctrl-C to stop this program.</P>
 * 
 */
public class GetFromMQ {
    public static short version[] = { 2, 0, 0, 5 }; 

    /**
     * How long we wait for a message to arrive.
     */
    public static final int WAIT_INTERVAL_MILLISECONDS = 1000 * 1000; // 1000 seconds

    /**
     * How should a user invoke this tool ?
     */
    public static String syntax = 
        "Syntax:\n"
        + "  java GetFromMQ <mq_q_mgr> <mq_q> <display_flag>\n"
        + "Parameters:\n"
        + "  mq_q_mgr:\n"
        + "    The name of the MQSeries queue manager to get the messages from.\n"
        + "    As this programs uses java bindings classes to access the MQ qmgr, the\n"
        + "    MQ queue manager needs to be on the same machine as the test program.\n"
        + "  mq_q:\n"
        + "    The name of the MQSeries queue from which messages will be taken and consumed.\n"
        + "  display_flag:\n"
        + "    one of the following strings...\n"
        + "      -contents\n"
        + "        displays the content of the message.\n"
        + "      -nocontents\n"
        + "        does not display the content of the message.\n"
        + "How to stop:\n"
        + "  Hit Ctrl-C to stop this program.\n";

    /**
     * The entry point from the command-line.
     */
    public static void main(String args[]) {
        try {
            GetFromMQ me = new GetFromMQ();

            me.activate(args);
        } catch (Exception e) {
            System.err.println("Error: Exception occurred: " + e.toString());
            // e.printStackTrace();
        }
    }

    /**
     * The work of the tool is done here.
     * @param args Arguments
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(String args[]) throws Exception {

        if (args.length < 3) {
            throw new Exception("Syntax Error: Not enough parameters.\n" + syntax); 
        }

        if (args.length > 3) {
            throw new Exception("Syntax Error: Too many parameters.\n" + syntax);
        }

        // Extract the command-line parameters into variables...
        String qmgrName = args[0];
        String qName = args[1];
        String displayFlag = args[2];

        boolean displayContents = false;

        if (displayFlag.equals("-contents")) {
            displayContents = true;
        } else {
            if (displayFlag.equals("-nocontents")) {
                displayContents = false;
            } else {
                throw new Exception("Syntax Error: display_flag value invalid (" + displayFlag + ").\n" + syntax);
            }
        }

        MQEnvironment env = new MQEnvironment();

        env.hostname = ""; // Use bindings.

        // Open the MQSeries queue manager...
        MQQueueManager qmgr = new MQQueueManager(qmgrName);

        // Access the queue for reading...
        int openOptions = MQC.MQOO_INPUT_SHARED;
        MQQueue q = qmgr.accessQueue(qName, openOptions);

        // A counter so we can count the messages as they arrive...
        int counter = 0;

        // We will use the following options to get messages...
        MQGetMessageOptions getOptions = new MQGetMessageOptions();

        getOptions.options = MQC.MQGMO_WAIT;
        getOptions.waitInterval = WAIT_INTERVAL_MILLISECONDS;

        // Loop until we're done.
        boolean done = false;

        while (!done) {
            // Get the next message, but wait if there are no messages to get yet...
            MQMessage msg = new MQMessage(); // Create empty message.

            q.get(msg, getOptions);

            // Get the current timestamp so we can print it out.
            long milliSecTime = System.currentTimeMillis();
            int  payloadLength = msg.getDataLength();

            System.out.println(milliSecTime + " Message " + counter + " arrived. data payload is "
                + Integer.toString(payloadLength) + " bytes long.");

            if (displayContents) {
                // Work through the message, converting each byte to a printable ascii character...
                int bytes = payloadLength;
                String byteAsAString;
                String messageAsString = "";
                String messageAsHex = "";

                while (bytes > 0) {
                    int byteValueAsInt = msg.readUnsignedByte();

                    messageAsString += byteToAscii((byte) (byteValueAsInt & 0x00FF));
                    messageAsHex += "0x" + byteToHex((byte) (byteValueAsInt & 0x00FF)) + " ";
                    bytes -= 1;
                }
    
                // Work through the resultant strings, outputting slices of each at a time.
                while (messageAsString.length() > 0) {
    
                    // Make sure that there are enough characters to fill a line of output.
                    while (messageAsString.length() < BYTES_PER_LINE) {
                        messageAsString += " ";
                        messageAsHex += BLANK_AS_HEX;
                    }
    
                    String asciiPart = messageAsString.substring(0, BYTES_PER_LINE);
                    String hexPart = messageAsHex.substring(0, BYTES_PER_LINE * CHARS_PER_HEX_BYTE);
    
                    System.out.println("     " + hexPart + "    \"" + asciiPart + "\"");
    
                    messageAsString = messageAsString.substring(BYTES_PER_LINE);
                    messageAsHex = messageAsHex.substring(BYTES_PER_LINE * CHARS_PER_HEX_BYTE);
                }
            }

            // Bump the counter of messages we keep up for next time...
            counter += 1;
        }
    }

    public static final int BYTES_PER_LINE = 16;
    public static final int CHARS_PER_HEX_BYTE = 5;
    public static final String BLANK_AS_HEX = ".... ";

    /**
     * Converts a byte array to Ascii string.
     * If the byte is a non-printable character, return the '.'character instead.
     * @param data Byte to be converted to Ascii
     * @return Ascii String 
     */
    public static String byteToAscii(byte data) {
        char tempC[] = new char[1];   
        String result;

        if ((data < 32) || (data > 127)) {
            tempC[0] = '.';
        } else {
            tempC[0] = (char) (data & 0x00FF);       /* convert to char      */
        }
        result = new String(tempC);
        return result;
    }

    public  final static char hex[] = { '0', '1', '2', '3', '4', '5', '6', '7',
            '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };

    /** 
     * Given a byte value, converts it into it's hex equivalent.
     * @param data Byte to be converted to Hex
     * @return Hex String.
     */
    public static String byteToHex(byte data) {
        StringBuffer result = new StringBuffer(4);

        result.append(hex[(data >> 4) & 15]); 
        result.append(hex[ data & 15]);    
        return (result.toString());   
    }    
}

