/*------------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.application;


import com.ibm.mq.*;               
               

/**
 * Tool to put a message to an MQSeries queue, such that the user can specify the
 * target queue and the target queue manager.
 * 
 * <P>Each message generated contains a message body which is a single unicode string,
 * as specified by the user on the command-line, followed by a number of bytes needed to pad-out
 * the message body to a specific size. All padding bytes have a value of zero.</P>
 * 
 * <P><B>Syntax : </B></P>
 *
 * <TT>java examples.mqbridge.application.PutFromMQ 
 * &lt;mq_q_mgr&gt; &lt;target_q&gt; &lt;target_qmgr&gt;
 * &lt;msgs&gt; &lt;size&gt; &lt;text&gt;</TT>
 *
 * <P><B>Parameters : </B></P>
 * <DL>
 *
 * <DT>mq_q_mgr : </DT>
 * <DD>The name of the MQSeries queue manager to connect to, from which messages are sent.
 * As this programs uses java bindings classes to access the MQ qmgr, the
 * MQ queue manager needs to be on the same machine as the test program. </DD>
 * 
 * <DT>target_q : </DT>
 * <DD>The name of the queue to which messages will sent.</DD>
 *                                                      
 * <DT>target_qmgr : </DT>
 * <DD>The queue manager on which the target queue resides.</DD>

 * <DT>msgs : </DT>
 * <DD>The number of messages to send. An integer.
 * If <=0 then a continual stream of messages will be generated as quickly as possible.
 * (so you will need to Ctrl-C out of the program to stop it).</DD>
 * 
 * <DT>size : </DT>
 * <DD>The size of the message payload to be sent.
 * This must be an integer.
 * The text message (see below) specified by the user is written to the message body,
 * followed by a number of bytes needed to pad-out the message body to this specific size.
 * A value of <=0 means don't add any padding bytes to the message.</DD>
 * 
 * <DT>text : </DT>
 * <DD>A unicode string which will be put into the message body of the message.
 * Note that with some shells, you may need to use quote characters if the
 * text string contains spaces.</DD>
 * 
 * </DL>
 * 
 * <P><B>Output :</B></P>
 * <P>The program will output one line of text for each message sent.
 * The line of text contains a millisecond-representation of the current system time
 * followed by the number of messages sent so far.</P>
 */
public class PutFromMQ {
    public static short version[] = { 2, 0, 0, 5 }; 

    /**
     * The syntax string.
     * Displayed to the caller if incorrect number of parameters are given.
     */
    public static String syntax = 
        "Syntax:\n"
        + "  java examples.mqbridge.application.PutFromMQ <mq_q_mgr> <target_q> <target_qmgr> <msgs> <size> <text>\n"
        + "Parameters:\n"
        + "  mq_q_mgr:\n"
        + "    The name of the MQSeries queue manager to connect to, from which messages are sent.\n"
        + "    As this programs uses java bindings classes to access the MQ qmgr, the\n"
        + "    MQ queue manager needs to be on the same machine as the test program.\n"
        + "  target_q:\n"
        + "    The name of the queue to which messages will sent.\n"
        + "  target_qmgr:\n"
        + "    The queue manager on which the target queue resides.\n"
        + "  msgs:\n"
        + "    The number of messages to send. An integer.\n"
        + "    If <=0 then a continual stream of messages will be generated as quickly as possible.\n"
        + "    (so you will need to Ctrl-C out of the program to stop it).\n"
        + "  size:\n"
        + "    The size of the message payload to be sent.\n"
        + "    This must be an integer.\n"
        + "    The text message (see below) specified by the user is written to the message body,\n"
        + "    followed by a number of bytes needed to pad-out the message body to this specific size.\n"
        + "    A value of <=0 means don't add any padding bytes to the message.\n"
        + "  text:\n"
        + "    A unicode string which will be put into the message body of the message.\n"
        + "    Note that with some shells, you may need to use quote characters if the\n"
        + "    text string contains spaces.\n";

    /**
     * The entry point to the program from the command-line.
     * 
     * @param args   The command-line arguments.
     *               See the documentation for the class for descriptions of 
     *               what parameters are expected.
     */
    public static void main(String args[]) {
        try {
            PutFromMQ me = new PutFromMQ();

            me.activate(args);
        } catch (Exception e) {
            System.err.println("Error: Exception occurred: " + e.toString());
            // e.printStackTrace();
        }
    }

    /**
     * Called to do the work of the program, from the main.
     * 
     * @param args   The arguments passed from the command-line arguments.
     *               See the documentation for the class for details of what these
     *               arguments should contain.
     * @exception Exception
     */
    public void activate(String args[]) throws Exception {

        if (args.length < 6) {
            throw new Exception("Syntax Error: Not enough parameters.\n" + syntax); 
        }

        if (args.length > 6) {
            throw new Exception("Syntax Error: Too many parameters.\n" + syntax);
        }

        // Extract the command-line parameters into variables...
        String qmgrName = args[0];
        String targetQ = args[1];
        String targetQMgr = args[2];
        int msgs;          // use args[3] later...
        int msgSize;       // use args[4] later...
        String msgText = args[5];

        // The next parameter is an int, which may fail to parse from the string given by the user...
        try {
            msgs = Integer.parseInt(args[3]);
        } catch (Exception e) {
            throw new Exception("Integer parameter <msgs> is not an integer.\n" + syntax);
        }

        // The next parameter is an int, which may fail to parse from the string given by the user...
        try {
            msgSize = Integer.parseInt(args[4]);
        } catch (Exception e) {
            throw new Exception("Integer parameter <size> is not an integer.\n" + syntax);
        }

        MQEnvironment env = new MQEnvironment();

        env.hostname = ""; // Use bindings.

        // Open the MQSeries queue manager...
        MQQueueManager qmgr = new MQQueueManager(qmgrName);

        // Access the queue for reading...
        int openOptions = MQC.MQOO_OUTPUT;
        MQQueue q = qmgr.accessQueue(targetQ, openOptions, targetQMgr, null  // No dynamic queue name required.
                , null  // No alternate user-id.
            );

        // A counter so we can count the messages as they are sent...
        int counter = 0;

        // We will use the following options to put messages...
        MQPutMessageOptions putOptions = new MQPutMessageOptions();

        putOptions.options = MQC.MQPMO_NONE | MQC.MQPMO_NEW_MSG_ID | MQC.MQPMO_NEW_CORREL_ID;

        // Work out the array of bytes which are needed to pad-out the message to 
        // ensure the message is the size required by the user.
        int textBytes = msgText.length() * 2; // x2 bytes for each unicode character.
        int padding = 0; // The number of padding bytes we require.

        if (textBytes > msgSize) {
            padding = 0;
        } else {
            padding = msgSize - textBytes;
        }
        // Define the padding byte array if needed.
        byte paddingBytes[] = new byte[1];

        if (padding > 0) {
            paddingBytes = new byte[padding];
        }

        // The length of the message.
        int msgLength = padding + textBytes;

        // Loop until we're done.
        boolean done = false;

        while (!done) {
            // Get the next message, but wait if there are no messages to get yet...
            MQMessage msg = new MQMessage(); // Create empty message.

            // Write the user-specified text message to the message body.
            msg.writeChars(msgText);

            // Optionally pad the message body to the required size if padding is required.
            if (padding > 0) {
                msg.write(paddingBytes);
            }

            // Put the message.
            q.put(msg, putOptions);

            // Get the current timestamp so we can print it out.
            long milliSecTime = System.currentTimeMillis();

            // Bump the counter of messages we keep up for next time...
            counter += 1;

            System.out.println("Time:" + milliSecTime + " Written message " + counter + " size:" + msgLength + " bytes.");

            // Check to see if we've sent enough messages yet.
            if ((counter >= msgs) && (msgs > 0)) {
                done = true;
            }
        }
    }
}

