/*------------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.rules;


import com.ibm.mqe.mqbridge.*;
import com.ibm.mqe.*;
import com.ibm.mq.*;
import java.text.*;
import java.util.*;


/******************************************************************************
 *
 * To be a model rules class for the sync queue purger.
 *
 * This is the basic class from which dictates what action to take upon
 * messages that are on the sync queue.
 * Whenever purging of the sync queue takes place, the bridge calls the
 * permit() method for each unconfirmed message that is currently on the sync
 * queue. This rules class then decides if this is an old message that is no
 * longer relevant, and thus if it can be safely purged by the bridge.
 *
 * <p> This class simply generates warnings for any messages that are over ten
 * minutes old. </p>
 *
 ******************************************************************************/
public class MQeSyncQueuePurgerRule extends MQeRule {

    public        static short    version[] = { 2, 0, 0, 5 }; 

    /**
     * The threshold for reporting a warning (in minutes)
     */
    private static final long MINUTES_BEFORE_WARN = 10;
    
    /**
     * Used to convert the above parameter into seconds
     */
    private static final long SECONDS_IN_A_MINUTE = 60;

    /**
     * Decides whether a given message can be deleted.
     * This implementation of the rule simply generates a warning
     * for messages that are over ten minutes old.
     * @param MQMessage the message object in question
     * @param Long the age of the message in seconds.
     * @param String the name of the sync queue being purged. 
     * @return Boolean true if the message can be deleted.
     */
    public Object permit(Object messageO, Object ageO, Object syncQNameO
    ) throws Exception {

        // Check that all the parameters are of the correct type...
        if ((!(messageO instanceof MQMessage)) 
            || (!(ageO instanceof Long))
            || (!(syncQNameO instanceof String))) {
            throw new MQeException(Except_Rule, "Disallowed by rule", "Parameters have incorrect types"
                );
        }    

        // Create a strongly typed version of the parameters...
        MQMessage message = (MQMessage) messageO;
        long age = ((Long) ageO).longValue();
        String syncQName = (String) syncQNameO;
        
        if (age > (MINUTES_BEFORE_WARN * SECONDS_IN_A_MINUTE)) {

            Date gcDate = message.putDateTime.getTime();

            SimpleDateFormat formatter = new SimpleDateFormat("hh:mm:ss 'on' dd MMMM, yyyy.");
            String dateString = formatter.format(gcDate);
	    
            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, syncQName, dateString);
        }
        
        // However old it is, we don't want to delete it!
        return new Boolean(false);
    }
}
