/*------------------------------------------------------------------------------*/

/*                    MQSeries Everyplace for Multiplatforms                    */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/*                                                                              */

/*------------------------------------------------------------------------------*/
package examples.mqbridge.rules;


import com.ibm.mqe.mqbridge.*;
import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;


/******************************************************************************
 *
 * To be a model rules class.
 *
 * This is the basic class from which dictates what action a listener 
 * administered MQBridge object should do if the message is undelivered for
 * a specific reason. 
 *
 * <p>The class name will be placed in the configuration of the listener object.
 *
 * <p>The class will be loaded dynamically if the listener cannot deliver 
 * a message.
 *
 * <p>This rules class must decide which action to perform: Either to retry
 * after a number of seconds of waiting, moving the message to the MQ dead
 * letter queue, or stopping the listener altogether.
 *
 * <p>This model class has the some behaviour as an example: It will
 * return a value of 
 * <ul>
 * <li>5 seconds for the first minute
 * <li>10 seconds for the second minute
 * <li>60 seconds for the third to ninth minute inclusive
 * <li>STOP_LISTENER after retries have failed for 10 minutes.
 * </ul>
 *
 ******************************************************************************/
public class MQeUndeliveredMessageRule extends MQeRule {

    public        static short    version[] = { 2, 0, 0, 5 }; 

    /** 
     * The value that can be returned which will cause the listener to stop.
     */
    public static final int STOP_LISTENER = -1;

    /**
     * The value that can be returned which will cause the listener to 
     * put the message onto the MQ dead letter queue, or discard it
     * (depending on the report options set inside the MQ message).
     */
    public static final int USE_MQ_REPORT_OPTIONS = -2;

    /*
     * No constructor, so MQeUndeliveredMessageRule() is provided by default.
     */


    /*************************************************************************
     *
     * To find out what behaviour should occur in the listener when 
     * a message cannot be delivered, but may benefit from a retry
     * at a later date.
     *
     * <p>The listener has a number of conditions under which a message
     * cannot be delivered to the MQe network.</p>
     *
     * <p>Whenever an MQ Message cannot be put to MQe a rule class like this
     * can decide on which course of action to take.</p>
     *
     * @param failureCodes int[] with the following two elements:
     * <ul>
     * <li>failureCodes[0] - Failure Reason. The reason why the message cannot 
     * currently be delivered to the target queue. This is one of the 
     * MQe.Except_* reasons from the list above.
     * <li>failureCodes[1] - Consecutive Failures. The number of failures to 
     * deliver that have occurred in a row. Each time this rule class has
     * been consulted to find out the required behaviour. Each time the
     * rule class has specified a timeout. This parameter holds the number of 
     * times the loop has occurred.
     * This parameter may be useful if you wish to specify a different
     * timeout for each retry, which increases with the number of retries
     * attempted, until a certain point at which the listener stops...etc.
     * </ul>
     *
     * @param targetNames String[] with the following two elements:
     * <ul>
     * <li>targetNames[0] = The name of the MQe QMgr that rejected the put.
     * <li>targetNames[1] = The name of the MQe queue that rejected the put.
     * </ul>
     *
     * @param failedMQeMessage A reference to the MQe Message that could
     * not be delivered to the target MQe queue manager. This may be null
     * if the MQe message had not yet been transformed (if an error occurred
     * during the transformation, for example).
     *
     * @return The required behaviour. An Integer.
     * The behaviour the listener should exibit following the undelivered
     * message condition can be specified by returning a different value
     * from this rule class as follows:
     * <dl>
     * <li>MQeUndeliveredMessageRule.STOP_LISTENER
     * <p>The listener should stop as a result of this message being 
     * undeliverable.
     * <li>MQeUndeliveredMessageRule.USE_MQ_REPORT_OPTIONS
     * <p>The message should either be discarded or moved to the MQ dead
     * letter queue, depending on the value of the original 'report' field
     * of the original MQ Message.
     * <li>>0 
     * <p>The number of seconds which the listener should wait before
     * retrying the MQ-to-MQe transfer operation.
     * </ul>
     * <p>If a value not in the list above is specified, or if the rule
     * throws an exception, then the listener will assume the STOP_LISTENER
     * result was returned.
     *
     * @exception Fails if the rule has some problem with it's logic. 
     * This will have the same result as returning the STOP_LISTENER value
     * in terms of how the listener will react.
     *
     *************************************************************************/
    public Object permit(Object failureCodesO          // int[]
        , Object targetNamesO           // String[]
        , Object failedMQeMessageO      // MQeMsgObject
    ) throws Exception {

        if (failedMQeMessageO != null) {
            // Check that all the parameters are of the correct type...
            if ((!(failureCodesO instanceof int[])) 
                || (!(targetNamesO instanceof String[]))
                || (!(failedMQeMessageO instanceof MQeMsgObject))
            ) {
                throw new MQeException(Except_Rule, "Disallowed by rule", "Parameters have incorrect types"
                    );
            }
        } else {
            if ((!(failureCodesO instanceof int[])) 
                || (!(targetNamesO instanceof String[]))
            ) {
                throw new MQeException(Except_Rule, "Disallowed by rule", "Parameters have incorrect types"
                    );
            }
        }

        // Create strongly typed versions of the parameters...
        int[]               failureCodes = (int[]) failureCodesO;
        String[]            targetNames = (String[]) targetNamesO;
        MQeMsgObject        failedMQeMessage = null;

        if (failedMQeMessageO != null) {
            failedMQeMessage = (MQeMsgObject) failedMQeMessageO;
        }

        // Work out the behaviour we want the listener to have...
        Integer result = permitUndeliveredMessage(failureCodes, targetNames, failedMQeMessage
            );

        // return the result...
        return (Object) result;
    }

    /***************************************************************************
     *
     * Exactly the same as the permit() method on this class, except the 
     * parameters are provided in strongly-typed form.
     *
     * <p>This model class has the some behavior as an example: It will
     * return a value of 
     * <ul>
     * <li>5 seconds for the first minute
     * <li>10 seconds for the second minute
     * <li>60 seconds for the third to ninth minute inclusive
     * <li>STOP_LISTENER after retries have failed for 10 minutes.
     * </ul>
     *
     ***************************************************************************/
    protected Integer permitUndeliveredMessage(int[]              failureCodes, String[]           targetNames, MQeMsgObject       failedMQeMessage
    ) throws Exception {
        
        // Create the Integer reference we will put the result into...
        Integer result = null;
        int failures;

        // If the failureCodes array is invalid something is seriously wrong.
        try {
            failures = failureCodes[1];
        } catch (ArrayIndexOutOfBoundsException ex) {
            failures = 30;
        }

        // For the first miniute, retry the transfer operation after 5 seconds of sleeping...
        if (failures <= 12) {
            result = new Integer(5);  
        } else {
            // For the second minute, retry every 10 seconds...
            if (failures <= 18) {
                result = new Integer(10);
            } else {
                // for the third to ninth minute (inclusive) retry after every 60 seconds...
                if (failures <= 26) {
                    result = new Integer(60);
                } else {
                    // If the problem still exists after ten minutes then stop the listener...
                    result = new Integer(STOP_LISTENER);
                }
            }
        }

        // Return the result.
        return result;
    }
}
