/*------------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/
package examples.mqbridge.transformers;


import com.ibm.mqe.mqbridge.*; 
import com.ibm.mqe.mqemqmessage.MQeMQMsgObject; 
import com.ibm.mqe.*;  
import com.ibm.mq.*;
import java.util.*;
import java.io.*;


/*****************************************************************************
 *
 * A simple MQ to MQe transformer class.
 *
 *****************************************************************************/

public class MQeBaseTransformer implements MQeTransformerInterface {

    /**
     * Version number of this class
     */
    public static short version[] = { 2, 0, 0, 5 }; 

    // The next two member variables are used in the MQ MsgType field.

    /**
     * An MQe encoded message that originated from MQe
     */
    public static final int MQMT_MQE_FIELDS_FROM_MQE = 112;

    /**
     * An MQe encoded message that did not originate from MQe.
     */
    public static final int MQMT_MQE_FIELDS = 113;

    /**
     * Tells the transformer of any parameters that were supplied
     * with the transformer definition. This transformer does not use
     * any of the supplied parameters.
     * @param StringTokenizer containing the transformer
     * definition parameters.
     * @exception java.lang.Exception If the parameters are invalid or
     * the transformer cannot be initialised.
     * @exception java.lang.Exception
     */
    public void activate(StringTokenizer params) throws Exception {}

    /***************************************************************************
     * Convert a MQ format message into a MQe format message. This method is
     * called by the MQ/MQe bridge.
     *
     * @param msg            The MQSeries message object we are to transform.
     * @param remoteQMgrName The Destination Remote Queue Manager Name
     *                       (from the MQ Remote Queue Definition)
     * @param remoteQName    The Destination Remote Queue Name (from the
     *                       MQ Remote Queue Definition).
     * @return               The equivalent MQe format message.
     * @exception java.lang.Exception  propagated from the called methods.
     ***************************************************************************/
    public MQeMsgObject transform(MQMessage msg,
        String remoteQMgrName,
        String remoteQName) throws Exception {

        MQeMsgObject result = null;

        if (msg.messageType == MQMT_MQE_FIELDS_FROM_MQE ||
            msg.messageType == MQMT_MQE_FIELDS) {

            // This is a MQe encoded message.
            byte[] userData = new byte[msg.getDataLength()];

            msg.readFully(userData);

            if (msg.messageType == MQMT_MQE_FIELDS_FROM_MQE) {
                // This message originated from MQe. All we have to do is
                // rebuild the MQE message object from the MQ user data.
                result = new MQeMsgObject(userData);
            } else { // (msg.messageType == MQMT_MQE_FIELDS)
                // This message did not originate from MQe. The message UID
                // (timestamp and originating Queue Manager) may not
                // be set, so we have to create it. The empty constructor for
                // MQeMsgObject() adds the timestamp to the message, and the
                // Originating Queue Manager will be added when the message is put.
                result = new MQeMsgObject();
                result.restore(userData);
            }

            // add the priority if it's been set explicitly
            if (msg.priority != MQeMQMsgObject.DEF_PRIORITY)
                result.putByte(MQe.Msg_Priority, (byte) msg.priority);

            // add the expiry time if it's been set explicitly
            if (msg.expiry != MQeMQMsgObject.DEF_EXPIRY) {
                // MQ expiry times are in 1/10ths of a second, and
                // MQe delta expiry times are in millisecs.
                result.putInt(MQe.Msg_ExpireTime, msg.expiry * 100);
            }
        } else {
            // This message is not MQe encoded.
            result = mqSeriesFormatToMQe(msg, remoteQMgrName, remoteQName);
        }

        return result;
    }

    /***************************************************************************
     * Convert a MQe format message into a MQ format message. This method is
     * called by the MQ/MQe bridge.
     *
     * @param msg     The MQe message object we are to transform.
     * @param queue   A reference to the MQe queue the put the message.
     *                (e.g. For determining default expiry time etc...)
     * @param options A reference to a MQPutMessageOptions object in which any
     *                required options for the put should be placed.
     * @return        The equivalent MQ format message.
     * @exception java.lang.Exception If the parameters are not valid, or if
     * the message is in a format that this transformer does not understand.
     ***************************************************************************/
    public MQMessage transform(MQeMsgObject msg, MQeMQBridgeQueue queue, MQPutMessageOptions options)
        throws Exception {
        MQMessage result;

        // is it a MQ Style message ? If so, set all context.
        if (msg instanceof MQeMQMsgObject) {
            options.options = MQC.MQPMO_SET_ALL_CONTEXT;
            result = mqSeriesFormatFromMQe((MQeMQMsgObject) msg,
                        queue.getMQQMgr(), queue.getRemoteQName());
        } else {

            // This is most likely a funnelled message.
            // send it on as a MQe Encoded message
            result = new MQMessage();
            result.messageType = MQMT_MQE_FIELDS_FROM_MQE;
            result.putApplicationName = "MQe Bridge";

            // dump the MQe message to the MQ Series data
            result.write(msg.dump());

            // add the priority if it's been set explicitly
            if (msg.contains(MQe.Msg_Priority))
                result.priority = (int) msg.getByte(MQe.Msg_Priority);

            // add the expiry time if it's been set explicitly
            // otherwise allow the default (no expiry time) to be set.
            if (msg.contains(MQe.Msg_ExpireTime)) {

                long    time = System.currentTimeMillis();       /* get current time     */
                char expiryFieldType = msg.dataType(MQe.Msg_ExpireTime);

                switch (expiryFieldType) {

                case MQeFields.TypeLong:
                    // The expiry time is a long, so it contains an absolute expiry
                    // time... in milliseconds.
                    result.expiry = MQeMQMsgObject.mqeAbsolutetExpiryToMQ(msg.getLong(MQe.Msg_ExpireTime));
                    break;

                case MQeFields.TypeInt:
                    // The expiry time is an int, so it is a delta expiry time
                    // on top of the message creation time.
                    result.expiry = MQeMQMsgObject.mqeRelativetExpiryToMQ(msg.getInt(MQe.Msg_ExpireTime));
                    break;

                default:
                    throw new Exception("ExpireTime neither int nor long!");
                } // End of the switch...
            }
        }

        return result;
    }

    /***************************************************************************
     * Convert a MQ format message which is not MQe encoded into an MQe format
     * message.
     *
     * @param msg            The MQSeries message object we are to transform.
     * @param remoteQMgrName The Destination Remote Queue Manager Name
     *                       (from the MQ Remote Queue Definition)
     * @param remoteQName    The Destination Remote Queue Name (from the
     *                       MQ Remote Queue Definition).
     * @return               The equivalent MQe format message.
     * @exception java.lang.Exception  propagated from the called methods.
     ***************************************************************************/
    public MQeMsgObject mqSeriesFormatToMQe(MQMessage msg,
        String remoteQMgrName,
        String remoteQName) throws Exception {
        // The default action is to transform all messages of this type to a
        // MQSeries style message within MQe, ie to a MQeMQMsgObject.
        return mqSeriesFormatToMQe(msg);
    }

    /***************************************************************************
     * Convert an MQ message to an MQeMQMsgObject, ie maintain the MQ Series
     * style of the message. This is the default when a non MQe-encoded message
     * arrives at the bridge from MQSeries. It can be overridden in a
     * sub-class.
     *
     * @param MQMsg  The MQSeries message object we are to transform.
     * @return       The equivalent MQeMQMsgObject message.
     * @exception java.lang.Exception   Propagated from called methods.
     ***************************************************************************/
    public MQeMQMsgObject mqSeriesFormatToMQe(MQMessage MQMsg) throws Exception {
        MQeMQMsgObject MQeMsg = new MQeMQMsgObject();

        // set all the Version 1 header field values
        MQeMsg.setPriority(MQMsg.priority);
        MQeMsg.setMessageId(MQMsg.messageId);
        MQeMsg.setCorrelationId(MQMsg.correlationId);

        // When we get the replyToQueueName from the MQ message, we need to
        // trim-off any trailing spaces. These are often added when the message
        // is created, and they do not form part of the valid Q or QMgr name.
        MQeMsg.setReplyToQueueName(MQMsg.replyToQueueName.trim());
        MQeMsg.setReplyToQueueManagerName(MQMsg.replyToQueueManagerName.trim());

        MQeMsg.setReport(MQMsg.report);
        MQeMsg.setMessageType(MQMsg.messageType);
        MQeMsg.setExpiry(MQMsg.expiry);
        MQeMsg.setFeedback(MQMsg.feedback);
        MQeMsg.setEncoding(MQMsg.encoding);
        MQeMsg.setCharacterSet(MQMsg.characterSet);

        // The format field is space-padded at the end, but
        // so are the MQC.FMT_* fields in the MQ headers are also
        // space padded... so we can't trim things, or comparisons to
        // the MQC.FMT_* constants will fail.
        MQeMsg.setFormat(MQMsg.format);

        MQeMsg.setPersistence(MQMsg.persistence);
        MQeMsg.setBackoutCount(MQMsg.backoutCount);

        // Although the user-id is space-padded we can't trim it, as the
        // whole thing is user-settable for user purposes and the
        // trailing space may be significant.
        MQeMsg.setUserId(MQMsg.userId);

        MQeMsg.setAccountingToken(MQMsg.accountingToken);

        // Although the applicationIdData is space-padded we can't trim it, as the
        // whole thing is user-settable for user purposes and the
        // trailing space may be significant.
        MQeMsg.setApplicationIdData(MQMsg.applicationIdData);

        MQeMsg.setPutApplicationType(MQMsg.putApplicationType);

        // Although the putApplicationName is space-padded we can't trim it, as the
        // whole thing is user-settable for user purposes and the
        // trailing space may be significant.
        MQeMsg.setPutApplicationName(MQMsg.putApplicationName);

        // Although the applicationOriginData is space-padded, we can't trim it, as the
        // whole thing is user-settable for user purposes, and the
        // trailing space may be significant.
        MQeMsg.setApplicationOriginData(MQMsg.applicationOriginData);
        MQeMsg.setPutDateTime(MQMsg.putDateTime);

        // set the Version 2 fields
        MQeMsg.setGroupId(MQMsg.groupId);
        MQeMsg.setMessageSequenceNumber(MQMsg.messageSequenceNumber);
        MQeMsg.setOffset(MQMsg.offset);
        MQeMsg.setMessageFlags(MQMsg.messageFlags);
        MQeMsg.setOriginalLength(MQMsg.originalLength);

        // set the data value
        byte[] userData = new byte[MQMsg.getDataLength()];

        MQMsg.readFully(userData);
        MQeMsg.setData(userData);

        // return the MQe message
        return MQeMsg;
    }

    /***************************************************************************
     * Convert a MQ-Style MQe message (MQeMQMsgObject) into a MQ message
     *
     * @param msg            The MQe message object we are to transform.
     * @param remoteQMgrName The Destination Remote Queue Manager Name
     * @param remoteQName    The Destination Remote Queue Name
     * @return               The equivalent MQ format message.
     * @exception java.lang.Exception If something goes wrong in the transform.
     ***************************************************************************/
    public MQMessage mqSeriesFormatFromMQe(MQeMQMsgObject msg,
        String remoteQMgrName,
        String remoteQName) throws Exception {
        // The default action is to transform all messages of this type directly
        // into a MQSeries message
        return mqSeriesFormatFromMQe(msg);
    }

    /***************************************************************************
     * Convert an MQ-Style MQe message (MQeMQMsgObject) into an MQ message.
     * This is the default when a MQ Style message arrives at the bridge
     * from MQe. It can be overridden in a sub-class.
     *
     * @param msg     The MQe message object we are to transform.
     * @return        The equivalent MQ format message.
     * @exception java.lang.Exception   Propagated from called methods.
     ***************************************************************************/
    public MQMessage mqSeriesFormatFromMQe(MQeMQMsgObject MQeMsg) throws Exception {
        MQMessage MQMsg = new MQMessage();

        // set all the Version 1 header field values
        MQMsg.priority = MQeMsg.getPriority();
        MQMsg.messageId = MQeMsg.getMessageId();
        MQMsg.correlationId = MQeMsg.getCorrelationId();
        MQMsg.replyToQueueName = MQeMsg.getReplyToQueueName();
        MQMsg.replyToQueueManagerName = MQeMsg.getReplyToQueueManagerName();
        MQMsg.report = MQeMsg.getReport();
        MQMsg.messageType = MQeMsg.getMessageType();
        MQMsg.expiry = MQeMsg.getExpiry();
        MQMsg.feedback = MQeMsg.getFeedback();
        MQMsg.encoding = MQeMsg.getEncoding();
        MQMsg.characterSet = MQeMsg.getCharacterSet();
        MQMsg.format = MQeMsg.getFormat();
        MQMsg.persistence = MQeMsg.getPersistence();
        MQMsg.backoutCount = MQeMsg.getBackoutCount();
        MQMsg.userId = MQeMsg.getUserId();
        MQMsg.accountingToken = MQeMsg.getAccountingToken();
        MQMsg.applicationIdData = MQeMsg.getApplicationIdData();
        MQMsg.putApplicationType = MQeMsg.getPutApplicationType();
        MQMsg.putApplicationName = MQeMsg.getPutApplicationName();
        MQMsg.applicationOriginData = MQeMsg.getApplicationOriginData();

        // MQ uses GregorianCalendar which doesn't exist in J2ME so a conversion
        // has to take place
        //*OLD* MQMsg.putDateTime             = MQeMsg.getPutDateTime();
        GregorianCalendar tempCal = new GregorianCalendar();

        tempCal.setTime(MQeMsg.getPutDateTime().getTime());
        MQMsg.putDateTime = tempCal;

        // set the Version 2 fields
        MQMsg.groupId = MQeMsg.getGroupId();
        MQMsg.messageSequenceNumber = MQeMsg.getMessageSequenceNumber();
        MQMsg.offset = MQeMsg.getOffset();
        MQMsg.messageFlags = MQeMsg.getMessageFlags();
        MQMsg.originalLength = MQeMsg.getOriginalLength();

        // set the data value
        byte[] data = MQeMsg.getData();

        if (data == null)
            data = new byte[0];
        MQMsg.write(data);

        // Return the MQ message.
        return MQMsg;
    }

}

