// Version 1.2 mqe_java/source/examples/os4690/Configure.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
package examples.os4690;


import examples.mqbridge.administration.programming.*;
import com.ibm.mqe.*;


/**
 * Configures the 4690 queue manager on your disk.
 * 
 * <p>The resultant queue manager can then be used by the 
 * {@link examples.os4690.Run} program, and destroyed using the 
 * {@link examples.os4690.UnConfigure} program.
 * 
 * <p>Most solutions separate the configuration of their disk-resident queue managers
 * from the use of those queue managers. This example does this also.
 */
public class Configure {
    public static short[] version = { 2, 0, 0, 5 }; 
    
    /**
     * Indicates whether this program puts MQe trace to stdout or not.
     */
    private static final boolean MQE_TRACE = false;

    /**
     * Entry point invoked from the command line.
     */
    public static void main(String[] args) {
        try {
            TraceController traceHelper = new TraceController();

            if (MQE_TRACE) {
                traceHelper.traceOn();
            }
        	
            Configure me = new Configure();

            me.define();
            me.defineTestQueue();
            
            if (MQE_TRACE) {
                traceHelper.traceOff();
            }
        } catch (Exception e) {
            System.out.println("Failure: " + e.toString());
            e.printStackTrace();
        }	
    }
    
    /**
     * Defines the queue manager. And creates a test queue on the queue manager for use later.
     * 
     * <p>All queues defined have names which are independent of the underlying 8.3 file system
     * on which they depend. Each queue is set up with an explicit directory in which messages are stored. 
     */
    public void define() throws Exception {
    	
        OS4690ConfigInfo configInfo = new OS4690ConfigInfo();

        configInfo.activate();
		
        MQeFields configFields = configInfo.getConfigFields();
        String storageLocation = configInfo.getStorageLocation();
        String queueManagerName = configInfo.getQueueManagerName();
        String fileSeparator = MQe.fileSeparator();
		
        System.out.println("Defining the queue manager.");
				
        // Construct a queue manager configuration utility object.
        MQeQueueManagerConfigure configurator = new MQeQueueManagerConfigure(configFields, storageLocation);

        // Define a queue manager.				
        configurator.defineQueueManager();

        // Define some queues on the queue manager.  
        // Explicitly stating where their storage locations are based.
        String adminQLocation = storageLocation + fileSeparator + "AQ";

        configurator.defineDefaultAdminQueue("My Admin Queue", adminQLocation);
        System.out.println("Defined the admin queue to store messages into location " + adminQLocation);
		
        String adminReplyQLocation = storageLocation + fileSeparator + "ARQ";   									

        configurator.defineDefaultAdminReplyQueue("My Admin Reply Queue", adminReplyQLocation);        									
        System.out.println("Defined the admin reply queue to store messages into location " + adminReplyQLocation);
						  
        String deadLetterQLocation = storageLocation + fileSeparator + "DLQ";						  

        configurator.defineDefaultDeadLetterQueue("My Dead Letter Queue", deadLetterQLocation);
        System.out.println("Defined the dead letter queue to store messages into location " + deadLetterQLocation);
		
        String defaultSystemQLocation = storageLocation + fileSeparator + "DSQ";

        configurator.defineDefaultSystemQueue("My Default System Queue", defaultSystemQLocation);
        System.out.println("Defined the default system queue to store messages into location " + defaultSystemQLocation);	
						
        // Close the queue manager configuration utility class.
        configurator.close();
        System.out.println("Defined queue manager " + queueManagerName);
    }
    
    /**
     * Define a test queue using administration messages, so we can put a test message to it.
     * 
     * <p>We could have used the system default local queue, but here we are showing an example
     * of how one might create a queue which has a long filename itself, but stores messages
     * on an 8.3 file sysyem. 
     * 
     * @throws Exception if there is a failure.
     */
    public void defineTestQueue() throws Exception {
        System.out.println("Creating a test queue");
    	
        OS4690ConfigInfo configInfo = new OS4690ConfigInfo();

        configInfo.activate();
		
        MQeFields configFields = configInfo.getConfigFields();
        String storageLocation = configInfo.getStorageLocation();
        String queueManagerName = configInfo.getQueueManagerName();
        String fileSeparator = MQe.fileSeparator();
        String testQueueLongName = configInfo.getTestQueueLongName();
        String testQueueShortName = configInfo.getTestQueueShortName();
    	
        // Now fire-up the queue manager so we can adminster it using adminstration messages...
        MQeQueueManager myQueueManager = new MQeQueueManager();

        myQueueManager.activate(configFields);
        
        // Construct a test queue explicitly which stores 8.3 message fies in a short filename directory.
        Reporter reporter = (Reporter) new AdminHelperReporter();
        AdminMessageDispatcher dispatcher = (AdminMessageDispatcher) new AdminHelperDispatcher(myQueueManager);
        AdminHelperLocalQueue helper = new AdminHelperLocalQueue(reporter, dispatcher);
		  
        try {
            System.out.println("Test queue name is " + testQueueLongName);
            System.out.println("Test queue will store messages to " + storageLocation + fileSeparator + testQueueShortName);
            helper.create(queueManagerName, testQueueLongName, storageLocation + fileSeparator + testQueueShortName);
        } catch (Exception e) {
            reporter.error("Failed to create the test queue.");        	
            reporter.error(e);
        } finally {
            // Even if we failed, make sure the queue manager is closed down within 5 seconds.
            myQueueManager.closeQuiesce(5000L);
        }		
    }	    	   
}
