package examples.osgi.server;


// Version 1.3 mqe_java/source/examples/osgi/server/MQeServerApp.java, MQeOSGi, la200, la200-L030324.1
/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/
 
// Used for Trace
import com.ibm.mqe.trace.*;
import examples.osgi.admin.*;

import com.ibm.mqe.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.communications.*;
import com.ibm.mqe.adapters.MQeDiskFieldsAdapter;
import com.ibm.mqe.adapters.MQeMemoryFieldsAdapter;
import com.ibm.mqe.administration.*;
import java.io.File;
import java.util.*;


/***
 * Example MQe Application.
 * An MQeQueueManager is defined and started with a listener and default
 * queues in memory. MQe code is then run in a new thread which sits 
 * and via a message listener waits for incoming messages, any received
 * messages are displayed in the console.  
 * This continues to  listen until shutdownMQeServer() method is called
 * at which time it stops and deletes the MQeQueueManager.
 * 
 * This Example uses the helper class:
 *   
 * <pre>
 *  . MQeAdmin - used to configure MQe Components. 
 *   	
 * </pre>
 *  This class contains the MQe code to create a message listener and wait for messages.
 *  It is written this way to separate the MQe configuration code from the 
 *  Application code.
 * 
 *  This class is expected to be used in a particular way with the 5 main methods being
 *  called in a defined order as detailed below:
 * <pre>
 *  . MQeServerApp(String qmName, String regLocation)
 * 		This constructor must be used, supplying the QM name and registry location
 * 		to be used.
 *  . setServerAddressAndPort() 
 * 		Sets the servers address and port, the address is ignored here.
 *  . setupMQeServer()
 * 		This defines and starts a QM in memory and then adds and starts a listener.
 *  . runTest()
 * 		This creates a new MQeServer and starts it in a new thread.
 *  . shutdownMQeServer()
 * 		This stops the MQeServer thread and shutsdown and deletes the QM.
 * </pre>
 *  A main method is also supplied to enable testing of this class outside of the 
 *  OSGI framework.
 *  By default the MQeTcpipHttpAdapter is used for comms and the MQeMemoryFieldsAdapter
 *  is used for registry storage. 
 *  The Port to be used by the MQeListener can be configured by using the 
 *  setReceiversAddressAndPort method.
 *
 **/

public class MQeServerApp  implements Runnable, MQeMessageListenerInterface {
	
    public static short[] version = { 2, 0, 0, 5 }; 
	
    private String myQmName = "myQM";
    private String myReg = "c:\\myQM\\";
    private MQeAdmin mqeAdmin;
    private Thread testThread;
    private MQeQueueManager qm;

    /**
     * Constructor to create a new MQeServerApp.
     * This sets the QueueManger name and Registry location
     * to be used by the MQeQueuemanager. It also 
     * creates a new MQeAdmin object to be used to setup MQe.
     */
    public MQeServerApp(String qmName, String regLocation) {
		
        myQmName = qmName;
        myReg = regLocation;
        mqeAdmin = new MQeAdmin(myQmName, myReg);	
	
    }
	
    /**
     * Run Method
     * Adds the message listener to the default queue.
     */
    public void run() {
        try {
			
            qm = MQeQueueManager.getDefaultQueueManager();
            System.out.println("..MQeServerBundle - registering message listener");
            qm.addMessageListener(this, MQe.System_Default_Queue_Name, (MQeFields) null);
        
        } catch (Exception e) {
            System.out.println("ERROR - " + e);
        }
		
    }

    /**
     * Method used to set the servers address and port. The
     * port number is what will be used by the MQeListener. If this method
     * is not used then then default is 8085. Address parameter 
     * is not currently used.
     */
    public void setServerAddressAndPort(String targetQMIPAddress, String targetQMPort) {
        mqeAdmin.setServerAddressAndPort(targetQMIPAddress, targetQMPort);
	
    }
	
    /**
     * Simple method that just checks if there is already a QM in this JVM.
     * It is used by the app to check a QM is not running before starting a 
     * new one. Returns true if a QM already exists.
     */
    private boolean qmExists() {
        MQeQueueManager qm = MQeQueueManager.getDefaultQueueManager();
	
        if (!(qm == null)) {
            //a QM exists 
            return true;
			
        }
        return false;
    }
	
    /**
     * Checks to see if a QM is already running. If not it uses the MQeAdmin
     * to define a new QM, start it then create a listener and start it.
     * If a QM is already running then an exception is thrown.
     */
    public void setupMQeServer() throws Exception {
        if (!qmExists()) {
		
            mqeAdmin.defineQM();
            mqeAdmin.startQM();
            mqeAdmin.createListener();
            mqeAdmin.startListener();
        } else {

            throw new Exception("ERROR - QM Already Running");			
			
        }

    }

    /**
     * This creates and starts a new MQeServerApp object in its own thread.
     * This sets a message listener on a Queue and prints out received
     * messages. The MQeServer must be run in it's own thread or the 
     * BundleActivator.start methods will not return and the bundle will not start
     * correctly.
     */
    public void runTest() {
	
        testThread = new Thread(this);
        testThread.start();
    }
	
    /**
     * This stops the MQeServer thread and shutsdown and deletes the QM. 
     */
    public void shutdownMQeServer() {
		
        //check the JMS test thread has stopped and if not stop it.
        //this.stopListening();
        if (testThread != null) {
            Thread temp = testThread;

            testThread = null;
            try {
                temp.join();
            } catch (InterruptedException e) {}
        }	
		
        mqeAdmin.stopListener();
        mqeAdmin.deleteListener();
        mqeAdmin.stopQM();
        mqeAdmin.deleteQM();
		
    }
	
    /**
     * Prints the contents of a JMS text message.
     */
    private void printJMSMessage(MQeMsgObject msg) throws MQeException {
	
        //check it is not null
        if (msg == null)
            throw new MQeException("Error: The jms msg is null");
	
        //check it is a text msg
        try {
            String msgtype = msg.getAscii("MQeJMSMessageClass");

            if (msgtype.equals("jms_text")) {
                byte[] msgBody = msg.getArrayOfByte("MQeJMSBody");
                String msgText = new String(msgBody, "UTF8");
				
                System.out.println("*****************************************************************");
                System.out.println("JMS Message Received = " + msgText);        			
                System.out.println("*****************************************************************");  
        							
                //String msgText =  msgBody.toString();
					
            }
		
        } catch (Exception e) {
            throw new MQeException("Error: The jms message class field is not present");
        }
		
    }

    /**
     * This method is called when a message arrives on a queue.
     * 
     * This checks the message arrived event is from the default queue and 
     * then checks if the message received is a JMS Message if so it calls 
     * printJMSMessage(msg) , to print it, otherwise it just prints out 
     * the contents of the messages "MsgData" field.
     * 
     * @param e com.ibm.mqe.MQeMessageEvent
     * @return void
     * @exception java.lang.Exception 
     */
    public void messageArrived(MQeMessageEvent e) throws Exception {
        String qName = e.getQueueName();
        String queueName = MQe.System_Default_Queue_Name;
        MQeMsgObject msg = null;

        /*
         * is this the queue we are interested in?
         */
        try {
            if (!qName.equals(queueName)) {
                System.out.println("..Expecting event from queue    " + queueName);
                System.out.println("..but received event from queue " + qName);
            } else {
                // get msg info
                MQeFields filter = e.getMsgFields();
 
                msg = qm.getMessage(null, "SYSTEM.DEFAULT.LOCAL.QUEUE", filter, null, 0);
            }
        } catch (Exception e1) {
            System.err.println("Failure ! " + e1.toString());
        }	
        
        try {
			
            short jms = msg.getShort(MQe.MQe_JMS_VERSION);

            //no exception thrown so this must be a jms message
            printJMSMessage(msg);
		    
        } catch (MQeException mqee) {
            //exception thrown so this is not a jms message as there is no version field
	        
            MQeFields attribs = e.getMsgFields();
            String message = msg.getAscii("MsgData");

            byte[] msgID = attribs.getArrayOfByte(MQe.Msg_MsgID);
        
            System.out.println("*****************************************************************");
            System.out.println("A new message arrived on Queue " + queueName +
                " with ID = " + new String(msgID));
            System.out.println("Message = " + message);           	
            System.out.println("*****************************************************************");   
			
        }

    }
	
    /**
     * This main method can be used to test the example outside of the OSGI
     * framework.  In the framework the same job is done by the Activators
     * start and stop methods.
     */
    public static void main(String[] args) {
		
        MQeServerApp dem = new MQeServerApp("MQeServer", "c:\\MQeServer");

        dem.setServerAddressAndPort("127.0.0.1", "8085");
        try {
            dem.setupMQeServer();
        } catch (Exception e) {
            System.out.println(e.toString());
        }
        dem.runTest();
        dem.shutdownMQeServer();
		
    }
}
