
// Version 1.3 mqe_java/source/examples/queuemanager/MQeMemoryQM.java, MQeExamples, la200, la200-L030324.1
/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 1999                                                *
 *-----------------------------------------------------------------------------*/
package examples.queuemanager;


import com.ibm.mqe.*;
import com.ibm.mqe.adapters.*;
import com.ibm.mqe.registry.*;
import com.ibm.mqe.trace.MQeTraceToReadable;


/**
 * To create a queue manager in-memory, with no disk footprint at all, use it and destroy it.
 *
 * <p>The steps performed by this example application are:
 * <ul>
 * <li>Instantiate an MQeMemoryFieldsAdapter to put hold the queue manager and queues.
 * A reference to this is kept to make sure the memory in which the queue manager exists is not
 * garbage-collected.
 * <li>Define a queue manager using the MQeMemoryFieldsAdapter
 * <li>Put a message to a local queue
 * <li>Get the message from a local queue
 * <li>Un-define the queue manager
 * </ul>
 */
public class MQeMemoryQM {

    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    /**
     * A definition of my queue manager.
     */
    private MQeQueueManager myQueueManager = null;

    /**
     * Private reference to a memory adapter, to make sure
     * that configuration information set up in one method is not
     * garbage collected before it is used in other methods.
     */
    private MQeMemoryFieldsAdapter myMemory = null;

    /**
     * Constant indicating whether we should collect MQe trace or not.
     */
    private static final boolean MQE_TRACE_ON = false;

    /**
     * Private reference to an MQeTrace object to make sure it doesn't get
     * garbage collected in-between methods.
     */
    private MQeTrace myMQeTrace = null;

    /**
     * Private reference to configuration information.
     * Created in the define method. Used there, and in the start method.
     */
    private MQeFields config = null;

    /**
     * The class name of the memory adapter.
     */
    private static final String MEMORY_ADAPTER_CLASS_NAME = "com.ibm.mqe.adapters.MQeMemoryFieldsAdapter";

    /**
     * How long to wait for the queue manager to close cleanly, before it is closed by force ?
     * <p>Using 1 second for this example.
     */
    private static final long QUIESCE_TIME = (long) 1000;

    /**
     * The name of the queue manager.
     */
    private String queueManagerName = null;

    /**
     * The logical location of the registry and queue stores in a memory address space.
     */
    private String baseDirectoryName = null;

    /**
     * The name of the queue to which this example puts a test message, then gets a test
     * message from.
     */
    private static final String TARGET_QUEUE_NAME = "SYSTEM.DEFAULT.LOCAL.QUEUE";

    /**
     * Constructor for MQeMemoryQM.
     */
    public MQeMemoryQM() {
        super();
    }

    /**
     * Entry point from the command line.
     *
     * <p>This method will perform the following operations:
     * <ul>
     * <li>Create an instance of the MQeMemoryQM class.
     * <li>Define the queue manager in memory.
     * <li>Start the queue manager.
     * <li>Put a simple message to a queue on the queue manager.
     * <li>Get the message off the queue.
     * <li>Stop the queue manager
     * <li>Remove the queue manager definition from memory.
     * <li>Optionally trace using a MQe trace handler to stdout, depending on whether a constant is set or not.
     * </ul>
     */
    public static void main(String[] args) {
        try {
            MQeMemoryQM me = new MQeMemoryQM();

            if (MQE_TRACE_ON) {
                me.traceOn();
            }
            me.define(me.inventQueueManagerName());
            me.start();
            me.put();
            me.get();
            me.stop();
            me.undefine();
            if (MQE_TRACE_ON) {
                me.traceOff();
            }
        } catch (Exception error) {
            System.err.println("Error: " + error.toString());
            error.printStackTrace();
        }
    }

    /**
     * Turns MQe trace on.
     */
    public void traceOn() throws Exception {

        // Allocate a trace instance, so our handler isn't garbage collected when its' on.
        myMQeTrace = new MQeTrace();

        // Allocate a trace handler
        // This one puts trace output to stdout by default.
        MQeTraceHandler handler = (MQeTraceHandler) new com.ibm.mqe.trace.MQeTraceToReadable();

        // Set this handler as the one MQe uses.
        MQeTrace.setHandler(handler);

        // Set the filter so we collect those pieces of trace we are interrested in.
        // In this case, it't the MQe API calls, errors, warnings, and exceptions.
        MQeTrace.setFilter(MQeTrace.GROUP_API | MQeTrace.GROUP_ERROR
            | MQeTrace.GROUP_WARNING | MQeTrace.GROUP_EXCEPTION
        );
    }
        
    /**
     * Invents a queue manager name for use in this example
     */
    public String inventQueueManagerName() {
        return "TempQM" + Long.toString(System.currentTimeMillis());
    }

    /**
     * Defines the queue manager using a memory adapter.
     *
     * <p>All configuration and messaging data is stored in memory.
     */
    public void define(String queueManagerName) throws Exception {

        System.out.println("Defining the queue manager.");

        // Generate a timestamp-related queue manager name.
        // That way we stand some chance that it will be unique.
        this.queueManagerName = queueManagerName;
        baseDirectoryName = "C:/MyQueueManagers/" + queueManagerName;

        myMemory = new com.ibm.mqe.adapters.MQeMemoryFieldsAdapter();

        // Create all the configuration information needed to construct the
        // queue manager in memory.
        config = new MQeFields();

        // Construct the queue manager section parameters.
        MQeFields queueManagerSection = new MQeFields();

        queueManagerSection.putAscii(MQeQueueManager.Name, queueManagerName);
        config.putFields(MQeQueueManager.QueueManager, queueManagerSection);

        // Construct the registry section parameters.
        // In this examples, we use a public registry.
        MQeFields registrySection = new MQeFields();

        registrySection.putAscii(MQeRegistry.Adapter, MEMORY_ADAPTER_CLASS_NAME);
        registrySection.putAscii(MQeRegistry.DirName, baseDirectoryName + "/Registry");
        config.putFields("Registry", registrySection);

        MQeQueueManagerConfigure configurator = new MQeQueueManagerConfigure(config, MEMORY_ADAPTER_CLASS_NAME + ":" + baseDirectoryName + "/Queues");

        configurator.setChnlAttributeRuleName("ChannelAttrRules");
        configurator.defineQueueManager();

        configurator.defineDefaultAdminQueue();
        configurator.defineDefaultAdminReplyQueue();
        configurator.defineDefaultDeadLetterQueue();
        configurator.defineDefaultSystemQueue();

        configurator.close();

        System.out.println("Defined queue manager " + queueManagerName + " in memory.");
    }

    /**
     * Starts the in-memory queue manager.
     */
    public void start() throws Exception {
        System.out.println("Starting the queue manager");
        myQueueManager = new MQeQueueManager();
        myQueueManager.activate(config);
        System.out.println("Queue manager started.");
    }

    /**
     * The name of the field in which this example puts a test message.
     */
    private static final String FIELD_NAME_TEXT_STRING = "myFieldName";

    /**
     * Puts a simple message to the system default local queue.
     */
    public void put() throws Exception {
        System.out.println("Putting the test message");
        MQeMsgObject msg = new MQeMsgObject();

        msg.putUnicode(FIELD_NAME_TEXT_STRING, "Hello");
        myQueueManager.putMessage(queueManagerName, TARGET_QUEUE_NAME, msg, null, 0L);
        System.out.println("Put the test message");
    }

    /**
     * Gets a simple message from the system default local queue.
     */
    public void get() throws Exception {
        System.out.println("Geting the test message.");
        MQeMsgObject msg = myQueueManager.getMessage(queueManagerName, TARGET_QUEUE_NAME, null, null, 0L);

        if (msg != null) {
            System.out.println("Got the test message.");
        }
    }

    /**
     * Stops the in-memory queue manager.
     */
    public void stop() throws Exception {
        System.out.println("Stopping the queue manager.");
        myQueueManager.closeQuiesce(QUIESCE_TIME);
        myQueueManager = null;
        System.out.println("Queue manager stopped.");
    }

    /**
     * Removes any definitions on the queue manager which were previously defined.
     */
    public void undefine() throws Exception {

        System.out.println("Queue manager " + queueManagerName + " will now be deleted.");

        MQeQueueManagerConfigure configurator = new MQeQueueManagerConfigure();

        configurator.activate(config, MEMORY_ADAPTER_CLASS_NAME + ":" + baseDirectoryName + "/Queues");

        // Remove the configured entries for the queues we defined.
        configurator.deleteSystemQueueDefinition();
        configurator.deleteDeadLetterQueueDefinition();
        configurator.deleteAdminReplyQueueDefinition();
        configurator.deleteAdminQueueDefinition();

        configurator.close();

        // Free any configuration information for garbage collection...
        myMemory = null;

        System.out.println("Queue manager is now deleted.");
    }

    /**
     * Turns MQe trace off.
     */
    public void traceOff() throws Exception {
        // Stop collecting trace information.
        MQeTrace.setFilter(MQeTrace.GROUP_MASK_NONE);

        // Remove the trace handler.
        MQeTrace.setHandler(null);
    }
}
