package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQePrivateClient.java, MQeExamples, la200, la200-L030324.1 1.9
/* -----------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.registry.MQeRegistry;


/**
 * <p>To provide a simple example MQe client capable of accepting private
 * startup parameters.
 *
 * <p>Components configured on a client at startup are :
 * <pre>
 *      ... Aliases
 *      ... Registry
 *      ... QueueManager
 * </pre>
 * <p><b>Note:</b>Only one client per JVM can be started at a time.
 * <p> This extends the {@link com.ibm.mqe.MQe} class.
 */
public class MQePrivateClient extends MQe {

    public static short version[] = { 2, 0, 0, 5 }; 
    public static MQeQueueManager queueManager;
    public static MQeFields sections;

    private String thisPIN;
    private String thisKeyRingPwd;
    private String thisCertReqPIN;

    /**
     * If you want to start MQe tracing, then set this flag to true.
     */
    private static final boolean START_MQE_TRACE = false;

    /**
     * Constructor with configuration parameters from an ini file.
     * 
     * @param iniFileName A String containing the ini file name containing startup parms.
     * @param pIN A String containing Number, password or passphrase to be used to open the private registry.
     * @param keyRingPwd A String password or passphrase used to protect the entitys private key.
     * @param certReqPIN The certificate request PIN supplied by the entity.
     * @exception java.lang.Exception
     */
    public MQePrivateClient(String iniFileName, String pIN, String keyRingPwd, String certReqPIN)
        throws Exception {
        super();
        thisPIN = null;
        thisKeyRingPwd = null;

        if (pIN != null && !(pIN.equals(""))) {
            thisPIN = pIN;
        }

        if (keyRingPwd != null && !(keyRingPwd.equals(""))) {
            thisKeyRingPwd = keyRingPwd;
        }

        if (certReqPIN != null && !(certReqPIN.equals(""))) {
            thisCertReqPIN = certReqPIN;
        }

        // Load the ini file
        init(MQeQueueManagerUtils.loadConfigFile(iniFileName));
    }

    /**
     * Stops an MQeClient.
     * 
     * @return void
     * @exception java.lang.Exception
     */
    public void close() throws Exception {
        // Stop the queue manager if its' available.
        if (queueManager != null) {
            queueManager.close();
        }

        // Free up the queue manager object for garbage collection.
        queueManager = null;

        // If this server turned trace on, then turn it off again.
        if (START_MQE_TRACE) {
            MQeQueueManagerUtils.traceOff();
        }

    }

    /**
     * First stage setup.
     * 
     * @param parms MQeFields object.
     * @return void
     * @exception java.lang.Exception
     */
    public void init(MQeFields parms) throws Exception {
        if (queueManager != null) {
            throw new Exception("Client already running");
        }
        // Remember startup parms
        sections = parms;

        // set any alias names
        MQeQueueManagerUtils.processAlias(sections);

        if (START_MQE_TRACE) {
            MQeQueueManagerUtils.traceOn();
        }

        // Display the startup parameters
        System.out.println(sections.dumpToString("#1\t=\t#2\r\n"));

        // use any given PIN, KeyRingPassword or CertReqPIN
        MQeFields regFields = sections.getFields(MQeQueueManager.Registry);
        
        if (thisPIN != null) {
            regFields.putAscii(MQeRegistry.PIN, thisPIN);
            System.out.println("Using given PIN");
        }
        
        if (thisKeyRingPwd != null) {
            regFields.putAscii(MQeRegistry.KeyRingPassword, thisKeyRingPwd);
            System.out.println("Using given KeyRingPwd");
        }
        
        if (thisCertReqPIN != null) {
            regFields.putAscii(MQeRegistry.CertReqPIN, thisCertReqPIN);
            System.out.println("Using given CertReqPIN");
        }
        
        sections.putFields(MQeQueueManager.Registry, regFields);
        thisPIN = null;
        thisKeyRingPwd = null;

        // Start the queue manager
        queueManager = MQeQueueManagerUtils.processQueueManager(sections);
        sections = null;
    }
}
