package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQePrivateServer.java, MQeExamples, la200, la200-L030324.1 1.10
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import java.io.File;
import java.util.Hashtable;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.registry.MQeRegistry;


/**
 * <p>To provide a simple example MQe server.
 *
 * <p>Components configured on a server at startup are :
 * <pre>
 *      ... Aliases
 *      ... ChannelManager
 *      ... ChannelListener
 *      ... Registry
 *      ... QueueManager
 *      ... Preload
 * </pre>
 * <p><b>Note:</b>Only one client per JVM can be started at a time.
 * <p> This extends the {@link com.ibm.mqe.MQe} class.
 */

public class MQePrivateServer extends MQe {
    public static short version[] = { 2, 0, 0, 5 }; 

    public static MQeQueueManager queueManager;
    public static Hashtable loadTable;
    public static boolean initialised = false;
    public static MQeFields sections = null;
    private String thisPIN;
    private String thisKeyRingPwd;
    private String thisCertReqPIN;

    /**
     * If you want to start MQe tracing, then set this flag to true.
     */
    private static final boolean START_MQE_TRACE = false;

    /**
     * Constructor with configuration parameters from an ini file

     * @param iniFileName A String containing the ini file name containing startup parms.
     * @param pIN A String containing Number, password or passphrase to be used to open the private registry.
     * @param keyRingPwd A String password or passphrase used to protect the entitys private key.
     * @param certReqPIN The certificate request PIN supplied by the entity.
     * @exception java.lang.Exception
     */
    public MQePrivateServer(String iniFileName, String pIN, String keyRingPwd, String certReqPIN)
        throws Exception {
        super();
        thisPIN = null;
        thisKeyRingPwd = null;

        // If we're given a PIN, save it
        if (pIN != null && !(pIN.equals(""))) {
            thisPIN = pIN;
        }

        // If we're given a keyring password, save it
        if (keyRingPwd != null && !(keyRingPwd.equals(""))) {
            thisKeyRingPwd = keyRingPwd;
        }

        // If we're given a certificate PIN, save it
        if (certReqPIN != null && !(certReqPIN.equals(""))) {
            thisCertReqPIN = certReqPIN;
        }

        // Load the ini file
        init(MQeQueueManagerUtils.loadConfigFile(iniFileName));
    }

    /**
     * Activate and de-activate the server.
     * 
     * @param start A boolean, True to activate and False to deactivate.
     * @return void
     * @exception java.lang.Exception
     */
    public void activate(boolean Start) throws Exception {
        if (Start) {
            if (!initialised) {
                // assign any class aliases
                MQeQueueManagerUtils.processAlias(sections);
                
                // check for any pre-loaded classes
                loadTable = MQeQueueManagerUtils.processPreLoad(sections);
                initialised = true;
            }
            // setup and activate the queue manager
            queueManager = MQeQueueManagerUtils.processQueueManager(sections);
            
            // setup and activate the listener for incomming connections
            MQeQueueManagerUtils.processListenerSections(sections, queueManager);
        }
    }

    /**
     * Stops an MQeServer.
     * 
     * @return void
     * @exception java.lang.Exception
     */
    public void close() throws Exception {
        activate(false);

        // Close the queue manager if there is one.
        if (queueManager != null) {
            queueManager.close();
        }

        // Free the queue manager up for garbage collection,
        queueManager = null;

        // Stop trace if we started it.
        if (START_MQE_TRACE) {
            MQeQueueManagerUtils.traceOff();
        }
    }

    /**
     * First stage setup.
     * 
     * @param parms MQeFields object.
     * @return void
     * @exception java.lang.Exception
     */
    public void init(MQeFields parms) throws Exception {
        if (initialised)
            throw new Exception("Server already running");
        // Remeber startup parms
        sections = parms;
        // set any alias names
        MQeQueueManagerUtils.processAlias(sections);

        if (START_MQE_TRACE) {
            MQeQueueManagerUtils.traceOn();
        }

        // Display the startup parameters
        System.out.println(sections.dumpToString("#1\t=\t#2\r\n"));

        // use any given PIN, KeyRingPassword or CertReqPIN
        MQeFields regFields = sections.getFields(MQeQueueManager.Registry);
        
        if (thisPIN != null) {
            regFields.putAscii(MQeRegistry.PIN, thisPIN);
            System.out.println("Using given PIN");
        }
        
        if (thisKeyRingPwd != null) {
            regFields.putAscii(MQeRegistry.KeyRingPassword, thisKeyRingPwd);
            System.out.println("Using given KeyRingPwd");
        }
        
        if (thisCertReqPIN != null) {
            regFields.putAscii(MQeRegistry.CertReqPIN, thisCertReqPIN);
            System.out.println("Using given CertReqPIN");
        }
        
        sections.putFields(MQeQueueManager.Registry, regFields);
        thisPIN = null;
        thisKeyRingPwd = null;
    }

    public static void main(String[] args) {
        try {
            String fileName = "." + File.separator + "MQePrivateServer.ini";
            String pIN = "12345678";
            String keyRingPwd = "It_is_a_secret";
            String certReqPIN = "12345678";

            if (args.length > 0) {
                fileName = args[0];
            }
            
            if (args.length > 1) {
                pIN = args[1];
            }
            
            if (args.length > 2) {
                keyRingPwd = args[2];
            }
            
            if (args.length > 3) {
                certReqPIN = args[3];
            }

            new MQePrivateServer(fileName, pIN, keyRingPwd, certReqPIN).activate(true);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
