package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQeQueueManagerUtils.java, MQeExamples, la200, la200-L030414.1 1.27
/* -----------------------------------------------------------------------------*/

/*                   WebSphere MQ Everyplace for Multiplatforms                 */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999, 2001                                           */

/* -----------------------------------------------------------------------------*/

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Properties;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeProperties;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.MQeTrace;
import com.ibm.mqe.MQeTraceHandler;
import com.ibm.mqe.adapters.MQeCommunicationsAdapter;
import com.ibm.mqe.administration.MQeCommunicationsListenerAdminMsg;

import examples.administration.commandline.ListenerCreate;
import examples.administration.commandline.ListenerInquire;
import examples.administration.commandline.ListenerStart;


/**
 * <p>To provide a set of helper methods for configuring and starting
 * various MQe components.
 *
 * <p>Each component has a Process* method which accepts configuration
 *      data via an MQeFields object and then initialises the component
 *      To allow the configuration data to be stored persistently on disk
 *      methods are provided to save and restore to/from an ini file.
 * This class extends {@link com.ibm.mqe.MQe} class.
 *
 * <p>------------------------------------------------------------------------------
 * <pre>
 * Example configuration data fields :
 *
 *   [Aliases]
 *      (ascii)EventLog=examples.eventLog.LogToDiskFile
 *      (ascii)MsgLog=com.ibm.mqe.adapters.MQeDiskFieldsAdapter
 *
 *   [ChannelManager]
 *      (int)MaxChannels=1
 *
 *   [Listener]
 *      (ascii)Listen=Network::80
 *      (ascii)Network=Network:
 *      (int)TimeInterval=300
 *
 *   [QueueManager]
 *      (ascii)Name=ServerQMgr
 *
 *   [Registry]
 *      (ascii)RegType=FileRegistry
 *      (ascii)Path=.\ExampleQM\Registry\
 *      (ascii)PrivRegPIN=12345678
 * </pre>
 */
public class MQeQueueManagerUtils extends MQe {

    public static short[] version = { 2, 0, 0, 5 }; 
    // Field item names in the Sections fields object
    public final static String Section_Alias = "Alias";
    public final static String Section_ChannelManager = "ChannelManager";
    public final static String Section_Listener = "Listener";
    public final static String Section_PreLoad = "PreLoad";
    public final static String Section_QueueManager = "QueueManager";
    public final static String Section_MQBridge = "MQBridge";
    public final static String SECTION_NAME_SYSTEM_PROPERTIES = "SystemProperties";

    /**
     * Listeners are named for administrative purposes.  This variable holds the
     * name used for the (single) listener created in this example.
     */
    private final static String LISTENER_NAME = "MyListener";

    // It would be useful to tell the user if there are sections in the .ini
    // file that we haven't processed. For example, if we're starting a queue
    // manager as a client, but the .ini file contains a Listener session,
    // the listener section will be ignored.
    private static Vector sectionsNotProcessed = new Vector();

    /**
     * Reads the .ini file described by the supplied string
     * and converts the data into an MQeFields object.
     * The MQeFields object returned contains a number of embedded MQeFields
     * objects, one for each section of the ini file.
     * Each embedded object contains a list of key/value string pairs
     * which correspond to each item in the section.
     *
     * @param String fileName The name of the file to read.
     * @return MQeFields, the converted MQeFields object.
     * @exception java.lang.Exception
     */
    public static MQeFields loadConfigFile(String fileName) throws Exception {
        // Access the file
        File diskFile = new File(fileName);

        // Create a byte array big enough to hold the file's contents.
        byte data[] = new byte[(int) diskFile.length()];

        // Read the file into the byte array and close the file.
        FileInputStream inputFile = new FileInputStream(diskFile);

        inputFile.read(data);
        inputFile.close();

        // Read the data into a buffer where it can be manipulated.
        StringBuffer configDataBuff = new StringBuffer(byteToAscii(data));

        // We want to look for single \n characters which are not adjacent to \r
        // characters. ie; On Unix systems, if someone uses vi or similar editor
        // to edit the .ini file, then lines will end in \n only.
        // To make MQe more tolerant of this, we convert lone \n characters to \r\n
        // sequences so the .ini files can cope.
        // This has the side-effect of dis-allowing embedded \n characters in an .ini
        // file, but this compromise is seen as being acceptable.

        // Method: Work through all the configuration information.
        // If we find a single \n character, insert a \r character before it.
        char prevChar = '\0';
        char thisChar = '\0';

        for (int along = 0; along < configDataBuff.length(); along += 1) {
            prevChar = thisChar;
            thisChar = configDataBuff.charAt(along);
            if ((thisChar == '\n') && (prevChar != '\r')) {
                // The character under strutiny was a '\n' on it's own.
                // Turn it into a '\r\n'
                configDataBuff.insert(along, '\r');
                // Skip the along counter on one, so we don't look at the same
                // '\n' character again.
                along += 1;
            } else {// The character under scrutiny was not a '\n' on it's own,
                //  so just leave it alone.
            }
        }

        MQeFields retVal =
            MQeFields.restoreFromString("\r\n", "[#0]", "(#0)#1=#2", configDataBuff.toString() + "\r\n");

        // Work out which sections there are in the .ini file that may need processing.
        // We can warn the user if fields have not been processed.
        Enumeration sectionsEnumeration = retVal.fields();

        sectionsNotProcessed = new Vector();
        while (sectionsEnumeration.hasMoreElements()) {
            sectionsNotProcessed.addElement((String) sectionsEnumeration.nextElement());
        }

        return retVal;

    }

    /**
     * Process the class alias names in the configuration data.
     *
     * <p>This maps one class name to another, or a nickname to a class name :
     * <pre>
     *  [Alias]
     *      (ascii)EventLog=.....        class to use to write event logs
     *      (ascii)Network=.....         Adapter class for network support
     *      (ascii)QueueManager=.....    class to use for Queue manager
     *      (ascii)Registry=.....        class to use for Registry functions
     *      (ascii)Trace=.....           class to provide trace functions
     * </pre>
     *
     * @param sections MQeFields object
     * @return void
     * @exception java.lang.Exception
     */
    public static void processAlias(MQeFields sections) throws Exception {
        if (sections.contains(Section_Alias)) {

            // Get all the fields inside the alias section
            MQeFields section = sections.getFields(Section_Alias);
            Enumeration keys = section.fields();

            while (keys.hasMoreElements()) {

                // For each key, get the value and add the mapping to the MQe
                // internal alias table
                String key = (String) keys.nextElement();

                MQe.alias(key, section.getAscii(key).trim());
            }

            sectionProcessed(Section_Alias);
        }
    }

    /**
     * <p>Process the pre-load classes configuration data.
     *
     * <p>Pre-load a class for any programs that run concurrently with the
     * server.
     *
     * <pre>
     *  [PreLoad]      { Optional } pre-load classes
     *      (ascii)nnn=.....             keyword (unique)=class name to load
     *      (ascii)nnn=.....             keyword (unique)=class name to load
     *      .....
     * </pre>
     *
     * @param sections MQeFields object
     * @return void
     * @exception java.lang.Exception
     */
    public static Hashtable processPreLoad(MQeFields sections) throws Exception {
        // Store a list of the classes in a hashtable
        // which we can return.
        Hashtable loadTable = new Hashtable();

        if (sections.contains(Section_PreLoad)) {

            MQeFields section = sections.getFields(Section_PreLoad);
            Enumeration keys = section.fields();

            // While there are still keywords (lines) in this section left to process...
            while (keys.hasMoreElements()) {
                try {
                    // ...put the key/value of the current line into the hashtable.
                    String key = section.getAscii((String) keys.nextElement()).trim();

                    loadTable.put(key, MQe.loadObject(key));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        sectionProcessed(Section_PreLoad);
        return (loadTable);
    }
    
    /**
     * Process the [SystemProperties] section of the configuration data.
     * <p>Extract each property in turn, and put them into the Java system property.
     * Values already in the java system properties, like those specified by the -Dname=value
     * mechanism on the command-line are over-written with the values within this section.
     */
    public static void processSystemProperties(MQeFields sections) throws Exception {
        if (sections.contains(SECTION_NAME_SYSTEM_PROPERTIES)) {
            MQeFields section = sections.getFields(SECTION_NAME_SYSTEM_PROPERTIES);
            Enumeration keys = section.fields();

            Properties props = System.getProperties();
            boolean somethingChanged = false;

            // While there are still keywords (lines) in this section left to process...
            while (keys.hasMoreElements()) {
                try {
                    // ...put the key/value of the current line into the hashtable.
                    String propertyName = (String) keys.nextElement();
                    String propertyValue = (String) section.getAscii(propertyName);

                    props.put(propertyName, propertyValue);
                    somethingChanged = true;
                    // System.out.println("Setting system property '"+propertyName+"' to value '"+propertyValue+"'\n");
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (somethingChanged) {
                System.setProperties(props);
            }
        }
    }

    /**
     * <p>Process the QueueManager section in the configuration data.
     *
     * <pre>
     *  [QueueManager]   parameters for the Queue Manager
     *      (ascii)Name=.....            Qmanager name
     *      (ascii)Path=.....            Qmanager ini file
     * </pre>
     * <p> Processing the Queue Manager also processes the
     * Registry, AppRunList and CloseAppRunList sections.
     *
     * @param sections MQeFields object
     * @param ght HashTable.
     * @return MQeQueueManager object.
     * @exception java.lang.Exception
     */
    public static MQeQueueManager processQueueManager(MQeFields sections, Hashtable ght)
        throws Exception {
        return processQueueManager(sections);
    }

    /**
     * Save the configuration (ini) data to disk (without comments).
     *
     * @param pathname String containing  the filename to save as
     * @param fields MQefields object to write.
     * @return void
     * @exception java.lang.Exception
     */
    public static void saveConfigFile(String pathname, MQeFields fields) throws Exception {
        // Work with a String buffer.
        StringBuffer result = new StringBuffer();

        // For each section in the MQeFields object...
        Enumeration sections = fields.fields();

        while (sections.hasMoreElements()) {

            // Write the section name
            String fieldName = (String) sections.nextElement();

            result.append("[");
            result.append(fieldName);
            result.append("]\r\n");

            // Write each field within the section
            result.append(fields.getFields(fieldName).dumpToString("(#0)#1=#2\r\n"));
            result.append("\r\n");
        }

        // Dump the contents of the buffer out to disk.
        FileOutputStream outputFile = new FileOutputStream(new File(pathname));

        outputFile.write(MQe.asciiToByte(result.toString()));
        outputFile.close();
    }

    /**
     * Turn trace off.
     *
     * @return void
     */
    public static void traceOff() {
        MQeTrace.setHandler(null);
    }

    /**
     * Start trace with default title.
     * @return void
     */
    public static void traceOn() {
        // Activate a new trace handler if one does not already exist.
        if (null == MQeTrace.getHandler()) {
            try {
                MQeTraceHandler tracer = (MQeTraceHandler) new com.ibm.mqe.trace.MQeTraceToReadable();

                MQeTrace.setHandler(tracer);
            } catch (Exception e) {
                System.out.println("Trace failed to start: " + e.toString());
                e.printStackTrace();
            }
        }
    }

    /**
     * Called when the initialisation routine has
     * successfully processed a section.
     * This method removes the section entry from
     * the sectionsnotProcessed vector.
     *
     * @param sectionName A String containing the section name
     * @return void
     */
    public static void sectionProcessed(String sectionName) {
        sectionsNotProcessed.removeElement(sectionName);
    }

    /**
     * <p>Process the section of the configuration data that relate to the
     * listeners.  This code uses the old format for the ini file, but
     * uses the new listener management code.
     * <pre>
     * This method replaces MQeQueueManagerUtils.processListener() and
     * MQeQueueManagerUtils.processChannelManager() both of which use
     * code that is now deprecated.
     * This method should be called after the queue manager has been
     * created and activated.  The listener(s) created using the new
     * API will be stopped and cleaned up when the queue manager is closed.
     *
     * The listener only needs to be created once. Once created, the queue
     * manager will automatically start the listener when it starts.
     *
     * Two sections from the ini file are processed:
     * [ChannelManager]     Parameters for the channel manager
     *      (int)MaxChannels=.....       max number of concurrent channels
     *
     * [Listener]
     *      (ascii)Listen=.....          Listener file descriptor
     *      (ascii)Network=.....         Read/Write file descriptor
     * </pre>
     * @param sections An MQeFields object representing the data from an ini file
     * @param queueManager An MQeQueueManager object
     * @return void
     * @exception java.lang.Exception
     */
    public static void processListenerSections(MQeFields sections, MQeQueueManager queueManager)
        throws Exception {

        if (sections.contains(Section_Listener)) {
            MQeFields section = sections.getFields(Section_Listener);

            // Check if the named listener exists.
            //  Note: If it already exists then it will be started automatically
            //  when the queue manager starts.
            boolean alreadyExists = true;

            try {
                // Use the listener inquire example to see if the listener exists.
                ListenerInquire.inquireListener(LISTENER_NAME, queueManager.getName(), queueManager);
            } catch (Exception e) {
                // An exception will be thrown if the named listener is
                // not found .
                alreadyExists = false;
            }

            // If the listener does not already exist then create it.
            //
            // The first parameter ("listen") is the adapter string used to do
            // the listening (e.g. Network::7999, which says to use the adapter
            // specified by the "Network" alias, and use a port parameter of 7999.
            // The second parameter is the adapter name used when spawning a new
            // adapter on the ACCEPT of a new incoming connection.
            // The third parameter says which channel manager to use.
            //
            // As of MQe Version 2.0 the listening adapter and the
            // spawning adapter must be the same. Hence when creating the new
            // listener, only the listening adapter parameter is used.
            if (alreadyExists) {
                System.out.println("Listener (" + LISTENER_NAME + ") already exists, no need to create a new one");
            } else {
                System.out.println("Listener (" + LISTENER_NAME + ") does not exist, create one and start it.");
                // Extract the listening adapter and port from the Listen parameter.
                //  Note: the conversion presumes the TCP/IP based protocol format of
                //  adapterName::port.  This may not be appropriate for non-tcpip protocols.
                String listen = section.getAscii("Listen").trim();
                String adapter = listen.substring(0, listen.indexOf(':')).trim();
                String portStr = listen.substring(listen.lastIndexOf(':') + 1).trim();
                int    port = Integer.parseInt(portStr);

                // If there is a channel timeout specified then use it, if
                // not use the default value.
                long channelTimeout = MQeCommunicationsListenerAdminMsg.LISTENER_DEFAULT_CHANNEL_TIMEOUT;

                if (section.contains("TimeInterval")) {
                    // Timeout in ini file is in seconds so convert to miliseconds
                    // as required by the listener.
                    channelTimeout = section.getInt("TimeInterval");
                    channelTimeout = channelTimeout * 1000;
                }

                // Listener section successfully processed.
                sectionProcessed(Section_Listener);

                // If a ChannelManager section exists, get the maximum number
                // of channels allowed. If it does not exist use the default value.
                int maxChannels = MQeCommunicationsListenerAdminMsg.LISTENER_DEFAULT_MAX_CHANNELS;

                if (sections.contains(Section_ChannelManager)) {
                    section = sections.getFields(Section_ChannelManager);
                    if (section.contains("MaxChannels")) {
                        maxChannels = section.getInt("MaxChannels");
                    }

                    sectionProcessed(Section_ChannelManager);
                }

                // Now create the listener using the listener create example.
                ListenerCreate.createListener(LISTENER_NAME, adapter, port, channelTimeout, maxChannels, queueManager.getName(), queueManager);

                // and then start it using the listener start example.
                ListenerStart.startListener(LISTENER_NAME, queueManager.getName(), queueManager);
            }
        }
    }

    /**
     * <p>Process the QueueManager section in the configuration data.
     *
     * <pre>
     *  [QueueManager]   parameters for the Queue Manager
     *      (ascii)Name=.....            Qmanager name
     *      (ascii)Path=.....            Qmanager ini file
     * </pre>
     * <p> Processing the Queue Manager also processes the
     * Registry, AppRunList and CloseAppRunList sections.
     * @param sections An MQeFields object
     * @return void
     * @exception java.lang.Exception
     */
    public static MQeQueueManager processQueueManager(MQeFields sections) throws Exception {
        MQeQueueManager queueManager = null;

        if (sections.contains(Section_QueueManager)) {
            queueManager = new MQeQueueManager();
            if (queueManager != null) {
                queueManager.activate(sections);

                // Processing was successful.
                sectionProcessed(Section_QueueManager);
                sectionProcessed(MQeQueueManager.Registry);
                sectionProcessed(MQeQueueManager.AppRunList);
                sectionProcessed(MQeQueueManager.CloseAppRunList);

            }
        }
        // Return the allocated queue manager
        return queueManager;
    }
}
