package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQeResourceMonitor.java, MQeExamples, la200, la200-L030408 1.1
/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 1999                                                *
 *-----------------------------------------------------------------------------*/
import java.io.*;
import com.ibm.mqe.*;
import java.lang.*;


/**
 * A monitor which periodically outputs statistics snap-shots to the specified
 * readable data stream.
 * 
 */
public class MQeResourceMonitor implements java.lang.Runnable {
	
    public MQeResourceMonitor() {}					
	
    private volatile boolean started = false;
	
    /**
     * Indicates whether the monitor is running or not.
     */
    public boolean isRunning() {
        return this.started;
    }
	
    /**
     * The number of milliseconds between successive readings being taken.
     */
    private volatile long milliSecsBetweenReadings = 10000;
	
    public void setMilliSecsBetweenReadings(int milliSecsBetweenReadings) {
        this.milliSecsBetweenReadings = milliSecsBetweenReadings;
    }	
	
    /**
     * Which version of MQe was this example code taken from ?                                       	
     */
    public static short[] version = { 2, 0, 0, 5 };     
    
    /** 
     * interval between which we check whether the monitor is closing down.
     */
    private static final long MILLISECS_BETWEEN_CLOSE_CHECKS = 1000;
    
    /**
     * Max wait of ten seconds to kill off the background thread when 
     * the queue manager is closing down.
     */
    private static final long MAX_WAIT_FOR_BACKGROUND_THREAD_MILLISECONDS = 10000;
    
    private boolean formatCommaSeparatedFields = false;
    private boolean formatReadableMemoryData = false;
    private boolean formatReadableThreadData = false;
    private boolean formatToTrace = false;
    
    /**
     * Reference to the control block used to communicate with the background thread
     * which does a sleep-monitor-sleep-monitor loop.
     * Note that freeing such blocks for garbage collection will not stop the thread
     * to which it refers.
     */
    private Thread th = null;
    
    /** 
     * Flag which is set when shutdown of the background thread is required.
     * Volatile because the thread using the flag and the thread setting it to true
     * are different threads, and it is important that the flag is not held in 
     * CPU registers, or one thread will see a different value to the other.
     */
    private volatile boolean toldToStop = false;
    
    /**
     * Where the monitor output goes to.
     */
    private volatile PrintStream outStream = null;	
    private volatile OutputStream rawOutStream = null;
	
    public void setOutputStream(OutputStream outStream) {
        this.rawOutStream = outStream;
        if (outStream == null) {
            this.outStream = null;
        } else {
            this.outStream = new PrintStream(outStream);
        }
    }
	
    /**
     * The trace point this monitor program uses. 
     * The trace data itself is logged against USER_DEFINED_1 trace group id.
     */
    private static final short MONITOR_TRACE_POINT = 5555;
	
    /**
     * Called when the monitoring is supposed to start.
     * @return void
     * @exception java.lang.Exception
     */
    public void start() {		
        if (!started) {
            // Not started already, so start a background thread.
			
            // background thread which triggers transmission                        
            th = new Thread(this, this.toString());

            // Make sure the thread doesn't stop the moment we tell it to start.
            toldToStop = false;
		     
            // start timer thread  
            th.start();                                                
	        
            // Indicate that we've started already.
            started = true;
        }
    }

    /**
     * Called when monitoring should be stopped.
     * @return void
     * @exception java.lang.Exception
     */
    public void stop() {

        if (!started) {// Not started, so there should be nothing to stop.
        } else {
        
            // Tell the background thread to stop, as the queue manager is closing now.
            toldToStop = true;
	        
            // Now wait for the background thread, if it's not already stopped.
            if (th != null) {
                try { 
                    // Only wait for a certain time before giving up and timing out.
                    th.join(MAX_WAIT_FOR_BACKGROUND_THREAD_MILLISECONDS);
	     
                    // Free up the thread control block for garbage collection.          	
                    th = null;
                } catch (InterruptedException e) {// Don't propogate the exception. 
                    // Assume that the thread will stop shortly anyway.
                }
            }   
            started = false;
	        
            // Clear the output stream if it supports it.
            clear();
        }               
    }

    /**
     * Background thread run method.                                            
     * Triggers transmission every interval until thread is stopped.           
     */
    public void run() {
		
        // Do a sleep-trigger-sleep-trigger loop until the queue manager closes or we get
        // an exception.
        while (!toldToStop) {
			
            try {           		
        		
                if (!toldToStop) {
                    // do the monitoring snap-shot
                    doMonitorWork();
                }        			
        		
                // Count down until we've waited enough
                // We do a tight loop with a smaller granularity because 
                // otherwise we would stop a queue manager from closing quickly
                long timeToWait = milliSecsBetweenReadings;

                while (timeToWait > 0 && !toldToStop) {
        		
                    // sleep for specified interval                                 
                    Thread.sleep(MILLISECS_BETWEEN_CLOSE_CHECKS);
            		
                    // We've waited for some time. Account for this in the overall wait.
                    timeToWait -= MILLISECS_BETWEEN_CLOSE_CHECKS;
                }            
            	
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * The monitor work needs doing now.
     */
    private synchronized void doMonitorWork() {
		
        // Get a reference to the java runtime.
        Runtime runTime = Runtime.getRuntime();
        long freeBytes = runTime.freeMemory();
        long totalBytes = runTime.totalMemory();
        long usedBytes = totalBytes - freeBytes;
        long time = System.currentTimeMillis();
		
        ThreadGroup group = Thread.currentThread().getThreadGroup();
        String threadGroupName = group.getName();
        int activeThreadsInGroup = group.activeCount();
        Thread[] threadList = new Thread[activeThreadsInGroup];

        group.enumerate(threadList);

        clear();
		
        if (formatCommaSeparatedFields) {
            if (outStream != null) {
                outStream.println(time + "," + usedBytes + "," + freeBytes + "," + totalBytes + "," + activeThreadsInGroup + ", @RESOUCE_REPORT@");
            }
        }
		
        if (formatReadableMemoryData) {
            if (outStream != null) {
                long left = freeBytes;
                long megaBytes = left / 1000000;

                left -= megaBytes * 1000000;
                long kBytes = left / 1000;

                left -= kBytes * 1000;		
                outStream.println(time + " Memory free inside JVM : " + megaBytes + "MB + " + kBytes + "k + " + left + " bytes. Total bytes: " + freeBytes);						
				
                left = totalBytes;
                megaBytes = left / 1000000;
                left -= megaBytes * 1000000;
                kBytes = left / 1000;
                left -= kBytes * 1000;
                outStream.println(time + " Memory allocated by JVM: " + megaBytes + "MB + " + kBytes + "k + " + left + " bytes. Total bytes: " + totalBytes);
				
                left = usedBytes;
                megaBytes = left / 1000000;
                left -= megaBytes * 1000000;
                kBytes = left / 1000;
                left -= kBytes * 1000;
                outStream.println(time + " Memory used inside JVM : " + megaBytes + "MB + " + kBytes + "k + " + left + " bytes. Total bytes: " + totalBytes);
				
                outStream.println(" ");
                outStream.flush();				
            }
        } 
		
        if (formatReadableThreadData) {
            if (outStream != null) {
                outStream.println(time + " Threads: " + activeThreadsInGroup + " in thread group \"" + threadGroupName + "\"");
			
                for (int threadIndex = 0; threadIndex < threadList.length; threadIndex += 1) {
                    Thread thisThread = threadList[threadIndex];

                    outStream.println(time + "    " + thisThread.getName());
                }	
                outStream.println(" ");
                outStream.flush();
            }
        }
		
        if (formatToTrace) {
            String toOutput = "Used:" + usedBytes + " Free:" + freeBytes + " Total:" + totalBytes + " Threads:" + activeThreadsInGroup + " @RESOURCE_REPORT@";

            MQeTrace.trace(this, (short) MONITOR_TRACE_POINT, MQeTrace.GROUP_USER_DEFINED_1, toOutput);
        }			
    }		
	
    /**
     * Clear the output stream if it supports it.
     */
    private void clear() {
        if (rawOutStream != null) {				
            if (rawOutStream instanceof MQeClearable) {
                MQeClearable clearableOutputStream = (MQeClearable) rawOutStream;

                clearableOutputStream.clear();
            }
        }
    }
	
    /**
     * Returns the formatCommaSeparatedFields.
     * @return boolean
     */
    public boolean isFormatCommaSeparatedFields() {
        return formatCommaSeparatedFields;
    }

    /**
     * Returns the formatReadableMemoryData.
     * @return boolean
     */
    public boolean isFormatReadableMemoryData() {
        return formatReadableMemoryData;
    }

    /**
     * Returns the formatReadableThreadData.
     * @return boolean
     */
    public boolean isFormatReadableThreadData() {
        return formatReadableThreadData;
    }

    /**
     * Returns the formatToTrace.
     * @return boolean
     */
    public boolean isFormatToTrace() {
        return formatToTrace;
    }

    /**
     * Sets the formatCommaSeparatedFields.
     * @param formatCommaSeparatedFields The formatCommaSeparatedFields to set
     */
    public void setFormatCommaSeparatedFields(boolean formatCommaSeparatedFields) {
        this.formatCommaSeparatedFields = formatCommaSeparatedFields;
    }

    /**
     * Sets the formatReadableMemoryData.
     * @param formatReadableMemoryData The formatReadableMemoryData to set
     */
    public void setFormatReadableMemoryData(boolean formatReadableMemoryData) {
        this.formatReadableMemoryData = formatReadableMemoryData;
    }

    /**
     * Sets the formatReadableThreadData.
     * @param formatReadableThreadData The formatReadableThreadData to set
     */
    public void setFormatReadableThreadData(boolean formatReadableThreadData) {
        this.formatReadableThreadData = formatReadableThreadData;
    }

    /**
     * Sets the formatToTrace.
     * @param formatToTrace The formatToTrace to set
     */
    public void setFormatToTrace(boolean formatToTrace) {
        this.formatToTrace = formatToTrace;
    }

}
