package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MQeServlet.java, MQeExamples, la200, la200-L030324.1 1.13
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeChannelManager;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.MQeTrace;


/**
 * <p>To provide a simple MQe servlet.
 *
 * <p>Components configured on a servlet at startup are :
 * <pre>
 *      ... Aliases
 *      ... ChannelManager
 *      ... ChannelListener (only TimeInterval parameter)
 *      ... Registry
 *      ... QueueManager
 *      ... Preload
 * </pre>
 * <p>This class extends HttpServlet, the servlet base class.
 * <p>------------------------------------------------------------------------------
 * <pre>
 * Example configuration data fields :
 *
 *   Fields=Aliases
 *      (ascii)EventLog=examples.eventLog.LogToDiskFile
 *      (ascii)Network=com.ibm.mqe.adapters.MQeTcpipHttpAdapter
 *      (ascii)QueueManager=com.ibm.mqe.MQeQueueManager
 *      (ascii)Trace=com.ibm.mqe.trace.MQeTrace
 *      (ascii)MsgLog=com.ibm.mqe.adapters.MQeDiskFieldsAdapter
 *
 *   Fields=ChannelManager
 *      (int)MaxChannels=0
 *
 *   Fields=Listener
 *      (int)TimeInterval=300
 *
 *   Fields=QueueManager
 *      (ascii)Name=ServerQMgr
 *
 *   Fields=Registry
 *      (ascii)RegType=FileRegistry
 *      (ascii)Path=.\ExampleQM\Registry\
 *      (ascii)PrivRegPIN=12345678
 *
 *   See ExampleMQeServer.ini for an example startup paramters file
 * </pre>
 */
public class MQeServlet extends HttpServlet {
    public static short version[] = { 2, 0, 0, 5 }; 
    public final static String Section_ChannelManager = "ChannelManager";

    // Defaults used if parameter not passed to servlet
    public String defaultStartupInifile = "./ExampleMQeServer.ini";

    protected MQe mqe = new MQe();
    protected MQeChannelManager channelManager = null; // Channel mgr object
    protected MQeQueueManager queueManager = null; // Queue manager object
    protected MQeChannelTimer channelTimer = null; // Channel timeout monitor
    protected Hashtable loadTable = null; // Preloaded objects

    /**
     * called when a request is made to stop the servlet.
     * 
     * @return void
     */
    public void destroy() {
        // Close the queue manager if it is running;
        try {
            if (queueManager != null) {
                queueManager.close();
            }
        } catch (Exception ex) {}

        queueManager = null;
        
        MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO);
      
        // Close any active trace
        MQeQueueManagerUtils.traceOff();
        mqe = null;

        // log the event, if necessary
        // log("MQe servlet stopped");

    }

    /**
     * Handle POST.
     * 
     * @param request HttpServletRequest object
     * @param response HttpServletResponse object
     * @return void
     * @exception java.io.IOException
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
        // any request to process ?
        if (request == null)
            throw new IOException("Invalid request");

        try {
            int max_length_of_data = request.getContentLength(); // data length
            byte[] httpInData = new byte[max_length_of_data]; // allocate data area

            ServletOutputStream httpOut = response.getOutputStream(); // output stream
            ServletInputStream httpIn = request.getInputStream(); // input stream

            // get the request
            read(httpIn, httpInData, max_length_of_data);

            // process the request
            byte[] httpOutData = channelManager.process(null, httpInData);

            // appears to be an error in that content-length is not being set
            // so we will set it here
            response.setContentLength(httpOutData.length);
            response.setIntHeader("content-length", httpOutData.length);

            // Pass back the response
            httpOut.write(httpOutData);
        } catch (Exception e) {
            // pass it on ...
            throw new IOException("Request failed" + e);
        }
    }

    /**
     * Return a reference to the channel manager.
     * 
     * @return MQeChannelManager the channel manager reference
     */
    public MQeChannelManager getChannelManager() {
        return channelManager;
    }

    /**
     * Return a reference to the channel timer.
     * 
     * @return MQeChannelTimer the Channel Timer
     */
    public MQeChannelTimer getChannelTimer() {
        return channelTimer;
    }

    /**
     * Return a reference to the load table.
     * 
     * @return Hashtable the table
     */
    public Hashtable getLoadTable() {
        return loadTable;
    }

    /**
     * Return a reference to the queue manager.
     * 
     * @return MQeQueueManager the queue manager
     */
    public MQeQueueManager getQueueManager() {
        return queueManager;
    }

    /**
     * Servlet Initialisation.
     * 
     * @param sc ServletConfig object
     * @return void
     * @exception ServletException
     */
    public void init(ServletConfig sc) throws ServletException {
        // Ensure supers constructor is called.
        super.init(sc);

        // log the event, if necessary       
        // log("MQe servlet starting");
        try {
            // Get the the server startup ini file
            String startupIni;

            if ((startupIni = getInitParameter("Startup")) == null)
                startupIni = defaultStartupInifile;

            // Load it
            MQeFields sections = MQeQueueManagerUtils.loadConfigFile(startupIni);

            // log the event, if necessary 
            // log("MQe servlet parameters:\r\n" + sections.dumpToString("#1\t=\t#2\r\n"));

            // assign any class aliases
            MQeQueueManagerUtils.processAlias(sections);

            // Uncomment the following line to start trace before the queue manager is started
            //      MQeQueueManagerUtils.traceOn("MQeServlet Trace", null);

            // Start channel manager
            channelManager = processChannelManager(sections);

            // check for any pre-loaded classes
            loadTable = MQeQueueManagerUtils.processPreLoad(sections);

            // setup and activate the queue manager
            queueManager =
                    MQeQueueManagerUtils.processQueueManager(sections, channelManager.getGlobalHashtable());

            // Start ChannelTimer  (convert timeout from secs to millsecs)
            int tI = sections.getFields(MQeQueueManagerUtils.Section_Listener).getInt("TimeInterval");
            long timeInterval = 1000 * tI;

            channelTimer = new MQeChannelTimer(channelManager, timeInterval);

            // Servlet initialisation complete
            // log("MQe servlet started");
            
            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO);
           
        } catch (Exception e) {
            // log("MQe servlet startup failed: " + e);
            
            MQeTrace.trace(this, (short) 0, MQeTrace.GROUP_INFO, e);
           
            throw new ServletException(e.toString());
        }

    }

    /**
     * Process the ChannelManager section of the configuration data.
     *
     * <pre>
     *  [ChannelManager]     Parameters for the channel manager
     *      (int)MaxChannels=.....       max number of concurrent channels
     * </pre>
     * @param sections MqeFields object
     * @return MQeChannelManager object reference.
     * @exception java.lang.Exception
     */
    public static MQeChannelManager processChannelManager(MQeFields sections) throws Exception {
        MQeChannelManager channelManager = null;

        // If there is a ChannelManager section attempt to
        // create a new channel manager, and set the maximum number
        // of channels to the value specified in the ini file.
        if (sections.contains(Section_ChannelManager)) {
            MQeFields section = sections.getFields(Section_ChannelManager);

            channelManager = new MQeChannelManager();
            channelManager.numberOfChannels(section.getInt("MaxChannels"));
            //sectionProcessed(Section_ChannelManager);
        }
        return (channelManager);
    }

    /**
     * Read a number of bytes from an input stream.
     * 
     * @param in InputStream, the input stream
     * @param data  byte array to fill
     * @param bytes An integer, number of bytes to read.
     * @return void
     * @exception java.io.IOException The exception description.
     */
    public void read(InputStream in, byte[] data, int bytes) throws IOException {
        // Bytes read so far
        int i = 0;

        // Carry on getting bytes until we have the required number
        while (i < bytes) {
            // Bytes read on this read
            int numBytes = 0;

            // read next batch of bytes
            numBytes = in.read(data, i, bytes - i);

            // Check for possible end of file
            if (numBytes < 0) {
                throw new EOFException();
            }
            // Increment bytes read count
            i = i + numBytes;
        }
    }
}
