package examples.queuemanager;


// Version mqe_java/source/examples/queuemanager/MessageWaiter.java, MQe, la200, la200-L030324.1 1.5
/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5765-E63                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 1999                                                 */

/* -----------------------------------------------------------------------------*/

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;
import com.ibm.mqe.MQeQueueManager;


/**
 * This class illustrates how to wait for a message to appear
 * on a particular queue.  It is intended as an example that
 * can be modified to suit an individual developers requirements,
 * rather than a reusable code snippet.
 *
 * <p>Note that although this method will wait for a message on a
 * local queue residing upon a remote queue manager, it will
 * do so by polling across the network.  It will also require
 * that the correct connection definitions are set up, and that
 * the remote queue manager is responding.
 */
public class MessageWaiter {
    public static short version[] = { 2, 0, 0, 5 }; 
    MQeQueueManager queueManager;
    String queueQueueManagerName;
    String queueName;

    /**
     * Create a message waiter on a specified queue.
     * Caching these values reduces the number of parameters passed to waitForMessage
     *
     * @param queueManager The queue manager instance used as an access point to the MQe system.
     * @param queueQueueManagerName The name of the queue manager upon which the local queue resides.
     * @param queueName The name of the queue manager upon which the desired message resides.
     */
    public MessageWaiter(
        MQeQueueManager queueManager,
        String queueQueueManagerName,
        String queueName) {
        super();
        this.queueManager = queueManager;
        this.queueQueueManagerName = queueQueueManagerName;
        this.queueName = queueName;
    }

    /**
     * Get a message from the receivers queue.  The message returned will have been
     * remove from the queue.  If the message is not on the queue return null (instead
     * of throwing an exception) this makes control more simple at the next level;
     * All other exceptions thrown by the MQeQueueManager.getMessage() call are propagated
     * up the stack to the caller.
     * 
     * <p>To ignore a particular exception and keep retrying, catch the exception in this
     * method and do not propagate it up the stack.  The original implementation of
     * MQeQueueManager.waitForMessage() can be duplicated by catching ALL exceptions
     * and propagating none.
     *
     * @param filter (com.ibm.mqe.MQeFields) a matching criteria for the message.  May be null;
     * @return MQeMsgObject object.
     * @exception java.lang.Exception
     */
    public final MQeMsgObject getMessage(MQeFields filter) throws Exception {
        MQeMsgObject msg = null;

        try {
            msg = queueManager.getMessage(queueQueueManagerName, queueName, filter, null, 0L);
        } catch (MQeException e) {
            if (e.code() != MQe.Except_Q_NoMatchingMsg) {
                // if the exception is 'no matching message' then ignore it.  This
                // will result in a null return value.
                // Rethrow all other exceptions.
                throw e;
            }
        }
        return msg;
    }

    /**
     * Wait for a message on the receivers queue.  The message returned will have been
     * remove from the queue.  If no message is found within the specified time then
     * null will be returned.
     *
     * <p>Any exception thrown while retreiving the message (except for message not found)
     * will be thrown to the caller.  This is not exactly the behaviour exhibited by the
     * deprecated MQeQueueManager.waitForMessage(), which would continue trying
     * until the time elapsed or a message was retreived.  The original behaviour can be
     * recreated by masking ALL exceptions in the MessageWaiter.getMessage() method.
     *
     * @param filter a matching criteria for the message.  May be null
     * @param totalWaitTime the maximum amount of time to wait for a message.
     * @param granularity the pause between attempts to retreive the message from the queue.
     * @return MQeMsgObject the message we were waiting for or null it none received.
     * @exception java.lang.Exception
     */
    public final MQeMsgObject waitForMessage(
        MQeFields filter,
        long totalWaitTime,
        long waitGranularity)
        throws Exception {
        // try to get a message straight away
        MQeMsgObject msg = getMessage(filter);

        while (null == msg && 0 < totalWaitTime) {
            // we did not get a message, so start polling at the specified granularity
            if (waitGranularity < totalWaitTime) {
                // the remaining time is greater than one grain, so wait for one grain
                // before getting message
                // Remove the elapsed time from from the totalWait time until it reaches zero
                totalWaitTime -= waitGranularity;
            } else {
                // at the end of the poll interval we may have some odd milliseconds over
                // which are less than the granularity, so we will wait for these to elapse
                waitGranularity = totalWaitTime;
                totalWaitTime = 0;
            }
            Thread.sleep(waitGranularity);
            msg = getMessage(filter);
        }
        return msg;
    }
}
