// Version mqe_java/source/examples/security/mAttrMessage.java, MQeSecurity, la200, la200-L030324.1 1.2

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/* -----------------------------------------------------------------------------*/

package examples.security;


import java.io.File;
import java.util.Date;
import com.ibm.mqe.*;
import com.ibm.mqe.attributes.*;
import examples.queuemanager.MQeClient;


/** mAttrMessage.java
 * <p>Use the MQeMAttribute to put and get a secure message.
 *
 * <p>This opens a queue manager and uses the MQeMAttribute to encrypt a
 * message and put it to a queue on the queue manager. The MQeMAttribute is
 * then used to read the message back and decrypt it.
 *
 * <p>The program accepts two command line parameters, one mandatory and one
 * optional.
 *
 * <p>The first parameter is a configuration file for the queue manager. This
 * is used to start the queue manager as a client.
 *
 * <p>The second parameter is the name of the target queue. If this is not
 * specified, the queue SYSTEM.DEFAULT.LOCAL.QUEUE is used.
 **/

public class mAttrMessage {
    public static short  version[] = { 2, 0, 0, 5 };  // MQe version

    // local variables for the queue manager and queue
    MQeQueueManager MyQM;
    MQeClient       client;
    String          qMgrName;
    String          queueName = "SYSTEM.DEFAULT.LOCAL.QUEUE";

    // the message data
    String          msgField = "MsgData";
    String          msgData = ", some interesting data";

    // the key seed for the cryptor
    String          key = "my secret key";

    /**
     * open the client queue manager.
     *
     * @param configFile the configuration (.ini) file for a queue manager
     * @param  qName     the name of the target queue. This can be null.
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void openQM(String configFile, String qName) throws Exception {
        // check whether a the target queue name was specified
        if (qName != null) {
            queueName = qName;
        }

        // start the client queue manager and gets its name
        System.out.println("mAttrMessage, start a queue manager using config file: " + configFile);
        client = new examples.queuemanager.MQeClient(configFile);
        MyQM = client.queueManager;
        qMgrName = MyQM.getName();
        System.out.println("started queue manager: " + qMgrName);
    }

    /**
     * protect a message using the MQeMAttribute and put it to the target queue
     *
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void putMessage() throws Exception {
        // create the cryptor, we'll use triple-DES
        MQeCryptor cryptor = new MQe3DESCryptor();

        // create the attribute using the cryptor
        MQeMAttribute attr = new MQeMAttribute(null, cryptor, null);

        // create a local key object
        MQeKey localkey = new MQeKey();

        // give it the key seed
        localkey.setLocalKey(key);

        // set the key in the attribute
        attr.setKey(localkey);

        // create the message
        MQeMsgObject msgObj = new MQeMsgObject();

        msgObj.putAscii(msgField, "mAttrMessage, " + new Date().toString() + msgData);

        // put the message using the attribute
        System.out.println("putMessage to queue '" + queueName + "' using MQeMAttribute");
        MyQM.putMessage(qMgrName, queueName, msgObj, attr, 0);
        System.out.println("...OK");
    }

    /**
     * get a message from the target queue, using the MQeMAttribute to
     * unprotect it
     *
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void getMessage() throws Exception {
        // create the attribute - we do not have to specify the cryptor
        // because the attribute can get this from the message itself
        MQeMAttribute attr = new MQeMAttribute(null, null, null);

        // create a local key object
        MQeKey localkey = new MQeKey();

        // give it the key seed
        localkey.setLocalKey(key);

        // set the key in the attribute
        attr.setKey(localkey);

        // get the message using the attribute
        System.out.println("getMessage from queue '" + queueName + "' using MQeMAttribute");
        MQeMsgObject msgObj = MyQM.getMessage(qMgrName, queueName, null, attr, 0);

        System.out.println("...OK");
        System.out.println("message data = " + msgObj.getAscii(msgField));
    }

    /**
     * close the client queue manager
     *
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void closeQM() throws Exception {
        client.close();
    }

    /**
     * main method.
     * The program takes one mandatory and one optional command line argument.
     *
     * The first argument is a configuration file for a queue manager.
     *
     * The second argument is optional and is the name of the target queue. If
     * this is not specified, the queue SYSTEM.DEFAULT.LOCAL.QUEUE is used.
     */
    public static void main(String args[]) throws Exception {
        // check the command line arguments
        if (args.length == 0)
            System.err.println("usage: mAttrMessage configFile [<qName>]");
        else {
            try {
                // is a target queue specified
                String qName = (args.length > 1) ? args[1] : null;
                // open the queue manager
                mAttrMessage mAttr = new mAttrMessage();

                mAttr.openQM(args[0], qName);

                // put a message to the target queue
                mAttr.putMessage();

                // read the message back
                mAttr.getMessage();

                // close the queue manager
                mAttr.closeQM();
            } catch (Exception ex) {
                System.out.println("mAttrMessage failed: " + ex);
                ex.printStackTrace();
            }
        }
    }
}

