// Version mqe_java/source/examples/security/mTrustAttrMessage.java, MQeSecurity, la200, la200-L030324.1 1.3

/* -----------------------------------------------------------------------------*/

/*                    WebSphere MQ Everyplace for Multiplatforms                */

/*                                                                              */

/* Licensed Materials - Property of IBM                                         */

/*                                                                              */

/* 5724-C77                                                                     */

/*                                                                              */

/* (C) Copyright IBM Corp. 2002                                                 */

/* -----------------------------------------------------------------------------*/
package examples.security;


import java.io.File;
import java.util.Date;
import com.ibm.mqe.*;
import com.ibm.mqe.attributes.*;
import com.ibm.mqe.registry.*;
import examples.queuemanager.MQeClient;


/** mTrustAttrMessage.java
 * <p>Use the MQeMTrustAttribute to put and get a secure message.
 *
 * <p>This program opens a queue manager. It demonstrates how to
 * encrypt a message on behalf of the sender and put it to a queue on the
 * queue manager.  It also demonstrates how to get the message from
 * the queue and decrypt it on behalf of the receiver.  The sender and receiver
 * must both own a private registry.
 * 
 * <p>In order to simplify the example and make it stand alone,
 * the sander and the receiver are asumed to be the same one at the
 * main() method level and the program plays different roles when doing
 * setup, put and get.  The lower level code, however, does not asume
 * that the sander and the receiver are the same.
 *
 * <p>The program requires four command line parameters:
 * <ul>
 * <li>a configuration file for the queue manager. This is used to start the
 * queue manager as a client.
 * <li>the name of the protected entity.  This entity is assumed to be both the sender and the receiver.
 * The entity's private registry is assumed to be
 * at the location ".\\MQeNode_PrivateRegistry".
 * <li>the PIN for the entity's private registry.
 * <li>the key-ring password for the entity's private registry.
 * </ul>
 *
 * <pThere must be a public registry in the current directory; it must be
 * called "MQeNode_PublicRegistry" and it must contain both the sender's and
 * the receiver's public certificates.
 **/
public class mTrustAttrMessage {

    public static short[] version = { 2, 0, 0, 5 };  // MQe version

    // local variables for the queue manager and queue
    MQeQueueManager MyQM;
    MQeClient       client;
    String          qMgrName;
    String          queueName = "SYSTEM.DEFAULT.LOCAL.QUEUE";
    final String    privateReg = ".\\MQeNode_PrivateRegistry";
    final String    publicReg = "MQeNode_PublicRegistry";

    /**
     * The IP address and the port the sender's QMgr is listening on.
     * Not needed for this particular program as the sender is the receiver.
     * Otherwise, this address will be used to get sender's
     * public key during getMessage.
     */
    static String senderIPAddr = "127.0.0.1:8085";

    /**
     * The IP address and the port the receiver's QMgr is listening on.
     * Not needed for this particular program as the receiver is the sender.
     * Otherwise, htis address will be used to get receiver's
     * public key during putMessage.
     */
    static String receiverIPAddr = "127.0.0.1:8085";

    // Adapter used by the receiver.
    static String adapter = "FastNetwork";

    // the message data
    String msgField = "MsgData";
    String msgData = ", some interesting data";

    /**
     * open the client queue manager.
     *
     * @param configFile the configuration (.ini) file for a queue manager
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void openQM(String configFile)
        throws Exception {

        // start the client queue manager and gets its name
        System.out.println(
            "mTrustAttrMessage, start a queue manager using config file: " + configFile);
        client = new examples.queuemanager.MQeClient(configFile);
        MyQM = client.queueManager;
        qMgrName = MyQM.getName();
        System.out.println("started queue manager: " + qMgrName);
    }

    /**
     * Put sender's and receiver's public keys into their public registries so they can be
     * obtained by the sender.  As the sender and receiver are the same one, only
     * setup for receiver is performed.
     *
     * @param  receiver   the name of the receiver.
     * @param  PIN      the private registry's PIN
     * @param  krp      the private registry's krp
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void setupCert(String receiver, String PIN, String krp)
        throws Exception {

        // open the private registry
        MQePrivateRegistry receiverReg = new MQePrivateRegistry();

        receiverReg.activate(receiver, privateReg, PIN, krp, null, null);

        // open the public registry
        MQePublicRegistry pr = new MQePublicRegistry();

        pr.activate(publicReg, "." + File.separator);

        MQeFields miniCert = pr.getCertificate(receiver);
		
        if (miniCert == null) {
            // receiver's mini certificate needs to be published.
            pr.putCertificate(receiver, receiverReg.getCertificate(receiver));
        }
        receiverReg.close();
        pr.close();
    }
		
    /**
     * protect a message using the MQeMTrustAttribute
     *
     * @param  sender   the name of the sender.
     * @param  PIN      the private registry's PIN
     * @param  krp      the private registry's krp
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void putMessage(String sender, String PIN, String krp)
        throws Exception {

        // create the cryptor, we'll use triple-DES
        MQeCryptor cryptor = new MQe3DESCryptor();

        // create the attribute using the cryptor
        MQeMTrustAttribute attr = new MQeMTrustAttribute(null, cryptor, null);

        // open the private registry
        MQePrivateRegistry sendReg = new MQePrivateRegistry();

        sendReg.activate(sender, privateReg, PIN, krp, null, null);

        // set the registry in the attribute
        attr.setPrivateRegistry(sendReg);

        // open the public registry
        MQePublicRegistry pr = new MQePublicRegistry();

        pr.activate(publicReg, "." + File.separator);

        // set the registry in the attribute
        attr.setPublicRegistry(pr);

        // set the target in the attribute (assume sender is the receiver)
        attr.setTarget(sender);

        // In case the public key of the receiver is
        // not already in the public registry, set
        // the mini-certificate address, from where
        // the public key can be obtained
        attr.setHomeServer(receiverIPAddr, adapter); // create the message

        MQeMsgObject msgObj = new MQeMsgObject();

        msgObj.putAscii(msgField, "mTrustAttrMessage, " + new Date().toString() + msgData);

        // put the message using the attribute
        System.out.println("putMessage to queue '" + queueName + "' using MQeMTrustAttribute");
        MyQM.putMessage(qMgrName, queueName, msgObj, attr, 0);
        System.out.println("...OK");
        sendReg.close();
        pr.close();
    }

    /**
     * get a message from the queue, using the MQeMTrustAttribute to unprotect it
     *
     * @param  receiver the name of the receiver.
     * @param  PIN      the private registry's PIN
     * @param  krp      the private registry's krp
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void getMessage(String receiver, String PIN, String krp)
        throws Exception {

        // create the cryptor
        MQeCryptor cryptor = new MQe3DESCryptor();

        // create the attribute using the cryptor
        MQeMTrustAttribute attr = new MQeMTrustAttribute(null, cryptor, null);

        // open the private registry
        MQePrivateRegistry recvReg = new MQePrivateRegistry();

        recvReg.activate(receiver, privateReg, PIN, krp, null, null);

        // set the registry in the attribute
        attr.setPrivateRegistry(recvReg);

        // open the public registry
        MQePublicRegistry pr = new MQePublicRegistry();

        pr.activate(publicReg, "." + File.separator);

        // set the registry in the attribute
        attr.setPublicRegistry(pr);

        // In case the public key of the sender is
        // not already in the public registry, set
        // the mini-certificate address, from where
        // the public key can be obtained
        attr.setHomeServer(senderIPAddr, adapter); // create the message

        // get the message using the attribute
        System.out.println("getMessage from queue '" + queueName + "' using MQeMTrustAttribute");

        MQeMsgObject msgObj = MyQM.getMessage(qMgrName, queueName, null, attr, 0);

        System.out.println("...OK");
        System.out.println("message data = " + msgObj.getAscii(msgField));
        recvReg.close();
        pr.close();
    }

    /**
     * close the client queue manager
     *
     * @exception  java.lang.Exception  propagated from the invoked methods
     **/
    void closeQM()
        throws Exception {
        client.close();
    }

    /**
     * main method.
     * The program takes seven command line arguments.
     *
     * The first argument is a configuration file for a queue manager.
     *
     * The next three arguments define the private registry: the first
     * is the name of the registry, the second is the registry's PIN and the
     * third is the registry's key-ring password.
     *
     */
    public static void main(String[] args)
        throws Exception {

        // check the command line arguments
        if (args.length < 4) {
            System.err.println(
                "usage: mTrustAttrMessage configFile regID PIN KRP");
        } else {

            try {

                // open the queue manager
                mTrustAttrMessage mTrust = new mTrustAttrMessage();

                mTrust.openQM(args[0]);

                // get certificate
                mTrust.setupCert(args[1], args[2], args[3]);
                
                // put a message to the target queue
                mTrust.putMessage(args[1], args[2], args[3]);

                // read the message back
                mTrust.getMessage(args[1], args[2], args[3]);

                // close the queue manager
                mTrust.closeQM();
            } catch (Exception ex) {
                System.out.println("mTrustAttrMessage failed: " + ex);
                ex.printStackTrace();
            }
        }
    }
}
