// 1.1 mqe_java/source/examples/stockplugin/model/Model.java, MQeExamples, la200, la200-L030324.1

/*-----------------------------------------------------------------------------*
 *                    WebSphere MQ Everyplace for Multiplatforms               *
 *                                                                             *
 * Licensed Materials - Property of IBM                                        *
 *                                                                             *
 * 5765-E63                                                                    *
 *                                                                             *
 * (C) Copyright IBM Corp. 2002                                                *
 *-----------------------------------------------------------------------------*/

package examples.stockplugin.model;


/* Standard Java Class required */
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import com.ibm.mqe.MQe;
import com.ibm.mqe.MQeAdminMsg;
import com.ibm.mqe.MQeException;
import com.ibm.mqe.MQeField;
import com.ibm.mqe.MQeFields;
import com.ibm.mqe.MQeMsgObject;
import com.ibm.mqe.MQeQueueManager;
import com.ibm.mqe.administration.MQeStoreAndForwardQueueAdminMsg;
import examples.stockplugin.gui.InfoLog;


/** 
 * This class provides the program logic that controls the server in the Native Broker Example.
 * As such it is the central point for the basic functionality
 * regarding the simple procesing of share trades. 
 * 
 * This provides no similation functions - it accepts admin messages, and will
 * add broker's QueueManager name to the StoreAndForward Queue.  
 * 
 * It can then respond to trade messages.  As no simulation is present it lets the Broker
 * buy and sell anything.
 * 
 * For information on the architecture of the queues, please see the Native C StockBroker Documentation.
 */
public class Model {

    public static short[] version = { 2, 0, 0, 5 }; 

    /* Constants for fields labels */
    
    /** Field Label for the type value */
    public static final String TYPE_FL = "type";
	
    /** Field label for the action value */
    public static final String ACTION_FL = "action";
	
    /** Field label for the sharename value */
    public static final String NAME_FL = "name";
	
    /** Field label for the tradeid value */
    public static final String TRADEID_FL = "tradeid";
	
    /** Field label for the price value */
    public static final String PRICE_FL = "price";
	
    /** Field label for the qty value */
    public static final String QTY_FL = "qty";
	
    /** Field label for the trade type value */
    public static final String TRADE_TYPE_FL = "tradeType";
	
    /** Field label for the result balue */
    public static final String RESULT_FL = "result";

    /* Constants for the type of action */
	
    /** Broker Admin action constant */
    public static final String ADMIN = "ADMIN";
	
    /** Broker Subcribe action constant */
    public static final String SUBSCRIBE = "SUBSCRIBE";
	
    /** Broker Trade Result action constant */
    public static final String TRADE_RESULT = "TRADE_RESULT";
	
    /** Broker Trade action constant */
    public static final String TRADE = "TRADE";

    /* Names of the queue, this server etc */
    String localQueueManagerName = "ServerQM";
    String safQueueName = "InformationQ";

    String brokerTradeName = "BrokerTradeQ";
    String brokerAdminName = "BrokerAdminQ";
    String brokerIncomingName = "IncomingQ";

    /** State values for each broker */
    public static final int STATE_UNSUBSCRIBED = 0;
    public static final int STATE_TRADING = 1;

    /**
     * Details of the brokers that are connected
     * Have a hastable to that information about each one can be held, 
     * indexed by their String name
     */
    Hashtable connectedBrokers;

    /**
     * Class acting as structure to hold information about
     * each broker.
     * 
     * State is also held.
     */
    class BrokerInfo {
        String name;
        int brokerState;
    }

    /** QueueManager
     */
    MQeQueueManager queueManager;

    /** InfoLog - writes output to the log window
     */
    private InfoLog adminLog, tradeLog;

    /** Constructor for the Model
     * 
     * Takes instance of the admind and trade logs
     * 
     * @param adminInfoLog  Interface to admin log - whatever that represented as
     * @param tradeInfoLog  Interface to trade log - whatever that represented as
     */
    public Model(InfoLog adminInfoLog, InfoLog tradeInfoLog) {

        // first create the hashtable
        connectedBrokers = new Hashtable();
        this.adminLog = adminInfoLog;
        this.tradeLog = tradeInfoLog;

        // Then get the Reference to the local queuemanaer
        try {
            queueManager = MQeQueueManager.getDefaultQueueManager();
        } catch (Exception e) {
            _error(adminLog, "Model new", e);
        }

        _information(adminLog, "Model created " + queueManager);
    }

    /** Threads for running the admin and the trading
     */
    Thread adminThread, tradeThread;

    /** Kicks of the admin processing
     */
    public void goAdminThread() {
        adminThread = new DoProcessing("admin");
        adminThread.start();
    }

    /** Stops the admin thread
     */
    public void stopAdminThread() {
        ((DoProcessing) adminThread).running = false;
    }

    /** Kicks of the trade processing thread
     */
    public void goTradeThread() {
        tradeThread = new DoProcessing("trade");
        tradeThread.start();
    }

    /** Stops the trade processing thread
     */
    public void stopTradeThread() {
        ((DoProcessing) tradeThread).running = false;
    }

    /** General Processing Thread
     * 
     * Thread that controls the processing of admin and trade requests
     */
    class DoProcessing extends Thread {

        /* Are we running ? */
        public boolean running = true;

        /* What this thread is controlling ? */
        String type;

        /** Constructore for thread
         * 
         * @param type Admin or Trade processing
         */
        public DoProcessing(String type) {
            this.type = type;
        }

        /** Run Method 
         */
        public void run() {

            boolean success;

            while (running) {

                if (type.equals("admin")) {
                    _information(adminLog, "-------------------------------\n");
                    success = processAdminRequests();
                } else if (type.equals("trade")) {
                    _information(tradeLog, "-------------------------------\n");
                    success = processTradeRequests();
                }

                // to be honest at the moment don't really care what happened
                success = true;

                try {
                    Thread.sleep(2000);
                } catch (Exception e) {
                    running = false;
                }

            }

        }
    }

    /** Process all the requests on the BrokerAdminQ 
     */
    public boolean processAdminRequests() {
        String type = null;
        String name = null;
        boolean success = true;

        _information(adminLog, "Waiting for admin requests\n");
        try {
            // wait for messages on the admin queue	
            _information(
                adminLog,
                "Checking on "
                + localQueueManagerName
                + " Q"
                + brokerAdminName);
            MQeMsgObject msg =
                queueManager.waitForMessage(localQueueManagerName,
                    // qm name
                    brokerAdminName, // queue name
                    null, // filter
                    null, // attribute
                    0, // confirmid
                    3600000); // time to wait

            _information(adminLog, "Msg is " + msg.toString());
            if (msg != null) {

                /* Get the type of the Message */
                type = msg.getAscii(Model.TYPE_FL);
                _information(adminLog, "Msg type is " + type + "\n");

                if (success && type.equals(Model.ADMIN)) {

                    /* Admin message */
                    name = msg.getAscii(Model.NAME_FL);
                    _information(
                        adminLog,
                        "Got " + name + " - sending reply\n");
                    _addQMtoSAFQueue(name);

                    /* Create Reply Message - currenlty assuming all is well*/
                    MQeMsgObject replyMsg = _createAdminReply(true);

                    queueManager.putMessage(
                        name,
                        brokerIncomingName,
                        replyMsg,
                        null,
                        0);

                }

            } else {
                _information(adminLog, "Msg is null");
            }

        } catch (Exception e) {
            _error(adminLog, "Process admin", e);
            success = false;
        }

        return success;

    }

    /** Process trade requests
     * 
     * @return TRUE if all is well
     */
    public boolean processTradeRequests() {
        _information(tradeLog, "Waiting for message on trade queue");
        boolean success = true;
        MQeMsgObject msg;

        try {

            /* Wait for mesasge */
            msg =
                    queueManager.waitForMessage(
                        localQueueManagerName,
                        brokerTradeName,
                        null,
                        null,
                        0,
                        3600000);

            if (msg != null) {
                // Get the broker's name aka queuemanager
                String brokersName = getBrokersName(msg);

                _information(tradeLog, "Got msg from " + brokersName);

                // check that the broker is allowed to send these messages
                if (brokerAllowedToTrade(brokersName)) {

                    // process the resultant message
                    MQeMsgObject replyMsg = processTrade(msg);

                    if (replyMsg != null) {
                        // send reply message back to the broker
                        _information(
                            tradeLog,
                            "About to send reply back to broker " + replyMsg);
                        queueManager.putMessage(
                            brokersName,
                            brokerIncomingName,
                            replyMsg,
                            null,
                            0);
                    } else {
                        _information(tradeLog, "Failed to create reply msg");
                        success = false;
                    }
                } else {
                    _information(
                        tradeLog,
                        "Broker " + brokersName + " not allowed to trade");
                }

            }

        } catch (Exception e) {
            _error(tradeLog, "Process trade", e);
            success = false;
        }

        return success;
    }

    /**
     * Process the trade, and generate the responce.
     * 
     * Currenlty only returns succes, as the stockmarket
     * simulation has yet to be integrated.
     * 
     * @param tradeRequest Msg with the Trade requested
     * @return Msg with the responce, succes or fail as approriate.
     */
    public MQeMsgObject processTrade(MQeMsgObject tradeRequest) {

        boolean success = true;

        int price = 0, qty = 0;
        String share = null, tradeID = null, brokersName = null;
        boolean isBuy = false;

        try {

            /* Extract the various elements from the trade message */
            price = tradeRequest.getInt(Model.PRICE_FL);
            qty = tradeRequest.getInt(Model.QTY_FL);

            share = tradeRequest.getAscii(Model.NAME_FL);
            tradeID = tradeRequest.getAscii(Model.TRADEID_FL);

            isBuy = tradeRequest.getBoolean(Model.TRADE_TYPE_FL);
        } catch (Exception e) {
            _error(tradeLog, e.toString());
            e.printStackTrace();
            success = false;
        }

        if (success) {

            brokersName = getBrokersName(tradeRequest);
            // now convert this to the stockmarket object and get it to process it
            // but haven't worked out that code yet so just assume that its ok

            _information(
                tradeLog,
                "Processed Trade for "
                + share
                + " at "
                + price
                + " for "
                + qty
                + " ["
                + tradeID
                + "]");

            return createOKTradeReply(share, price, qty, isBuy, tradeID);
        } else {
            return null;
        }

    }

    /**
     * Get the brokers name aka queue manager name
     * 
     * @param tradeRequest Trade Message
     * @return Name of the Broker's Queue Manager
     */
    public String getBrokersName(MQeMsgObject tradeRequest) {
        String name = null;

        try {
            name = tradeRequest.getAscii(MQe.Msg_OriginQMgr);
        } catch (Exception e) {
            _error(tradeLog, e.toString());
            e.printStackTrace();
        }
        return name;

    }

    boolean brokerAllowedToTrade(String name) {
			
        boolean success = true; 	
			
        // need to really check the array of connected people at this point
		
        return success;
			
    }

    /** Creats a reply to an admin(subscribe) request
     */
    public MQeMsgObject _createAdminReply(boolean okFlag) {
        MQeMsgObject replyMsg = null;

        try {
            replyMsg = new MQeMsgObject();
            replyMsg.putAscii(TYPE_FL, ADMIN);
            replyMsg.putBoolean(RESULT_FL, okFlag);

            //_information(tradeLog, "Sending reply message " + replyMsg);
        } catch (Exception e) {// some weird MQeException probably
        }

        return replyMsg;

    }

    /**
     * Create a responce to a trade, confirm a successful
     * trade.
     * 
     * @param shareName Name of the share
     * @param price     Unit Price shares purchased at
     * @param qty       Number of shares purchased
     * @param isBuy     True if buy, False if sell
     * @param tradeID   UID of Trade, specified by Broker
     * @return Msg with responce to trade
     */
    public MQeMsgObject createOKTradeReply(
        String shareName,
        int price,
        int qty,
        boolean isBuy,
        String tradeID) {
        MQeMsgObject msg = null;

        try {
            msg = new MQeMsgObject();
        } catch (Exception e) {
            _error(tradeLog, e.toString());
            e.printStackTrace();
        }

        if (msg != null) {
            try {

                msg.putAscii(Model.TYPE_FL, Model.TRADE_RESULT);
                msg.putBoolean(Model.TRADE_TYPE_FL, isBuy);
                msg.putAscii(Model.TRADEID_FL, tradeID);
                msg.putInt(Model.PRICE_FL, price);
                msg.putInt(Model.QTY_FL, qty);
                msg.putAscii(Model.NAME_FL, shareName);
                msg.putBoolean(Model.RESULT_FL, true);
            } catch (Exception e) {
                _error(tradeLog, e.toString());
                e.printStackTrace();
            }

        }

        return msg;
    }

    /** Reports information events
     */
    private void _information(InfoLog infoLog, String info) {
        infoLog.addStatus(info + "\n");
        System.out.println(info);
    }

    /** Reports error events
     */
    private void _error(InfoLog infoLog, String info) {
        infoLog.addStatus(info + "\n");
        System.err.println(info);
    }

    private void _error(InfoLog infoLog, String info, Exception e) {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);

        e.printStackTrace(pw);
        System.err.println(info + ":" + sw.getBuffer());
        _error(infoLog, e.getMessage());

    }

    private void _addQMtoSAFQueue(String newQueueManagerName)	throws Exception {
        MQeStoreAndForwardQueueAdminMsg createMessage =
            new MQeStoreAndForwardQueueAdminMsg();
			
        // set the name of the resource we wish to update
        createMessage.setName(localQueueManagerName, "InformationQ");
			
        createMessage.addQueueManager(newQueueManagerName);
			
        createMessage.setTargetQMgr(localQueueManagerName);

        // indicate that we require a reply message
        createMessage.putInt(MQe.Msg_Style, MQe.Msg_Style_Request);

        // use default reply-to queue on the target queue manager.
        createMessage.putAscii(MQe.Msg_ReplyToQ, MQe.Admin_Reply_Queue_Name);
        createMessage.putAscii(MQe.Msg_ReplyToQMgr, localQueueManagerName);

        // create a unique tag that we can identify the reply with
        String match = "Msg" + System.currentTimeMillis();

        createMessage.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        System.out.println("connection administration message created");

        // now put the administration message to the default administration queue
        queueManager.putMessage(
            localQueueManagerName,
            MQe.Admin_Queue_Name,
            createMessage,
            null,
            0);

        System.out.println(
            "create administration message put to " + MQe.Admin_Queue_Name);

        // create the filter so we get the match
        MQeFields filter = new MQeFields();

        filter.putArrayOfByte(MQe.Msg_CorrelID, match.getBytes());

        // now wait for a reply
        MQeAdminMsg response =
            (MQeAdminMsg) queueManager.waitForMessage(
                localQueueManagerName,
                MQe.Admin_Reply_Queue_Name,
                filter,
                null,
                0,
                3000);

        // the administration message has a method that will get out the return code 
        switch (response.getRC()) {
        case MQeAdminMsg.RC_Success:
            System.out.println("SAFQ created");
            break;

        case MQeAdminMsg.RC_Fail:
            System.out.println(
                "SAFQ not created, failed with: "
                + response.getReason());
            break;

        case MQeAdminMsg.RC_Mixed:
            System.out.println(
                "Partial failure with: " + response.getReason());
            MQeFields errors = response.getErrorFields();
            Enumeration fields = errors.fields();

            while (fields.hasMoreElements()) {
                String value[];
                String name = (String) fields.nextElement();

                if (errors.dataType(name)
                    == MQeField.TypeArrayElements)
                    value = errors.getAsciiArray(name);
                else
                    value = new String[] { errors.getAscii(name)};
                for (int j = 0; j < value.length; j++)
                    System.out.println(
                        "Field " + name + " failed with " + value[j]);
            }
            System.out.println(
                "SAFQ connection failed with: " + response.getReason());
            break;

        }
        //if (response.getRC() != MQeAdminMsg.RC_Success) {
        //	throw new MQeException(response.getReason());				
        //}
        // IF this is already know about get s very awful message		
    }

}
